Package: VariantTools
Type: Package
Title: Tools for Exploratory Analysis of Variant Calls
Version: 1.55.0
Author: Michael Lawrence, Jeremiah Degenhardt, Robert Gentleman
Maintainer: Michael Lawrence <lawremi@gmail.com>
Description: Explore, diagnose, and compare variant calls using filters.
Depends: R (>= 3.5.0), S4Vectors (>= 0.17.33), IRanges (>= 2.13.12),
        GenomicRanges (>= 1.31.8), VariantAnnotation (>= 1.11.16),
        methods
Imports: Rsamtools (>= 1.31.2), BiocGenerics, Biostrings, parallel,
        GenomicFeatures (>= 1.31.3), Matrix, rtracklayer (>= 1.39.7),
        BiocParallel, GenomeInfoDb, BSgenome, Biobase
Suggests: RUnit, LungCancerLines (>= 0.0.6), RBGL, graph, gmapR (>=
        1.21.3), TxDb.Hsapiens.UCSC.hg19.knownGene, org.Hs.eg.db
License: Artistic-2.0
LazyLoad: yes
biocViews: Genetics, GeneticVariability, Sequencing
git_url: https://git.bioconductor.org/packages/VariantTools
git_branch: devel
git_last_commit: 48b610f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:46:49 UTC; biocbuild
Built: R 4.6.0; ; 2026-05-19 05:13:24 UTC; unix

import(methods)
import(BiocGenerics)
import(S4Vectors)
import(IRanges)
import(GenomicRanges)
import(VariantAnnotation)
import(GenomeInfoDb)
importFrom(stats, dbinom, dhyper, pbinom, pt, setNames)
importFrom(utils, packageDescription)
importFrom(parallel, mclapply, mcmapply)
importFrom(Biostrings, DNAStringSet, detail, DNA_BASES)
importFrom(Rsamtools, BamFile, ScanBamParam, bamWhich, ApplyPileupsParam,
           PileupFiles, plpWhat, "plpWhat<-", applyPileups)
importFrom(GenomicFeatures, cdsBy, exonsBy, transcriptLocs2refLocs)
importClassesFrom(Matrix, dgTMatrix)
importFrom(rtracklayer, import)
importClassesFrom(rtracklayer, BigWigFile)
importFrom(tools, file_ext)
importFrom(BiocParallel, bpvec, registered, bplapply, bpmapply, bpnworkers)
importFrom(BSgenome, getSeq)
importFrom(Biobase, sampleNames, "sampleNames<-")

export(callVariants, qaVariants, tallyVariants, postFilterVariants)
export(MinTotalDepthFilter)

export(calculateConcordanceMatrix,
       callVariantConcordance, calculateVariantConcordance)
export(VariantQAFilters, VariantCallingFilters,
       VariantPostFilters, TallyVariantsParam)
export(variantGR2Vcf)

export(callSampleSpecificVariants, SampleSpecificVariantFilters,
       annotateWithControlDepth)

export(SetdiffVariantsFilter, DepthFETFilter, MaxControlFreqFilter)

export(callWildtype, minCallableCoverage)

## some utilities
export(matchVariants, "%variant_in%", extractCoverageForPositions)
export(pileupVariants)

export(CallGenotypesParam)
exportMethods(callGenotypes)

This README covers the usage of many of the functions and scripts
found in the scripts directory.

#################################
## Running the variant calling ##
#################################

The first set of scripts covered are the basic wrappers for running
the variant calling and T/N comparison functions from the command
line.

run_callVar_wrapper.R will run the single sample variant calling using
the same parameters as the pipeline. This script may need modified
slightly due to some changes in the pipeline code. It used to depend
on RNAseqGenie however this package seems to no longer exist. The only
function needed is saveWithID and sourcing in this file will solve the
issue.

run_TN_wrapper.R is similar and suffers from a similar issue due to
the removal of RNAseqGenie.


#############################
## Mutations by transcript ##
#############################

The second set of scripts covered are the scripts used to generate the
transcript mutation matrix.

To generate the mutation matrix for a set of sample you will want to
open the file wrap_annotate_trans.R. This wrapper function loads the
needed data and scripts to generate the final files. Running this
wrapper depends on having generated variantEffectPredictor results for
the set of tumor-specific calls. You can look at the example_runs
directory to see how to generate these files. In addition the mkRuns.R
file has a bit of code for generating these run files given a set of
SAM IDs. After generating these VEP file you will want to edit dir to
point to the directory where they can be found. The first half of this
script simply data. The file then contains a simple function that
generates the intermediate object followed by a bit of code to
summarize this.


############################
## RNA vs Exome frequency ##
############################


The Next file is make_RNA_EX_gene_plot.R. This script will generate
the "RNA vs Exome" fequency plots per gene for Robert. Again the basic
set up is the same. The first half of the script loads the data while
the second half contains a function that needs to be loaded and will
then generate the summary.  there is also a simple plot call to
generate the per gene plots.

In addition to this there is a script N_NS_RNA_EX.R that contains a
single function. This function will generate a similar plot to the
above RNA/DNA comparison except that it will make the plot only for a
single sample and it will be across all genes. This will also color
the mutations by non-synonymous /synonymous status.

#################################
## Validation to array and RNA ##
#################################

The scripts array_het.R and arraySNP_v_seq.R contain methods for
calculating the error rates with respect to the SNP array data and
also for making plots of this data. Further work is still needed here
to reslove issues with the definition of hom-ref on the array. I am
still running into issues with this.

Further the scripts rnaValRate.R and rnaValWrapper.R have methods for
checking the Exome SNVs against those seen in the RNA.

######################
## mutation by gene ##
######################

getAlleleByLocus_mod.R has code to get mutation calls on a per gene
bases. For single gene questions this is much faster than loading the
variant call GRanges.

test_calculateConcordanceMatrix_fileInput <- function() {
  p53 <- gmapR:::exonsOnTP53Genome("TP53")
  bams <- LungCancerLines::LungCancerBamFiles()
  tally.param <- VariantTallyParam(gmapR::TP53Genome(), 
                                   readlen = 100L,
                                   high_base_quality = 23L,
                                   which = range(p53))
  called.variants1 <- callVariants(bams$H1993, tally.param)
  called.variants2 <- callVariants(bams$H2073, tally.param)
  
  varFile1 <- tempfile()
  varFile2 <- tempfile()
  save(called.variants1, file=varFile1)
  save(called.variants2, file=varFile2)
  on.exit(unlink(varFile1), add=TRUE)
  on.exit(unlink(varFile2), add=TRUE)
  variantFiles <- c(varFile1, varFile2)

  mat <- calculateConcordanceMatrix(variantFiles)
  expected <- matrix(c(1, 1, 1, 1), ncol = length(variantFiles))
  rownames(expected) <- colnames(expected) <- variantFiles
  checkIdentical(expected, mat)
}

test_getConcordantCliques_twoConcord <- function() {
  input <- matrix(c(1, 0.9, 0.9, 1), ncol=2)
  threshold <- 0.8 
  output <- .getConcordantCliques(input, threshold)
  expectedOutput <- list("1" = c("id1", "id2"))
  checkIdentical(expectedOutput, output)
}

test_getConcordantCliques_noConcord <- function() {
  input <- matrix(c(1, 0.1, 0.1, 1), ncol=2)
  threshold <- 0.8 
  output <- .getConcordantCliques(input, threshold)
  expectedOutput <- list("1" = "id1", "2" = "id2")
  checkIdentical(expectedOutput, output)
}

test_getConcordantCliques_twoIdenticalPairs <- function() {
  ##A is C and B is D
  input <- matrix(c(1, 0, 0.9, 0,
                    0, 1, 0, .95,
                    .9, 0, 1, 0,
                    0, .95, 0, 1),
                  ncol=4)
  threshold <- 0.85
  rownames(input) <- colnames(input) <- LETTERS[seq_len(ncol(input))]
  output <- .getConcordantCliques(input, threshold)
  expectedOutput <- list("1" = c("A", "C"), "2" = c("B", "D"))
  checkIdentical(expectedOutput, output)
}

test_idVerify_twoConcordant <- function() {
  p53 <- gmapR:::exonsOnTP53Genome("TP53")
  bams <- LungCancerLines::LungCancerBamFiles()
  tally.param <- VariantTallyParam(gmapR::TP53Genome(), 
                                   readlen = 100L,
                                   high_base_quality = 23L,
                                   which = range(p53))
  called.variants1 <- callVariants(bams$H1993, tally.param)
  called.variants2 <- callVariants(bams$H2073, tally.param)
  
  varFile1 <- tempfile()
  varFile2 <- tempfile()
  save(called.variants1, file=varFile1)
  save(called.variants2, file=varFile2)
  on.exit(unlink(varFile1), add=TRUE)
  on.exit(unlink(varFile2), add=TRUE)

  variantFiles <- c(varFile1, varFile2)
  concordReport <- idVerify(variantFiles, threshold=0.8)
  expected <- rep("concordant", 2L)
  names(expected) <- variantFiles
  checkIdentical(expected, concordReport)
}

test_checkVariantConcordance <- function() {
  p53 <- gmapR:::exonsOnTP53Genome("TP53")
  bams <- LungCancerLines::LungCancerBamFiles()
  bam <- bams$H1993
  tally.param <- VariantTallyParam(gmapR::TP53Genome(), 
                                   readlen = 100L,
                                   high_base_quality = 23L,
                                   which = range(p53))
  called.variants <- callVariants(bam, tally.param)
  concord <- checkVariantConcordance(called.variants,
                                     called.variants)
  checkTrue(1L, concord$fraction)
}

RVW <- function(sam){
  f1 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/",
              sam,"/",
              sam,
              ".sample_specific_variants_granges.RData", sep = "")
  if(file.exists(f1)){
    gr <- get(load(f1))
  }else{
    return(NULL)
  }
  
  f2 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/RNA/",
              sam,"/results/",
              sam, ".variants_granges.RData", sep = "")
  
  f3 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS115/",
              sam, "/results/",
              sam, ".coverage.RData", sep = "")
  if(file.exists(f2)){
    
    rna <- get(load(f2))
    cov <- get(load(f3))
  }else{
    return(NULL)
  }
  val <- rnaValRate(gr=gr, rna=rna, cov=cov)
  return(val)
}

x <- Lgr[["SAM587376"]]

sams <- names(kras)
sams_list <- as.list(sams)
with_cons <- lapply(sams_list, function(x){
  gr <- kras[[x]]
  if(file.exists(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/"
                       , x,"_variant_annotations.txt", sep =""))){
    tab <- read.table(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/"
                            , x,"_variant_annotations.txt", sep =""), sep = "\t")
  }else{
    return(gr)
  }
  if(length(gr)>0){
    values(gr)$cons <- NA
    pos_x <- paste(sub("chr", "", seqnames(gr)), start(gr), sep = ":")
    for(i in 1:length(pos_x)){
      cons <- unique(as.character(tab[which(tab$V2 == pos_x[i]), 7]))
      if(length(cons) ==1){
        values(gr[i])$cons<- cons
      }else{
        cons <- paste(cons, collapse=",")
        values(gr[i])$cons<- cons
      }
    }
    return(gr)
  }else{
    return(gr)
  }
})

final <- lapply(sams_list, function(sam){
  x <- with_cons[[sam]]
  if(length(x)>0){
    t <- cbind(sam, as.character(seqnames(x)),
               start(x), as.character(values(x)$ref),
               as.character(values(x)$read),values(x)$count,
               values(x)$count.total,values(x)$cons)
    return(t)
  }else{
    return(NULL)
  }
})

variantKeys <- function(x) paste0(x$location, x$alt)

matchVariants <- function(x, table) {
  .Deprecated("match,VRanges,VRanges")
  if (is.null(x$alt))
    stop("matchVariants requires an 'alt' metacolumn")
  hits <- findOverlaps(x, table)
  same.alt <- x$alt[queryHits(hits)] == table$alt[subjectHits(hits)]
  ans <- rep(NA, length(x))
  ans[queryHits(hits)[same.alt]] <- subjectHits(hits)[same.alt]
  ans
}

mergeVariantInfo <- function(x, info) {
  m <- matchVariants(x, info)
  info.cols <- setdiff(colnames(mcols(info)), colnames(mcols(x)))
  mcols(x)[info.cols] <- lapply(mcols(info)[info.cols], `[`, m)
  x
}

setGeneric("%variant_in%", function(x, table) standardGeneric("%variant_in%"))

setMethod("%variant_in%", c("GenomicRanges", "GenomicRanges"),
          function(x, table) {
            .Deprecated("%in%,VRanges,VRanges")
            !is.na(matchVariants(x, table))
          })

variant_setdiff <- function(x, y) {
  x[!x %variant_in% y]
}

## note this is asymmetric in that it keeps the metadata from 'x'
variant_intersect <- function(x, y) {
  x[x %variant_in% y]
}

byCov <- function(sam){
  som <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/",
                  sam , "/",
                  sam,
                  ".sample_specific_variants_granges.RData", sep = "")))
  som <- som[values(som)$count/values(som)$count.total >.1]
  cov <- get(load(paste("/gne/research/data/cgp/2.0/", sam, "/exome_merged/RData/",sam,".cov.RData", sep = "")))
  X <- slice(cov, lower = 8, upper =12)
  XX <- slice(cov, lower = 18, upper = 22)
  XXX <- slice(cov, lower = 28, upper = 32)
  gr10x <- as(X, "GRanges")
  gr20x <- as(XX, "GRanges")
  gr30x <- as(XXX, "GRanges")
  over <- findOverlaps(som, gr10x)
  in10 <- som[queryHits(over)]
  over <- findOverlaps(som, gr20x)
  in20 <- som[queryHits(over)]
  over <- findOverlaps(som, gr30x)
  in30 <- som[queryHits(over)]
  x <- c(length(in10)/(sum(width(gr10x))/10e5), length(in20)/(sum(width(gr20x))/10e5), length(in30)/(sum(width(gr30x))/10e5))
  barplot(x, main = sam, ylab = "Tumor-specific mutations per Mb", names = c("10X", "20X", "30X"))
}

path <- "/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS169/"
sams <- dir(path)
genome_dir = "/gne/research/workspace/degenhj2/"
genome = "hg19_IGIS21"
post = ".analyzed.bam"
sams <- as.list(sams)
calls <- mclapply(sams, mc.cores = detectCores(), function(sam){
  bam_file <- file.path(path,sam,"bams/", paste(sam, post, sep = ""))
  gr <- tally2GR(bamfiles=bam_file, genome=genome,genome_dir=genome_dir,chr_gr=gene, variant_strand=1, map_qual=13, bqual_thresh=23)
  gr <-variantFilter(gr, useQual=TRUE)
  gr <- gr$filtered_granges
  return(gr)
}
)

library(CGPtools)
library(VariantTools)
library(parallel)
library(TxDb.Hsapiens.BioMart.igis)
library(org.Hs.eg.db)
source("~/svn/Packages/VariantTools/scripts/getAlleleByLocus_mod.R")
txdb <- TxDb.Hsapiens.BioMart.igis
genes <- cdsBy(txdb, "gene")

x <- org.Hs.egSYMBOL2EG
mapped_genes <- mappedkeys(x)
xx <- as.list(x[mapped_genes])

meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/release_2.1/released.json")

data <- get(load("~/svn/Projects/CRCanalysis/trunk/data/RNAseq_64_matchedDFs_PTIDdf.rda"))
colon_sams <- as.list(as.character(data[,2]))


##APC
kp <- unlist(mget(c("APC"), org.Hs.egSYMBOL2EG))
apc <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
kp <- unlist(mget(c("WNT5A"), org.Hs.egSYMBOL2EG))
wnt5a <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("CTNNB1"), org.Hs.egSYMBOL2EG))
beta <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("FZD1"), org.Hs.egSYMBOL2EG))
fzd1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("FZD2"), org.Hs.egSYMBOL2EG))
fzd2 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("DVL1"), org.Hs.egSYMBOL2EG))
dvl1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("DVL2"), org.Hs.egSYMBOL2EG))
dvl2 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("DVL3"), org.Hs.egSYMBOL2EG))
dvl3 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
apc_freq<-freq(apc)
wnt5a_freq<-freq(wnt5a)
beta_freq<-freq(beta)
fzd1_freq<-freq(fzd1)
fzd2_freq<-freq(fzd2)
dvl1_freq<-freq(dvl1)
dvl2_freq<-freq(dvl2)
dvl3_freq<-freq(dvl3) 
mat <- cbind(apc_freq, wnt5a_freq, beta_freq,fzd1_freq, fzd2_freq, dvl1_freq, dvl2_freq, dvl3_freq)
colname <- c("APC", "WNT5a", "beta-catenin", "Fzd1", "Fzd2", "dvl1", "dvl2", "dvl3")
mat2 <- mat[sort.list(mat[,1]), ]

par(mar= c(8,5, 3, 5))
image(mat2, col = tim.colors(64), axes = F, main = "WNT pathway mutations")
axis(1, at=seq(0,1, by=1/(length(rownames(mat))-1)),lab=rownames(mat2), las = 2)
axis(2, at=seq(0,1, by=1/(length(colname)-1)),lab=colname, las = 2)
image.plot(mat2, legend.only=T, legend.lab="Mutation Frequency", legend.mar=4.0)

kp <- unlist(mget(c("FAT4"), org.Hs.egSYMBOL2EG))
fat4 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("PIK3CA"), org.Hs.egSYMBOL2EG))
pik3ca <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("CSMD1"), org.Hs.egSYMBOL2EG))
csmd1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("CSMD2"), org.Hs.egSYMBOL2EG))
csmd2 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("CSMD3"), org.Hs.egSYMBOL2EG))
csmd3 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("SYNE1"), org.Hs.egSYMBOL2EG))
syne1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
kp <- unlist(mget(c("MUC6"), org.Hs.egSYMBOL2EG))
muc6 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("ATM"), org.Hs.egSYMBOL2EG))
atm <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("OBSCN"), org.Hs.egSYMBOL2EG))
obscn <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
##SMAD4
kp <- unlist(mget(c("SMAD4"), org.Hs.egSYMBOL2EG))
smad4 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("SMAD3"), org.Hs.egSYMBOL2EG))
smad3 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("SMAD2"), org.Hs.egSYMBOL2EG))
smad2 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
##APC
kp <- unlist(mget(c("APC"), org.Hs.egSYMBOL2EG))
apc <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
##TP53
kp <- unlist(mget(c("TP53"), org.Hs.egSYMBOL2EG))
tp53 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
##RAS
kp <- unlist(mget(c("KRAS"), org.Hs.egSYMBOL2EG))
kras <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("NRAS"), org.Hs.egSYMBOL2EG))
nras <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("HRAS"), org.Hs.egSYMBOL2EG))
hras <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
##RAF
kp <- unlist(mget(c("BRAF"), org.Hs.egSYMBOL2EG))
braf <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
kp <- unlist(mget(c("RAF1"), org.Hs.egSYMBOL2EG))
craf <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
kp <- unlist(mget(c("ARAF"), org.Hs.egSYMBOL2EG))
araf <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 
##MEK
kp <- unlist(mget(c("MAP2K1"), org.Hs.egSYMBOL2EG))
map2k1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("MAP2K2"), org.Hs.egSYMBOL2EG))
map2k2 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
##ERK
kp <- unlist(mget(c("MAPK1"), org.Hs.egSYMBOL2EG))
mapk1 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic")
kp <- unlist(mget(c("MAPK3"), org.Hs.egSYMBOL2EG))
mapk3 <- getAlleleByLocus_mod(gene=kp, meta=meta, gene_db=genes, ids=colon_sams, tech="exome", type = "somatic") 

kras_freq <-freq(kras)
nras_freq <-freq(nras)
hras_freq <-freq(hras)
braf_freq <-freq(braf)
craf_freq <- freq(craf)
araf_freq<-freq(araf)
map2k1_freq <-freq(map2k1)
map2k2_freq <-freq(map2k2)
mapk1_freq <-freq(mapk1)
mapk3_freq <-freq(mapk3)
apc_freq<-freq(apc)
tp53_freq<-freq(tp53)
smad4_freq<-freq(smad4)
smad3_freq<-freq(smad3)
smad2_freq<-freq(smad2)
atm_freq<-freq(atm)
obscn_freq<-freq(obscn)
muc6_freq<-freq(muc6)
syne1_freq<-freq(syne1)
csmd1_freq<-freq(csmd1)
csmd2_freq<-freq(csmd2)
csmd3_freq<-freq(csmd3)
pik3ca_freq<-freq(pik3ca)
fat4_freq<-freq(fat4)

mat <- cbind(kras_freq, nras_freq, hras_freq, braf_freq, craf_freq,
             araf_freq, map2k1_freq, map2k2_freq, mapk1_freq,
             mapk3_freq, apc_freq, tp53_freq, smad4_freq, smad3_freq,
             smad2_freq, atm_freq, obscn_freq, syne1_freq, csmd1_freq,
             csmd2_freq, csmd3_freq,muc6_freq, pik3ca_freq, fat4_freq)

loh <- get(load("~/svn/Projects/CRCanalysis/trunk/data/loh_data.RData"))
cnv <- get(load("~/svn/Projects/CRCanalysis/trunk/data/cnv_data.RData"))
vec <- matrix(NA, nrow=64, ncol=2)
rownames(vec) <- rownames(mat)
colnames(cnv) %in% rownames(mat)->cols
cnv[paste("GeneID.", kp, sep = ""),cols]->kras_cn
loh[paste("GeneID.", kp, sep = ""),cols]->kras_loh
vec[names(kras_cn), 1]<-kras_cn
vec[names(kras_loh), 2]<-kras_loh

colname <- c("KRAS", "NRAS", "HRAS", "BRAF", "c-RAF", "a-RAF",
             "MAP2K1", "MAP2K2", "MAPK1", "MAPK3", "APC", "TP53",
             "SMAD4", "SMAD3", "SMAD2", "ATM", "OBSCN", "SYNE1",
             "CSMD1", "CSMD2", "CSMD3", "MUC6", "PIK3CA", "FAT4")

mat2 <- mat[sort.list(mat[,19]), ]
vec2 <- vec[sort.list(mat[,1]),]

pdf(file = "RAS_PATH_MUT.pdf", height = 8, width = 16)
#par(mfrow=c(2,1))
par(mar= c(8,5, 3, 5))
image(mat2, col = tim.colors(64), axes = F, main = "RAS pathway mutations")
axis(1, at=seq(0,1, by=1/(length(rownames(mat))-1)),lab=rownames(mat2), las = 2)
axis(2, at=seq(0,1, by=1/(length(colname)-1)),lab=colname, las = 2)
image.plot(mat2, legend.only=T, legend.lab="Mutation Frequency", legend.mar=4.0) 
dev.off()

pdf(file = "KRAS_LOH.pdf", height = 5, width = 16)
par(mar= c(8,5, 3, 5))
image(vec2, col = tim.colors(64), axes = F, main = "KRAS CN/LOH")
axis(1, at=seq(0,1, by=1/(64-1)),lab=rownames(mat2), las = 2)
axis(2, at=c(0.05, 0.95),lab=c("CNV state", "LOH state"), las = 2)
image.plot(vec2, legend.only=T, legend.lab="CN/LOH value", legend.mar=4.0)
dev.off() 

freq<- function(list){
  freq <- lapply(list, function(x) {
    if(length(x)>0){
      return(max(values(x)$count/values(x)$count.total))
    }else{
      return(0)
    }
  })
  return(unlist(freq))
}

### =========================================================================
### Finding Somatic Mutations
### -------------------------------------------------------------------------

setGeneric("callSampleSpecificVariants", function(case, control, ...) {
  standardGeneric("callSampleSpecificVariants")
})

### TODO: check for positions that are completely non-ref in normal
### and somehow gain the reference allele in tumor.

## NEW ALGORITHM
## - Filter out all normal calls from tumor calls
## - For the remaining tumor calls, ask two questions:
##   - Did we have sufficient coverage in normal to make a call?
##   - Is the normal frequency reasonable, given the tumor frequency?

## IDEAS FROM ROBERT:
## - Assume het frequency (~50%) for power and extremity filter.
##   - But this will miss artifacts in the normal, and thus
##     conclude that they are tumor-specific...
##   - The power filter might work best as an annotation.
## - Maybe even assume het frequency when calling in normal,
##   which would automatically fix the power filter.
## - Maybe discard tumor variants that look het, but risky;
##   analysts always have access to frequencies.

setMethod("callSampleSpecificVariants", c("VRanges", "VRanges"),
          function(case, control, control.cov, ...)
          {
            filters <- SampleSpecificVariantFilters(control,
                                                    control.cov,
                                                    hardFilters(case),
                                                    ...)
            case.specific <- subsetByFilter(case, filters)
            annotateWithControlDepth(case.specific, control, control.cov)
          })

setMethod("callSampleSpecificVariants", c("BamFile", "BamFile"),
          function(case, control, tally.param,
                   calling.filters = VariantCallingFilters(),
                   post.filters = FilterRules(),
                   ...)
          {
            case.raw <- tallyVariants(case, tally.param)
            case.called <- callVariants(case.raw, calling.filters, post.filters)

            control.raw <- tallyVariants(control, tally.param)
            
            which <- tally.param@bamTallyParam@which
            if (is(which, "GRangesList")) which<-unlist(which)
            sbp <- ScanBamParam(which = which)
            control.cov <- coverage(control, drop.D.ranges = TRUE, param = sbp)

            callSampleSpecificVariants(case.called, control.raw,
                                       control.cov=control.cov,
                                       ...)
          })

setMethod("callSampleSpecificVariants", c("character", "character"),
          function(case, control, ...)
          {
            callSampleSpecificVariants(BamFile(case), BamFile(control), ...)
          })

SampleSpecificVariantFilters <-
  function(control, control.cov, calling.filters, power = 0.8, p.value = 0.01)
{
  control.called <- callVariants(control, calling.filters,
                                 post.filters = FilterRules())
  FilterRules(c(calledInControl = SetdiffVariantsFilter(control.called),
                power = CallableInOtherFilter(control.cov, calling.filters,
                  power),
                extremity = LowerFrequencyInOtherFilter(control, control.cov,
                  p.value)
                ))
}

SetdiffVariantsFilter <- function(other) {
  function(x) {
    !(x %in% other)
  }
}

SetdiffPositionsFilter <- function(other) {
  function(x) {
    !(x %over% other)
  }
}

extractCoverageForPositions <- function(cov, pos) {
  if (length(setdiff(seqlevels(pos), names(cov))) > 0L)
    stop("Some seqlevels are missing from coverage")
  if (any(width(pos) > 1L))
    stop("Some ranges are of width > 1")
  seqlevels(pos) <- names(cov)
  ord <- order(seqnames(pos))
  ans <- integer(length(pos))
  ans[ord] <- unlist(mapply(function(v, p) {
    runValue(v)[findRun(p, v)]
  }, cov, split(start(pos), seqnames(pos)), SIMPLIFY=FALSE), use.names=FALSE)
  ans
}

CallableInOtherFilter <-
  function(other.cov, calling.filters, min.power = 0.8,
           p.lower = params(calling.filters$likelihoodRatio)$p.lower)
{
  function(x) {
    calculatePowerInOther(x, other.cov, calling.filters, p.lower) >= min.power
  }
}

calculatePowerInOther <-
  function(x, other.cov, calling.filters = hardFilters(x),
           p.lower = params(calling.filters$likelihoodRatio)$p.lower)
{
  lr.filter <- calling.filters$likelihoodRatio
  if (is.null(lr.filter))
    f <- 0
  else
    f <- lrtFreqCutoff(params(lr.filter)$p.error, params(lr.filter)$p.lower)
  rc.filter <- calling.filters$readCount
  if (is.null(rc.filter))
    min.depth <- 0L
  else
    min.depth <- params(rc.filter)$min.depth
  other.n <- extractCoverageForPositions(other.cov, resize(x, 1))
  min.depth <- ceiling(pmax(min.depth, other.n * f))
  1 - pbinom(min.depth-1L, other.n, p.lower)
}

LowerFrequencyInOtherFilter <- function(other, other.cov, p.value = 0.01)
{
  function(x) {
      ## NOTE: this was once based on the raw (not quality filtered)
      ## counts, but bam_tally is no longer giving us those for the alts.
    x <- annotateWithControlDepth(x, other, other.cov)
    p <- with(x, pbinom(control.alt.depth, control.total.depth,
                        as.integer(altDepth / totalDepth)))
    p < p.value
  }
}

annotateWithControlDepth <- function(case, control, control.cov) {
  m <- match(case, control)
  control.alt.depth <- as.vector(altDepth(control))[m]
  control.alt.depth[is.na(control.alt.depth)] <- 0L
  control.total.depth <- as.vector(totalDepth(control))[m]
  control.raw.total.depth <- as.vector(rawTotalDepth(control))[m]
  control.raw.total.depth[is.na(m)] <-
      extractCoverageForPositions(control.cov, resize(case[is.na(m)], 1))
  control.total.depth[is.na(m)] <- control.raw.total.depth[is.na(m)]
  case$control.alt.depth <- control.alt.depth
  case$control.total.depth <- control.total.depth
  case$control.raw.total.depth <- control.raw.total.depth
  case
}

caseControlFET <- function(case, control, control.cov) {
  case <- annotateWithControlDepth(case, control, control.cov)
  with(case,
       fisher_p(altDepth, (totalDepth - altDepth),
                altDepth + control.alt.depth,
                (totalDepth - altDepth) +
                (control.total.depth - control.alt.depth)))
}

DepthFETFilter <- function(control, control.cov, p.value.cutoff = 0.05) {
  function(x) {
    p.value <- caseControlFET(x, control, control.cov)
    p.value < p.value.cutoff
  }
}

MaxControlFreqFilter <- function(control, control.cov, max.control.freq = 0.03) {
  function(x) {
    x <- annotateWithControlDepth(x, control, control.cov)
    freq <-  x$control.alt.depth / x$control.total.depth
    ifelse(is.na(freq), 0, freq) <= max.control.freq
  }
}

plotT <- function(normal_sam, tumor_sam){
  dir <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/Exome/"
  tumor_gr <- get(load(paste(dir, tumor_sam, "/results/", tumor_sam, ".filtered_variants_granges.RData", sep = "")))
  normal_gr <- get(load(paste(dir, normal_sam,"/results/", normal_sam, ".filtered_variants_granges.RData", sep = "")))
  normal_raw <- get(load(paste(dir, normal_sam, "/results/", normal_sam, ".variants_granges.RData", sep="")))
  normal_cov <-  get(load(paste("/gne/research/data/cgp/2.0/", normal_sam, "/exome_merged/RData/", normal_sam, ".cov.RData", sep = "")))
  VariantTools:::plotVarGenome(tumor_gr= tumor_gr, normal_gr = normal_gr, normal_raw=normal_raw, normal_cov=normal_cov)
}

## TODO: use dispatch on special file classes
file_ext_sans_gz <- function(x) {
  x <- sub("[.]b?gz$", "", x)
  file_ext(x)
}

loadVariants <- function(x, ...) {
  if (file_ext_sans_gz(x) == "vcf") {
    vcf <- readVcf(x, ...)
    geno(vcf)$DP <- NULL
    as(vcf, "VRanges")
  }
  else get(load(x))
}

calculateConcordanceMatrix <- function(variantFiles, ...) {
  if (length(variantFiles) < 2L) {
    stop("There must at least two variant sets.")
  }
  if (is.character(variantFiles)) {
      if (!all(file.exists(variantFiles))) {
          stop("Some of the files could not be found.")
      }
      nms <- variantFiles
  } else {
      if (!all(vapply(variantFiles, is, class2="VRanges", logical(1L))))
          stop("'variantFiles' must be a vector of file names or ",
               "a list of VRanges objects")
      nms <- names(variantFiles)
  }

  ## initialize vcmat, pairwise variance concordant score between dirs
  n <- length(variantFiles)
  vcmat <- matrix(NA, nrow=n, ncol=n)
  rownames(vcmat) <- colnames(vcmat) <- nms

  ## compute vcmat
  for (i in 1:(n - 1)) {
    avar <- if (is(variantFiles[[i]], "VRanges"))
                x
            else try(loadVariants(variantFiles[i], ...), silent=TRUE)
    if (class(avar) == "try-error") {
      stop("error: cannot load '", nms[i], "': ", avar)
    }
  
    for (j in (i+1):n) {
      bvar <- if (is(variantFiles[[j]], "VRanges"))
                  x
              else try(loadVariants(variantFiles[j], ...), silent=TRUE)
      if (class(bvar) == "try-error") {
        stop("error: cannot load '", nms[j], "': ", bvar)
      }
    
      ## compute variant concordance
      vc <- calculateVariantConcordance(avar, bvar)

      ## output results
      vcmat[i, i] <- 1
      vcmat[j, j] <- 1
      vcmat[i, j] <- vc[1]$fraction
      vcmat[j, i] <- vc[1]$fraction

      if (getOption("verbose"))
        message(paste("variantConcordance:",
                      nms[i],
                      nms[j],
                      vc$fraction, vc$denominator, sep="\t"))
    }
  }
  return(vcmat)  
}

callVariantConcordance <- function(concordanceMatrix,
                                   threshold) {
  if (!installed("RBGL"))
    stop("The 'RBGL' package is required for finding the concordant cliques")
  
  concordantCliques <- .getConcordantCliques(concordanceMatrix,
                                             threshold)
  ## test if connected components are cliques of
  ##at least 2 vertices

  n <- sum(elementNROWS(concordantCliques))
  vstate <- rep("non-concordant", n)
  names(vstate) <- unlist(concordantCliques)

  ##if there are multiple cliques of size 2 or greater, the entire set
  ##is undecidable
  numMultinodeCliques <- sum(elementNROWS(concordantCliques) >= 2)
  if (numMultinodeCliques >= 2L) {
    vstate <- rep("undecidable", n)
  } else if (numMultinodeCliques == 1L) {
    ##if in the clique with at least two elements, is concordant
    multinodeCliqueElements <-
      concordantCliques[[which(elementNROWS(concordantCliques) >= 2)]]
    vstate[multinodeCliqueElements] <- "concordant"
  }
  return(vstate)
}

##################
##HELPER FUNCTIONS
##################

.getConcordantCliques <- function(concordanceMatrix, threshold) {
  adj <- concordanceMatrix > threshold
  if (is.null(colnames(adj)))
    colnames(adj) <- paste0("id", seq_len(ncol(adj)))
  V <- colnames(adj)
  edL <- vector("list", length=length(V))
  names(edL) <- V
  for (i in seq_along(V)) {
    indicesOfConnectedNodes <- which(adj[, i] == TRUE)
    edL[[i]] <- list(edges=indicesOfConnectedNodes)
  }
  gR <- graph::graphNEL(nodes=V, edgeL=edL)
  RBGL::maxClique(gR)$maxCliques
}

plotFreq <- function(sam){
  par(mfrow = c(1,2))
  vep <- read.table(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/",
                          sam,
                          "_variant_annotations.txt",sep = ""),
                    sep = "\t")
  rna_gr <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/RNA/",
                           sam,
                           "/results/",
                           sam,
                           ".variants_granges.RData", sep = "")
                     ))
  ex_gr <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/Exome/",
                          sam,
                          "/results/",
                          sam,
                          ".filtered_variants_granges.RData", sep = "")
                    ))
  rna_gr <- rna_gr[values(rna_gr)$count.total >10]
  ns <- vep[vep$V7 == "NON_SYNONYMOUS_CODING",]
  s <- vep[vep$V7 == "SYNONYMOUS_CODING",]
  s_locs <- paste("chr", s$V2, ":*", sep = "")
  ns_locs <- paste("chr", ns$V2, ":*", sep = "")
  ns_ex_gr <- ex_gr[values(ex_gr)$location %in% ns_locs]
  s_ex_gr <- ex_gr[values(ex_gr)$location %in% s_locs]
  ns_over <- findOverlaps(ns_ex_gr, rna_gr)
  s_over <- findOverlaps(s_ex_gr, rna_gr)
  ns_over_ex <- ns_ex_gr[queryHits(ns_over)]
  ns_over_rna <- rna_gr[subjectHits(ns_over)]
  s_over_ex <- s_ex_gr[queryHits(s_over)]
  s_over_rna <- rna_gr[subjectHits(s_over)]
  plot(values(ns_over_ex)$count/values(ns_over_ex)$count.total, values(ns_over_rna)$count/values(ns_over_rna)$count.total, pch = 19, cex = 0.4, col = "blue", xlim = c(0,1), ylim =c(0,1), ylab = "Alternate read frequency in RNA", xlab = "Alternate read frequency in Exome", main = sam)
  par(new=T)
  plot(values(s_over_ex)$count/values(s_over_ex)$count.total, values(s_over_rna)$count/values(s_over_rna)$count.total, pch = 19, cex = 0.4, col = "red", xlim = c(0,1), ylim = c(0,1), xlab= NA, ylab = NA)
  abline(0,1)
  v2 <- (values(ns_over_ex)$count/values(ns_over_ex)$count.total)/(values(ns_over_rna)$count/values(ns_over_rna)$count.total)
  v1 <- (values(s_over_ex)$count/values(s_over_ex)$count.total)/(values(s_over_rna)$count/values(s_over_rna)$count.total)
  breaks = seq(-5, 20, by = 0.5)
  names = breaks[1:(length(breaks)-1)]
  c1 = hist(log2(v1), breaks = breaks, plot = FALSE)$count
  c2 = hist(log2(v2), breaks = breaks, plot = FALSE)$count
  barplot(rbind(c1/sum(c1), c2/sum(c2)), beside =T, legend = c("SYN", "Non-SYN"), names = names, las=2, ylab = "Normalized fraction", xlab = "log2 (exome freq / rna freq)" )
}




getGenes <- function(sam, freq){
  vep <- read.table(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/",
                          sam,
                          "_variant_annotations.txt",sep = ""),
                    sep = "\t")
  if(file.exists(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/RNA/",
                       sam,
                       "/results/",
                       sam,
                       ".variants_granges.RData", sep = "")
                 )){
    rna_gr <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/RNA/",
                             sam,
                             "/results/",
                             sam,
                             ".variants_granges.RData", sep = "")
                       ))
    ex_gr <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/Exome/",
                            sam,
                            "/results/",
                            sam,
                            ".filtered_variants_granges.RData", sep = "")
                      ))
  }else{
    return(NULL)
  }
  rna_gr <- rna_gr[values(rna_gr)$count.total >10]
  ns <- vep[vep$V7 == "NON_SYNONYMOUS_CODING",]
  s <- vep[vep$V7 == "SYNONYMOUS_CODING",]
  s_locs <- paste("chr", s$V2, ":*", sep = "")
  ns_locs <- paste("chr", ns$V2, ":*", sep = "")
  ns_ex_gr <- ex_gr[values(ex_gr)$location %in% ns_locs]
  s_ex_gr <- ex_gr[values(ex_gr)$location %in% s_locs]
  ns_over <- findOverlaps(ns_ex_gr, rna_gr)
  s_over <- findOverlaps(s_ex_gr, rna_gr)
  ns_over_ex <- ns_ex_gr[queryHits(ns_over)]
  ns_over_rna <- rna_gr[subjectHits(ns_over)]
  s_over_ex <- s_ex_gr[queryHits(s_over)]
  s_over_rna <- rna_gr[subjectHits(s_over)]
  ns_over_rna[values(ns_over_rna)$count/values(ns_over_rna)$count.total >freq]->high
  vep[which(ns_locs %in% values(high)$location),]->damaging
  split <-strsplit(as.character(damaging$V14), split = "=")
  gene_ids2 <- lapply(split, function(x) x[5])
  gene_ids2 <- unique(unlist(gene_ids2))
  return(gene_ids2)
}

library(CGPtools)

colnames <- c("Uploaded_variation",
              "Location",
              "Allele", "Gene", "Feature",
              "Feature_type", "Consequence", "DNA_position",
              "CDS_position", "Protein_position",
              "Amino_acids", "Codons", "Existing_variation", "Extra")

dir <-"/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/"
files <- dir(dir)
files <- files[grepl(pattern=".txt", files)]
file_sam <- sub("_variant_annotations.txt", "", files)

meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/release_2.1/released.json")
colon <- c('Colon', 'Colon Sigmoid', 'Rectum', 'Cecum', 'Ileum', 'Intestine Small')
exome <- meta[meta$project == "exome",]
normals <- as.character(exome$srcid[exome$tissue_diagnosis %in% c("Normal", "Normal Adjacent Tumor")])
tissues <- as.character(exome$primary_tissue)
names(tissues) <- as.character(exome$srcid)
tissues[normals]<-"normal"
tissues[tissues %in% colon]<-"Colon"
colon_sams <- names(tissues[tissues=="Colon"])
colon_sams <- as.list(colon_sams)


sam <- colon_sams[[1]]
DG <- damaging_genes(sam)

DG <- lapply(colon_sams, function(sam) damaging_genes(sam))

lung_DG <- lapply(lung_sams, function(sam) damaging_genes(sam))

SpG <- lapply(colon_sams, function(sam) splice_genes(sam))

damaging_genes <- function(SAM){
  x <- files[which(file_sam ==SAM)]
  if(length(x) != 0){
    tab <- read.table(file.path(dir, x), sep = "\t", stringsAsFactors=FALSE, colClasses="character")
    colnames(tab)<-colnames
    damaging <- tab[grepl(pattern="STOP_GAINED", tab$Consequence),]
    split <-strsplit(damaging$Extra, split = "=")
    gene_ids1 <- lapply(split, function(x) x[2])
    gene_ids1 <- unique(unlist(gene_ids1))
    damaging <- tab[(grepl(pattern="damaging", tab$Extra) | grepl(pattern="deleterious", tab$Extra)),]
    split <-strsplit(damaging$Extra, split = "=")
    gene_ids2 <- lapply(split, function(x) x[5])
    gene_ids2 <- unique(unlist(gene_ids2))
    gene_ids<-c(gene_ids1, gene_ids2)
    return(gene_ids)
  }else{
    return(NULL)
  }
}

splice_genes <- function(SAM){
  x <- files[which(file_sam ==SAM)]
  if(length(x) != 0){
    tab <- read.table(file.path(dir, x), sep = "\t", stringsAsFactors=FALSE, colClasses="character")
    colnames(tab)<-colnames
    damaging <- tab[grepl(pattern="SPLICE_SITE", tab$Consequence),]
    split <-strsplit(damaging$Extra, split = "=")
    gene_ids <- lapply(split, function(x) x[5])
    gene_ids <- unique(unlist(gene_ids))
    return(gene_ids)
  }else{
    return(NULL)
  }
}

## Convert variant GRanges into a VCF
## **DEPRECATED** in favor of VRanges

variantGR2Vcf <- function(x, sample.id, project = NULL,
                          genome = unique(GenomicRanges::genome(x)))
{
  variantGRangesIsDeprecated("variantGR2Vcf")
  vr <- makeVRangesFromVariantGRanges(x, genome)
  sampleNames(vr) <- sample.id
  genome(vr) <- genome
  metadata(vr)$project <- project
  asVCF(vr, meta = if (!is.null(project)) "project" else character())
}

rnaValRate <- function(gr, rna, cov, type = "rate",min=0){
  values(gr)$norm_ref <- as.numeric(NA)
  values(gr)$norm_alt <- as.numeric(NA)
  over2 <- which(paste(values(gr)$location,values(gr)$read) %in% paste(values(rna)$location, values(rna)$read))
  over <- which(paste(values(rna)$location,values(rna)$read) %in% paste(values(gr)$location, values(gr)$read))
  values(gr[over2])$norm_ref <- values(rna[over])$count.ref
  values(gr[over2])$norm_alt <- values(rna[over])$count
  values(gr[-over2])$norm_alt <- 0
  gr<-gr[values(gr)$count.total>min]
  grl <- split(gr, as.factor(seqnames(gr)))
  names <- as.list(names(grl))
  n <- lapply(names, function(x) { as.numeric(cov[[x]][ranges(grl[[x]])]) })
  n <- unlist(n)
  gr <- unlist(grl)
  values(gr)$norm_cov<-n
  gr[values(gr)$norm_cov >20]->high
  if(length(high)>=0){
    val_rate <- 1- length(high[values(high)$norm_alt==0])/length(high)
    cat("RNA validation rate: ", val_rate, "\n", sep = "")
    if(type == "rate"){
      return(c(val_rate, length(high), length(high[values(high)$norm_alt==0])))
    }else{
      return(gr)
    }
  }else{
    return(NULL)
  }
}

### =========================================================================
### Genotype Calling
### -------------------------------------------------------------------------
###
### Routines for calling and manipulating genotypes.
###

setGeneric("callGenotypes",
           function(variants, cov, ...) standardGeneric("callGenotypes"))

### Strategy: calculate binomial probabilities and call
### whichever PL is 0 (p=1.0).

### This is equivalent to the LRT, as long as:
## p(0/0) is asking if the rate is the error rate (e)
## p(0/1) is asking if the rate is 0.5
## p(1/1) is asking if the rate is 1-e

## Generated fields:
## totalDepth: average coverage (DP)
## MIN_DP: minimum coverage
## GQ: genotype quality (p(GT) / p(AA) + p(RA) + p(AA))
## PL: likelihood of each genotype (RR, RA, AA) - binomial

phred <- function(p, max = 9999L) {
  ans <- pmin(round(-10 * log10(p)), max)
  storage.mode(ans) <- "integer"
  ans
}

compute_GQ <- function(gl, max = 99L) {
  phred(1-gl[cbind(seq_len(nrow(gl)), max.col(gl))] / rowSums(gl), max=max)
}

compute_GL <- function(ad, dp, p.error = 0.05) {
  gl <- cbind(dbinom(ad, dp, p.error),
              dbinom(ad, dp, 0.5),
              dbinom(ad, dp, 1-p.error))
  gl * 1.0 / gl[cbind(seq_len(nrow(gl)), max.col(gl, ties.method="first"))]
}

compute_GT <- function(gl) {
  c("0/0", "0/1", "1/1")[max.col(gl, ties.method="first")]
}

GenotypeRunVRanges <- function(ranges, proto, which, genome) {
  runs.gr <- GRanges(seqnames(which), shift(ranges, start(which) - 1L))
  ref <- getSeq(genome, resize(runs.gr, 1L))
  runs.vr <- VRanges(seqnames(runs.gr), ranges(runs.gr),
                     ref=ref, alt="<NON_REF>",
                     softFilterMatrix=matrix(NA, length(runs.gr),
                       ncol(softFilterMatrix(proto))),
                     seqlengths = seqlengths(proto))
  ml <- rep(list(rep(NA, length(runs.gr))), ncol(mcols(proto)))
  mcols(runs.vr) <- as(setNames(ml, names(mcols(proto))), "DataFrame")
  seqinfo(runs.vr) <- seqinfo(proto)
  runs.vr
}

## We take the GQ binning approach of GATK. An alternative strategy
## (from gvcftools) would be requiring data to fall into [x,y] where
## y <= x+max(3,x*0.3).

## We also take the average coverage for DP, like GATK, despite the
## gVCF spec recommending we take the min. Instead, the min is MIN_DP.

loadAndComputeGenotypesForRegion <- function(vcf, cov, which, ...) {
  vr <- readVcfAsVRanges(vcf, which=which)
  if (length(runValue(sampleNames(vr))) > 1L) {
    stop("variants must be from a single sample")
  }
  callGenotypesOneRegion(vr, cov, which, ...)
}

callGenotypesOneRegion <- function(variants, cov, which, param) {
  gl <- compute_GL(altDepth(variants), totalDepth(variants), param@p.error)
  variants$GT <- compute_GT(gl)
  variants$GQ <- compute_GQ(gl)
  variants$PL <- phred(gl)
  variants$MIN_DP <- rep(NA_integer_, length(variants))
  if (!is.null(cov)) {
    variants <- addGenotypeRuns(variants, cov, which, param)
  }
  variants
}

addGenotypeRuns <- function(variants, cov, which, param) {
  if (length(runLength(seqnames(variants))) > 1L) {
    stop("currently all ranges must be on the same sequence")
  }
  which <- keepSeqlevels(which, as.character(seqnames(which)))
  cov <- if (is(cov, "BigWigFile"))
      import(cov, which=which, as="NumericList")[[1L]]
  else cov[which][[1L]]
  cov.v <- as.integer(cov)
  gl <- compute_GL(0, cov.v, param@p.error)
  gq <- compute_GQ(gl)
  gq.runs <- ranges(Rle(cut(gq, param@gq.breaks)))
  runs <- setdiff(gq.runs, shift(ranges(variants), 1L - start(which)))
  runs.vr <- GenotypeRunVRanges(runs, variants, which, param@genome)
  runs.vr$GT <- Rle("0/0", length(runs.vr))
  runs.vr$GQ <- viewMins(Views(gq, runs))
  runs.vr$PL <- matrix(apply(phred(gl), 2,
                             function(gli) viewMins(Views(gli, runs))),
                       ncol=3L)
  cov.views <- Views(cov.v, runs)
  totalDepth(runs.vr) <- viewMeans(cov.views)
  runs.vr$MIN_DP <- viewMins(cov.views)
  c(variants, runs.vr)
}

setClass("CallGenotypesParam",
         representation(gq.breaks = "numeric",
                        p.error = "numeric",
                        genome = "ANY",
                        which = "GenomicRangesList"))

CallGenotypesParam <- function(genome, gq.breaks = c(0, 5, 20, 60, Inf),
                               p.error = 0.05,
                               which = tileGenome(seqinfo(genome), ntile=ntile),
                               ntile = 100L)
{
  if (!hasMethod("getSeq", class(genome))) {
    if (!isSingleString(genome))
      genome <- genome(genome)
    if (requireNamespace("gmapR", quietly=TRUE)) {
      genome <- gmapR::GmapGenome(genome)
    } else {
      genome <- BSgenome::getBSgenome(genome)  
    }
  }
  if (any(is.na(gq.breaks)) || any(gq.breaks < 0)) {
    stop("'gq.breaks' values must be non-negative and non-NA")
  }
  if (!isSingleNumber(p.error) || p.error < 0) {
    stop("'p.error' must be a single, non-negative, non-NA number")
  }
  if (!isSingleNumber(ntile) || ntile < 0) {
    stop("'ntile' must be a single, non-negative, non-NA number")
  }
  new("CallGenotypesParam", gq.breaks=gq.breaks, p.error=p.error,
      genome=genome, which=which)
}

setMethod("callGenotypes", "VRanges",
          function(variants, cov,
                   param = CallGenotypesParam(variants),
                   BPPARAM = defaultBPPARAM())
          {
            sample <- runValue(sampleNames(variants))
            if (length(sample) > 1L) {
              stop("variants must be from a single sample")
            }
            which <- unlist(param@which)
            f <- factor(findOverlaps(variants, which, select="arbitrary"),
                        seq_len(length(which)))
            vl <- split(variants, f)
            ansl <- bpmapply(callGenotypesOneRegion,
                             vl,
                             as(which, "GRangesList"),
                             MoreArgs=list(
                               cov=cov,
                               param=param
                               ),
                             BPPARAM=BPPARAM)
            ans <- do.call(c, unname(ansl))
            sampleNames(ans) <- sample
            ans
          })

setMethod("callGenotypes", "TabixFile",
          function(variants, cov,
                   param = CallGenotypesParam(variants),
                   BPPARAM = defaultBPPARAM())
          {
            which <- as.list(unlist(param@which))
            ansl <- bplapply(which, loadAndComputeGenotypesForRegion,
                             vcf=variants, bigwig=cov, param=param,
                             BPPARAM=BPPARAM)
            ans <- do.call(c, unname(ansl))
            sample <- na.omit(unique(sampleNames(ans)))
            sampleNames(ans) <- sample
            ans
          })

setMethod("show", "CallGenotypesParam", function(object) {
  cat("A", class(object), "object\n", sep = " ")
  cat(gmapR:::showSlots(object, count = FALSE), sep = "")
})

library(VariantTools)
##library(RNASeqGenie)


cmd_args = commandArgs(trailingOnly=TRUE);
bam_file <- cmd_args[1];
read_length=cmd_args[2];
genome<- cmd_args[3];
genome_dir <- cmd_args[4];
save_dir <- cmd_args[5];
SAMid <- cmd_args[6];
project<-cmd_args[7];

if(file.exists(save_dir)){
  stop("Error: Save directory already exists")
}else{
  dir.create(save_dir)
  dir.create(file.path(save_dir, "results"))
}
           
qual <- detectQualityType(bam_file)
if(qual=="illumina"){
  var<- callVariantsP(bam=bam_file, genome=genome, genome_dir=genome_dir,mapq=13, bqual=56, force=TRUE,with_qual=T, read_length=read_length)
}else if(qual=="sanger"){
  var<- callVariantsP(bam=bam_file, genome=genome, genome_dir=genome_dir,mapq=13, bqual=23,with_qual=T, read_length=read_length)
}else{
  stop("Error: Non-standard quality in BAM")
}
saveWithID(var$raw_granges, "variants_granges", SAMid,
           save_dir = file.path(save_dir, "results"),
           compress = FALSE)
saveWithID(var$filtered_granges, "filtered_variants_granges", SAMid,
           save_dir = file.path(save_dir,"results"),
           compress = FALSE)

VariantTools::cgpGr2vcf(GR=var$filtered_granges, sample_id=SAMid,
          filename=file.path(save_dir, "results",
            paste(SAMid,".vcf", sep = "")),
          project=project)

### =========================================================================
### Variant Calling
### -------------------------------------------------------------------------

setGeneric("callVariants", function(x, ...) standardGeneric("callVariants"))

## This method has explicit arguments for each stage in the
## pipeline. This is a concious choice to expose the algorithmic
## details (but not the implementation details).

setMethod("callVariants", "BamFile",
          function(x, tally.param,
                   calling.filters = VariantCallingFilters(...),
                   post.filters = FilterRules(),
                   ...)
          {
            if (!missing(calling.filters) && length(list(...)) > 0L) {
              warning("ignored arguments: ",
                      paste(names(list(...)), collapse=", "))
            }
            raw_variants <- tallyVariants(x, tally.param)
            callVariants(raw_variants, calling.filters, post.filters)
          })

setMethod("callVariants", "character", function(x, ...) {
  callVariants(BamFile(x), ...)
})

setMethod("callVariants", "GenomicRanges",
          function(x, calling.filters = VariantCallingFilters(...),
                   post.filters = FilterRules(), ...)
          {
            variantGRangesIsDeprecated("callVariants,GenomicRanges")
            callVariants(variantGRangesToVRanges(x),
                         calling.filters = calling.filters,
                         post.filters = post.filters, ...)
          })

setMethod("callVariants", "VRanges",
          function(x, calling.filters = VariantCallingFilters(...),
                   post.filters = FilterRules(), ...)
          {
            called_variants <- subsetByFilter(x, calling.filters)
            postFilterVariants(called_variants, post.filters)
          })

VariantCallingFilters <-
  function(read.count = 2L, p.lower = 0.2, p.error = 1/1000)
{
  c(VariantSanityFilters(),
    FilterRules(list(readCount = MinAltDepthFilter(read.count),
                     likelihoodRatio = BinomialLRFilter(p.lower, p.error))))
}

MinAltDepthFilter <- function(min.depth = 2L) {
  function(x) {
    altDepth(x) >= min.depth
  }
}

MinTotalDepthFilter <- function(min.depth = 10L) {
  function(x) {
    totalDepth(x) >= min.depth
  }
}

BinomialLRFilter <-
  function(p.lower = 0.2, p.error = 1/1000)
{
  function(x) {
    freq.cutoff <- lrtFreqCutoff(p.error, p.lower)
    sample.freq <- altFraction(x)
    passed <- sample.freq >= freq.cutoff
    passed[is.na(passed)] <- FALSE
    passed
  }
}

lrtFreqCutoff <- function(p0, p1, n = if (C == 1L) 1L else n, C = 1L) {
  num <- (1/n) * log(C) + log(1-p0) - log(1-p1)
  denom <- log(p1) - log(p0) + log(1-p0) - log(1-p1)
  num/denom
}

sanitizeVariants <- function(x, ...) {
  subsetByFilter(x, VariantSanityFilters(...))
}

VariantSanityFilters <- function() {
  FilterRules(list(nonRef = NonRefFilter(), nonNRef = NonNRefFilter()))
}

### =========================================================================
### Filter Diagnostics
### -------------------------------------------------------------------------
###
### To diagnose filter performance, several pieces of information
### would be helpful:
### * Results for each filter (as evalSeparately would return, but convenient)
### * Statistics considered by each filter. How to modularize this?
###   - Filter object has a function that returns the stats
###   - Filter function returns stats as an attribute
###   - Should wait to see whether we ever need this
### * Concordance with some ground truth, including FN/TN rates, +/- exclusive
###
### There may also be some helpful plots.

### =========================================================================
### Variant QA (as opposed to the calling filters)
### -------------------------------------------------------------------------

qaVariants <- function(x, qa.filters = VariantQAFilters(...), ...)
{
  softFilter(x, qa.filters)
}

VariantQAFilters <- function(fisher.strand.p.value = 1e-4, min.mdfne = 10L)
{
  FilterRules(c(mdfne = MedianDistFromNearestEndFilter(min.mdfne),
                fisherStrand = StrandFETFilter(fisher.strand.p.value)))
}

## With new gmapR, this is only necessary for filtering the ref N's.
## In theory, we could keep positions with ref N and at least two alts.
## But this is OK for now.
NonNRefFilter <- function() {
  function(x) {
    as.character(ref(x)) != 'N'
  }
}

## Drops the ref rows (should not be necessary)
NonRefFilter <- function() {
  function(x) {
    !is.na(alt(x))
  }
}

ReadPosCountFilter <- function(read.pos.count = 1L) {
  function(x) {
    (if (!is.null(x$ncycles)) x$ncycles else x$n.read.pos) >= read.pos.count
  }
}

StrandFETFilter <- function(p.value = 1e-4) {
  function(x) {
    p <- with(mcols(x),
              fisher_p_vectorized(count.plus.ref,
                                  count.minus.ref,
                                  (count.plus.ref + count.plus),
                                  (count.minus.ref + count.minus)))
    p > p.value
  }
}

InternalReadPosBinFilter <- function(min.count = 1L) {
  function(x) {
    read_pos_columns <- grep("readPosCount", colnames(mcols(x)), value = TRUE)
    alt_columns <- grep("ref", read_pos_columns, invert = TRUE, value = TRUE)
    internal_columns <- tail(head(alt_columns, -1), -1)
    if (length(internal_columns) > 0L)
      rowSums(as.matrix(mcols(x)[internal_columns])) >= min.count
    else rep.int(TRUE, length(x))
  }
}

t.test_welch <- function(m1, m2, s1, s2, n1, n2) {
  s <- s1 / n1 + s2 / n2
  t <- (m1 - m2) / sqrt(s)
  v <- s^2 / ((s1 / n1)^2 / (n1 - 1L) + (s2 / n2)^2 / (n2 - 1L))
  pt(-abs(t), v) * 2L
}

ReadPositionTTestFilter <- function(p.value.cutoff = 1e-4) {
  function(x) {
    p <- with(mcols(x), t.test_welch(read.pos.mean, read.pos.mean.ref,
                                     read.pos.var, read.pos.var.ref,
                                     rawDepth(x), totalDepth(x)))
    ans <- p > p.value.cutoff
    ans[is.na(ans)] <- TRUE
    ans
  }
}

DistanceToNearestFilter <- function(min.dist = 10L) {
  function(x) {
    mcols(distanceToNearest(x))$distance > min.dist
  }
}

NeighborCountFilter <- function(max.count = 2L, window.size = 75L) {
  function(x) {
    countOverlaps(resize(x, window.size, fix = "center"), x) <= max.count
  }
}

IndelsNotSupportedFilter <- function() {
  function(x) {
    nzchar(ref(x)) & nzchar(alt(x))
  }
}

MaskFilter <- function(mask) {
  function(x) {
    !overlapsAny(x, mask, ignore.strand = TRUE)
  }
}

MedianDistFromNearestEndFilter <- function(min.mdfne) {
  function(x) {
    !is.na(x$mdfne) & x$mdfne >= min.mdfne
  }
}

### =========================================================================
### Utilities
### -------------------------------------------------------------------------

## May push up to gmapR if it proves more generally useful

flankingCycleBreaks <- function(read_length, width = 10L) {
  read_length <- as.integer(read_length)
  if (is.na(read_length))
    return(NULL)
  if (read_length < 1)
    stop("'read_length' must be >= 1 or NA")
  if (width < 0)
    stop("'width' must be non-negative")
  
  as.integer(c(0L, width, read_length - width, read_length))
}

installed <- function(x) {
  !identical(suppressWarnings(packageDescription(x)), NA)
}

chunkRange <- function(which, n) {
  ## Why not just use breakInChunks(width(which), nchunk=n) here?
  ## H.P. [Jan 25, 2018]
  chunks <- breakInChunks(width(which), chunksize=ceiling(width(which) / n))
  which <- GRanges(seqnames(which),
                   IRanges(start(which) + start(chunks) - 1L,
                           width = width(chunks)))

}

## some internal compatibility wrappers for older versions
rawTotalDepth <- function(x) {
    if (!is.null(x$raw.count.total))
        x$raw.count.total
    else x$count.total
}

TwoSampleFET <- function(gr, raw, cov, pval=0.001, freq=0.05){
  values(gr)$norm_ref <- as.numeric(NA)
  values(gr)$norm_alt <- as.numeric(NA)
  over2 <- which(paste(values(gr)$location,values(gr)$read) %in% paste(values(raw)$location, values(raw)$read)) 
  over <- which(paste(values(raw)$location,values(raw)$read) %in% paste(values(gr)$location, values(gr)$read))
  values(gr[over2])$norm_ref <- values(raw[over])$count.ref
  values(gr[over2])$norm_alt <- values(raw[over])$count
  values(gr[-over2])$norm_alt <- 0

  grl <- split(gr, as.factor(seqnames(gr)))
  names <- as.list(names(grl))
  n <- lapply(names, function(x) { as.numeric(cov[[x]][ranges(grl[[x]])]) })
  n <- unlist(n)
  gr <- unlist(grl)
  values(gr)$norm_cov<-n
  pvals <- fisher_p(values(gr)$count, (values(gr)$count.total -values(gr)$count),
           values(gr)$count+values(gr)$norm_alt,
           (values(gr)$count.total-values(gr)$count)+(values(gr)$norm_cov-values(gr)$norm_alt))
  values(gr)$pval <- pvals
  TS <- gr[values(gr)$pval<pval &
                 values(gr)$norm_alt/
                 (values(gr)$norm_cov)<freq]
  return(TS)
}

### =========================================================================
### Tallying Alignments for Variant Calling
### -------------------------------------------------------------------------

### Thoughts on a tally abstraction:
###
### Want to support both bam_tally and applyPileups. Ideally, there
### would be one generic called tallyVariants(). We could dispatch on
### the parameter object, with a BamTallyVariantsParam() and a
### PileupTallyVariantsParam(). BamTallyVariantsParam could be
### provided by the gmapR package, which would depend on VariantTools,
### instead of the other way around.

### Or, tallyVariants() could move to VariantAnnotation.

setGeneric("tallyVariants", function(x, ...) standardGeneric("tallyVariants"))

defaultBPPARAM <- function() registered()[[1]]

setMethod("tallyVariants", "BamFile",
          function(x, param = TallyVariantsParam(...), ...,
                   BPPARAM = defaultBPPARAM())
          {
            if (!missing(param) && length(list(...)) > 0L) {
              warning("arguments in '...' are ignored when passing 'param'")
            }
            if (!is(param, "TallyVariantsParam")) {
              stop("'param' must be a TallyVariantsParam")
            }
            tally_region <- function(x, which, param) {
              iit <- gmapR::bam_tally(x, param@bamTallyParam, which = which)
              keep_ref_rows <- param@bamTallyParam@variant_strand == 0L  
              ans <- gmapR::variantSummary(iit, param@read_pos_breaks,
                                           keep_ref_rows,
                                           param@read_length,
                                           param@high_nm_score)
              ## usage of start() is intentional to avoid dropping indels
              ## that extend outside of window
              ans <- ans[start(ans) >= start(which) & start(ans) <= end(which)]
              if (!param@keep_extra_stats)
                mcols(ans) <- NULL
              ans[!(ans %over% param@mask)]
            }
            tally_region_job <- function(x, which, param) {
                do.call(c, unname(lapply(as(which, "GRangesList"), tally_region,
                                         x=x, param=param)))
            }
            which <- param@bamTallyParam@which
            if (length(which) == 0L) {
              which <- tileGenome(seqlengths(param@bamTallyParam@genome),
                                  bpnworkers(BPPARAM))
            }
            if (is(which,"GenomicRanges")) {
 		if (length(which) == 1L) {
                    which <- tile(which, n=min(width(which),
                                               bpnworkers(BPPARAM)))[[1L]]
             	}
                which <- as(which, "GRangesList")
             }
            ans <- bplapply(which, tally_region_job, x = x, param = param,
                            BPPARAM = BPPARAM)
            do.call(c, unname(ans))
          })

setMethod("tallyVariants", "BamFileList", function(x, ...) {
  stackSamples(VRangesList(bplapply(x, tallyVariants, ...)))
})

setMethod("tallyVariants", "character", function(x, ...) {
  tallyVariants(BamFile(x), ...)
})

.valid_TallyVariantsParam <- function(object) {
    c(if (!is(object@bamTallyParam, "BamTallyParam"))
          "@bamTallyParam must be a BamTallyParam",
      if (anyNA(object@read_pos_breaks))
          "@read_pos_breaks must not contain NAs",
      if (!isTRUEorFALSE(object@keep_extra_stats))
          "@keep_extra_stats must be TRUE or FALSE",
      if (!isSingleNumberOrNA(object@read_length))
          "@read_length must be a single number or NA",
      if (!isSingleNumberOrNA(object@high_nm_score))
          "@high_nm_score must be a single number or NA")
}

setClassUnion("integerORNULL", c("integer", "NULL"))

setClass("TallyVariantsParam",
         representation(bamTallyParam = "ANY",
                        read_pos_breaks = "integerORNULL",
                        mask = "GenomicRanges",
                        keep_extra_stats = "logical",
                        read_length = "integer",
                        high_nm_score = "integer"),
         validity=.valid_TallyVariantsParam)

TallyVariantsParam <- function(genome,
                               read_pos_breaks = NULL,
                               high_base_quality = 0L,
                               minimum_mapq = 13L,
                               variant_strand = 1L,
                               ignore_query_Ns = TRUE,
                               ignore_duplicates = TRUE,
                               mask = GRanges(),
                               keep_extra_stats = TRUE,
                               read_length = NA_integer_,
                               read_pos = !is.null(read_pos_breaks),
                               high_nm_score = NA_integer_,
                               ...)
{
  if (!isSingleNumber(variant_strand) || !(variant_strand %in% c(0, 1, 2)))
    stop("'variant_strand' must be either 0, 1, or 2")
  if (!isTRUE(ignore_query_Ns))
      stop("'ignore_query_Ns' must be TRUE")
  bam.tally.args <- list(genome = genome,
                         variant_strand = variant_strand,
                         ignore_query_Ns = ignore_query_Ns,
                         minimum_mapq = minimum_mapq,
                         ignore_duplicates = ignore_duplicates,
                         min_base_quality = high_base_quality,
                         read_pos = read_pos,
                         nm = !is.na(high_nm_score),
                         ...)
  if (!requireNamespace("gmapR"))
      stop("tallying requires the gmapR package")
  bam.tally.param <- do.call(gmapR::BamTallyParam, bam.tally.args)
  new("TallyVariantsParam", bamTallyParam = bam.tally.param,
      read_pos_breaks = as.integer(read_pos_breaks),
      mask = mask, keep_extra_stats = as.logical(keep_extra_stats),
      read_length = read_length, high_nm_score = high_nm_score)
}

setMethod("show", "TallyVariantsParam", function(object) {
  cat("A", class(object), "object\n", sep = " ")
  cat(gmapR:::showSlots(object@bamTallyParam, count = FALSE), sep = "")
  cat(gmapR:::showSlots(object, exclude = "bamTallyParam", count = FALSE),
      sep = "")
})

library(VariantTools)
library(CGPtools)
library(parallel)
library(TxDb.Hsapiens.BioMart.igis)
library(org.Hs.eg.db)


txdb <- TxDb.Hsapiens.BioMart.igis
genes <- cdsBy(txdb, "gene")

x <- org.Hs.egSYMBOL2EG
mapped_genes <- mappedkeys(x)
xx <- as.list(x[mapped_genes])


##get_sams
meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/release_2.1/released.json")
colon <- c('Colon', 'Colon Sigmoid', 'Rectum', 'Cecum', 'Ileum', 'Intestine Small')
exome <- meta[meta$project == "exome",]
normals <- as.character(exome$srcid[exome$tissue_diagnosis %in% c("Normal", "Normal Adjacent Tumor")])
tissues <- as.character(exome$primary_tissue)
names(tissues) <- as.character(exome$srcid)
tissues[normals]<-"normal"
tissues[tissues %in% colon]<-"Colon"
colon_sams <- names(tissues[tissues=="Colon"])
colon_sams <- as.list(colon_sams)
##
gene <- "TP53"
gene_id <- unlist(mget(gene, org.Hs.egSYMBOL2EG))
gene_gr <- genes[[paste("GeneID:", gene_id,sep = "")]]

list <- mclapply(colon_sams, mc.cores=detectCores(), function(x){freq_info(x, gene_gr)})
x <- do.call(c, list)
pdf(file = "TP53_EXvR_freq.pdf", height=8, width=8)
plot(values(x)$count/values(x)$count.total, values(x)$norm_alt/values(x)$norm_cov,
     xlim = c(0,1), ylim = c(0,1), pch = 19, xlab = "Alt read frequency Exome",
     ylab = "Alt read frequency RNA", main =gene )
abline(0,1)
dev.off()
freq_info <- function(sam, gene_gr){
  f1 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/",
              sam,
              "/",
              sam,
              ".sample_specific_variants_granges.RData", sep = "")
  f2 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/RNA/",
              sam,
              "/results/",
              sam,
              ".variants_granges.RData", sep = "")
  f3 <- paste("/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS115/",
              sam,
              "/results/",
              sam
              ,".coverage.RData", sep = "")
  if(file.exists(f1) & file.exists(f2) &file.exists(f3)){
    gr <- get(load(f1))
    raw<- get(load(f2))
    cov <- get(load(f3))
    gr_in_gene <- findOverlaps(gr, gene_gr)
    gr <- gr[queryHits(gr_in_gene)]
    if(length(gr)==0){
      return(GRanges())
    }
    gr<- sort(gr)
    raw<-sort(raw)
    values(gr)$norm_ref <- as.numeric(NA)
    values(gr)$norm_alt <- as.numeric(0)
    over2 <- which(paste(values(gr)$location,values(gr)$read) %in% paste(values(raw)$location, values(raw)$read))
    over <- which(paste(values(raw)$location,values(raw)$read) %in% paste(values(gr)$location, values(gr)$read))
    if(length(over)==0 |length(over2) ==0){
      return(GRanges())
    }
    values(gr[over2])$norm_ref <- values(raw[over])$count.ref
    values(gr[over2])$norm_alt <- values(raw[over])$count
    ##    values(gr[-over2])$norm_alt <- 0
    
    grl <- split(gr, as.factor(seqnames(gr)))
    names <- as.list(names(grl))
    n <- lapply(names, function(x) { as.numeric(cov[[x]][ranges(grl[[x]])]) })
    n <- unlist(n)
    gr <- unlist(grl)
    values(gr)$norm_cov<-n 
    return(gr)
  }else{
    return(GRanges())   
  }
}

calculateVariantConcordance <- function(gr1, gr2, which = NULL) {
  filters <- VariantConcordanceFilters()
  
  gr1 <- subsetByFilter(gr1, filters)
  gr2 <- subsetByFilter(gr2, filters)
  
  if(!is.null(which)){
    gr1 <- subsetByOverlaps(gr1, which)
    gr2 <- subsetByOverlaps(gr2, which)
  }
  
  overlaps <- findOverlaps(gr1, gr2)
  test1 <-gr1[queryHits(overlaps)]
  test2 <-gr2[subjectHits(overlaps)]

  callHetHom <- function(x) {
    alt <- alt(x)
    ifelse(refDepth(x) > 2,
           paste0(as.character(ref(x)), as.character(alt)),
           paste0(as.character(alt), as.character(alt)))
  }

  t1geno <- callHetHom(test1)
  t2geno <- callHetHom(test2)
  
  fraction <- sum(t1geno == t2geno) / length(test1)

  DataFrame(fraction = fraction, denominator = length(test1))
}

SingleVariantFilter <- function() {
  function(x) {
    !duplicated(granges(x))
  }
}

VariantConcordanceFilters <- function() {
  FilterRules(c(NonN = NonNRefFilter(),
                minCount = MinAltDepthFilter(10L),
                singleVariant = SingleVariantFilter(),
                onlySNV = function(x) isSNV(x)))
}

library(CGPtools)
library(TxDb.Hsapiens.BioMart.igis)
library(parallel)
library(VariantTools)
source("~/svn/Packages/VariantTools/scripts/functional_vep.R")
txdb <- TxDb.Hsapiens.BioMart.igis 

colnames <- c("Uploaded_variation",
              "Location",
              "Allele", "Gene", "Feature",
              "Feature_type", "Consequence", "DNA_position",
              "CDS_position", "Protein_position",
              "Amino_acids", "Codons", "Existing_variation", "Extra")

dir <-"/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/varEffOut/"
files <- dir(dir)
files <- files[grepl(pattern=".txt", files)]
file_sam <- sub("_variant_annotations.txt", "", files)

cov_dir <- "/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/"


meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/release_2.1/released.json")
colon <- c('Colon', 'Colon Sigmoid', 'Rectum', 'Cecum', 'Ileum', 'Intestine Small')
exome <- meta[meta$project == "exome",]
normals <- as.character(exome$srcid[exome$tissue_diagnosis %in% c("Normal", "Normal Adjacent Tumor")])
tissues <- as.character(exome$primary_tissue)
names(tissues) <- as.character(exome$srcid)
tissues[normals]<-"normal"
tissues[tissues %in% colon]<-"Colon"
colon_sams <- names(tissues[tissues=="Colon"])
colon_sams <- as.list(colon_sams)

run_anno <- function(sam){
  file <- files[which(file_sam ==sam)]
  if(length(file) != 0){
    tab <- read.table(file.path(dir, file), sep = "\t", stringsAsFactors=FALSE, colClasses="character")
    colnames(tab)<-colnames
  }else{
    return(NULL)
  }
  vep_gr <- VEP2gr(tab)
  vep_gr <- getFunctional(vep_gr)
  cov_file <- paste(cov_dir, sam, "/results/", sam, ".coverage.RData", sep = "")
  if(file.exists(cov_file)){
    cov <- get(load(cov_file))
  }else{
    return(NULL)
  }
  anno <- annotateTrans(txdb, cov, vep_gr)
  return(anno)
}

all_anno <- lapply(colon_sams, function(sam){run_anno(sam)})
names <- names(all_anno[[1]])
names(all_anno) <- unlist(colon_sams)
mat <- matrix(ncol=73, nrow = 42969, NA)
colnames(mat) <- unlist(colon_sams)
rownames(mat) <- names
for(i in 1:73){
  if(!is.null(all_anno[[i]])){
    x <- values(all_anno[[i]])
    v1 <-ifelse(x$prot_alt_count>0, yes=1, no=NA)
    mat[,i] <- ifelse(x$callable_fraction>.8 & x$prot_alt_count==0, yes = 0, no=v1)
  }else{
  }
}

out_dir <- "/gne/research/workspace/degenhj2/run_cgp3_ex/"
dir <- "/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/"
sams <- dir(dir)

header <- "#!/bin/bash\nsource /gne/research/apps/modules/common/bashrc\nmodule load apps/R\n"
run <- "Rscript ~/svn/VariantTools/scripts/run_callVar_wrapper.R"
genome_info <-"hg19_IGIS21 /gne/research/apps/ngs_pipeline/dev/x86_64-linux-2.6-sles11/share 75"
out <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/"
for(i in 1:length(sams)){
  if(file.exists(paste(dir, sams[i], "/bams/", sams[i], ".analyzed.bam", sep = ""))){
    bam_file <- paste(dir, sams[i], "/bams/", sams[i], ".analyzed.bam", sep = "")
    con <-file.path(out_dir, paste(sams[i], "_run.sh", sep = ""))
    writeLines(header, con = con)
    cat(paste(run, bam_file, genome_info, paste(out, sams[i], sep = ""), sams[i], "cgp_rerun\n"), append=T, file= con)
  }
  
}

out_dir <- "/gne/research/workspace/degenhj2/run_cgp3_rna/"
dir <- "/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS115/"
sams <- dir(dir)

header <- "#!/bin/bash\nsource /gne/research/apps/modules/common/bashrc\nmodule load apps/R\n"
run <- "Rscript ~/svn/VariantTools/scripts/run_callVar_wrapper.R"
genome_info <-"hg19_IGIS21 /gne/research/apps/ngs_pipeline/dev/x86_64-linux-2.6-sles11/share 75"
out <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/RNA/"
for(i in 1:length(sams)){
  if(file.exists(paste(dir, sams[i], "/bams/", sams[i], ".analyzed.bam", sep = ""))){
    bam_file <- paste(dir, sams[i], "/bams/", sams[i], ".analyzed.bam", sep = "")
    con <-file.path(out_dir, paste(sams[i], "_run.sh", sep = ""))
    writeLines(header, con = con)
    cat(paste(run, bam_file, genome_info, paste(out, sams[i], sep = ""), sams[i], "cgp_rerun\n"), append=T, file= con)
  }
  
}


library(CGPtools)
meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/released.json")
meta <- meta[which(!is.na(meta$patient_num)),]
exome <- meta[meta$project=="exome",]
pat_ids <- unique(exome$patient_num)

header <- "#!/bin/bash\nsource /gne/research/apps/modules/common/bashrc\nmodule load apps/ngs_pipeline/3.4\nR_LIBS=/gne/home/degenhj2/R/x86_64-unknown-linux-gnu-library/2.14/:$R_LIBS\n"
run <- "Rscript ~/svn/VariantTools/scripts/run_TN_wrapper.R"
path <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/Exome/"
out_dir <- "/gne/research/workspace/degenhj2/run_cgp2_ex_TN/"
out <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/"
for(i in 1:length(pat_ids)){
  ids <- as.character(exome[exome$patient_num ==pat_ids[i],"srcid"])

  tgr <- paste(path, ids[1], "/results/", ids[1], ".filtered_variants_granges.RData", sep = "")
  traw <- paste(path, ids[1], "/results/", ids[1], ".variants_granges.RData", sep = "")
  ngr <- paste(path, ids[2], "/results/", ids[2], ".filtered_variants_granges.RData", sep = "")
  nraw <- paste(path, ids[2], "/results/", ids[2], ".variants_granges.RData", sep = "")
  tcov <- paste("/gne/research/data/cgp/2.0/", ids[1], "/exome_merged/RData/", ids[1], ".cov.RData", sep = "")
  ncov <- paste("/gne/research/data/cgp/2.0/", ids[2], "/exome_merged/RData/", ids[2], ".cov.RData", sep = "")
  con1 <-file.path(out_dir, paste(ids[1], "_run_TN.sh", sep = ""))
  con2 <-file.path(out_dir, paste(ids[2], "_run_TN.sh", sep = ""))

  writeLines(header, con = con1)
  cat(paste(run, tgr, ngr, nraw, ncov,paste(out, ids[1], sep = ""), ids[1], "cgp_2.0_TN_rerun\n"), file=con1, append=TRUE)

  writeLines(header, con = con2)
  cat(paste(run, ngr, tgr, traw, tcov,paste(out, ids[2], sep = ""), ids[2], "cgp_2.0_TN_rerun\n"), file=con2, append=TRUE)
}


##########################################
## Running VariantEffectPredictor
#########################################

out_dir <- "/gne/research/workspace/degenhj2/run_cgp3_TN_vareff"
dir <- "/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/"
sams <- dir(dir)
header <- "#!/bin/bash\nsource /gne/research/apps/modules/common/bashrc\nmodule load apps/igis\n"

for(i in 1:length(sams)){
  con = file.path(out_dir, paste(sams[i], "_varEff.sh", sep = ""))
  writeLines(header, con = con)
  cat(paste("zcat ",
            dir, "/",
            sams[i],
            "/",
            sams[i],
            "_sample_specific_variants.vcf.gz |variant_effect_predictor.pl --sift=b --polyphen=b --condel=b --hgnc --check_existing --output_file ",
            paste(dir,"varEffOut/", sep = ""),
            sams[i],
            "_variant_annotations.txt",
            sep = ""),
      file=con, append=TRUE)
              
 }

library(genoset)
library(BigMatrix)
library(CGPtools)
library(VariantTools)
source("~/svn/Packages/VariantTools/scripts/getAlleleByLocus_mod.R")

meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/released.json")
## ds = readGenoSet("/gne/research/data/cgp/SNPArray.V4/illumina_2.5M.genoset.genotypes.and.raw.RData")
ds = readCGPSNPArray(version="Genotype")   # Very recent addition
my.levels = levels(ds[ , , "geno"])

het.levels = c("AC","AG","AT", "CA", "CG", "CT", "GA", "GC", "GT", "TA", "TC", "TG") # or something

locs_gr <- as(locData(ds), "GRanges")
seqlevels(locs_gr) <- paste("chr", seqlevels(locs_gr), sep = "")


sample.44.hets = ds[ , 44, "geno"] %in% het.levels
sample.56.hets = ds[ , 105, "geno"] %in% het.levels

###looking at positions in sample with sufficient coverage

FN <- mclapply(sams, mc.cores =detectCores(), function(sam) check_het(sam))

check_het <- function(SAM, return_gr =FALSE){
  cov <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/", SAM, "/results/",SAM,".coverage.RData", sep = "")))
  
  gr1 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/",
                        SAM,
                        "/results/",
                        SAM,
                        ".filtered_variants_granges.RData", sep = "")))
  seqlevels(gr1) <- paste("chr", seqlevels(gr1), sep = "")
  names(cov) <- paste("chr", names(cov), sep = "")
  names <- seqlevels(locs_gr)
  grl <- split(locs_gr, as.factor(seqnames(locs_gr)))
  n <- lapply(names, function(x) { as.numeric(cov[[x]][ranges(grl[[x]])]) })
  values(locs_gr)$cov <- unlist(n)
  int <- which(pData(ds)$SAMPLE_ID ==sub("SAM", "", SAM))
  sample.hets = ds[ , int, "geno"] %in% het.levels
  het_locs <- locs_gr[sample.hets]
  high_cov_locs <- het_locs[values(het_locs)$cov >15]
  over <- findOverlaps(gr1, high_cov_locs)
  not_found <- high_cov_locs[-subjectHits(over)]
  ret <- 1- (length(not_found)/length(high_cov_locs))
  if(return_gr){
    gr2 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/",
                          SAM,
                          "/results/",
                          SAM,
                          ".variants_granges.RData", sep = "")))
    seqlevels(gr2) <- paste("chr", seqlevels(gr2), sep = "")
   
    cat(ret," ", length(high_cov_locs),"\n", sep = "")
    over <- findOverlaps(gr2, not_found)
    gr3 <- gr2[queryHits(over)]
    var <- variantFilter(gr3,pval=0.001, useQual=TRUE, read_pos= "cycleCount.15.60")
    return(c(ret, length(high_cov_locs),
             length(not_found),
             length(var$raw_granges),
             var$low_count_rejects,
             var$frequency_rejects,
             var$fisher_test_rejects_2by2,
             var$read_pos_rejects))
  }else{
    return(c(ret, length(high_cov_locs)))
  }
}  

check <-getAlleleByLocus_mod(locus = not_found, meta=meta, ids=SAM, tech="exome", type = "raw")

var <- check[[1]][as.character(values(check[[1]])$ref) != as.character(values(check[[1]])$read)]
variantFilter(var)

gr2 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/",
                      SAM,
                      "/results/",
                      SAM,
                      ".variants_granges.RData", sep = "")))    


** Actually it looks like the levels fell off of the PRD genotype
object.  Lovely.  They are

paste( c(rep("A",4),rep("C",4),rep("G",4),rep("T",4)),

##' function to return of list of variant GRanges that overlap a particular gene/locus for a set of ids 
##'
##' This function takes a geneID, list of sample IDs, a metadata table, and returns a list of
##' alleles/variants that overlap a gene/locus or a set of genes/loci from these samples
##' @title 
##' @param gene the Entrez gene id(s) for the gene(s) of intest. if multiple it should be a vector
##' @param locus a GRanges object with the locus (loci) positional information and allele informtion. see help for example
##' @param txdb txdb of all genes
##' @param ids a list of ids for the samples of interest
##' @param meta a metaData table as generated by getCGPMetadata()
##' @param tech a character string denoting the technology ('rna_seq' or 'exome') to use
##' @param type a character string denoting the type of variants to be checked. Can be somatic, called, or raw
##' @param exact only used with the locus option, is a true/false to determine if exact matches to alleles are wanted.
##'              If true the GR must have a elementMetaData called variant with the allele of interest given  
##' @param save_dir directory to save VCF files. If null, no vcf files will be generated
##' @param cores number of cores to run on
##' @return a list of variant GRanges
##' @author Jeremiah Degenhardt
##'
##' 
##' @export
getAlleleByLocus_mod<- function(gene=NULL,
                              locus =NULL,
                              gene_db,
                              ids,
                              meta,
                              tech = "exon",
                              type="somatic",
                              exact=FALSE,
                              save_dir=NULL,
                              cores=detectCores()){
  warning("This function is hard coded with the genome information from the CGP3.0 release...")
  genome_dir = '/gne/research/workspace/degenhj2'
  genome = 'hg19_IGIS21'
  if(tech=="exon"){
    bq = 56
  }else{
    bq=23
  }
  if(!is.null(gene) && !is.null(locus)){
    stop("must supply either a gene ID or locus GRanges but not both")
  }
  if(!is.null(gene)){
    ##genes <- normalizeGeneNames(gene_db)
    gId <- paste("GeneID:", gene, sep = "")
    InTxdb <- gId %in% names(genes)
    if(any(!InTxdb)){
      missing <- gId[which(!InTxdb)]
      gId <- gId[InTxdb] 
      message(missing, " is not in the txdb. \nRemoving this gene from further analysis")
    }
    gene <- genes[gId]
    gene <-unlist(gene)
    gene<-reduce(gene)
  }
  if(!is.null(locus)){
    gene <- locus
  }
  ##gene <- normalizeChrNames(gene)
  if(type =="somatic"){
    post <- ".sample_specific_variants_granges.RData"
  }else{
    post <- ".merged.uniques.bam"
  }
  in_gene <- mclapply(ids, mc.cores=cores, function(sam){
    if(tech == "exome"){
      meta<-meta[meta$project == "exome",]
    }else if(tech =="rna_seq"){
      meta<-meta[meta$project == "rna_seq",]
    }
    ##dir<-meta$results_dir[meta$srcid==sam]
    if(type =="somatic"){
      dir <-"/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/"
    }else{
     dir<-meta$results_dir[meta$srcid==sam]
    }
    if(length(dir)==0){
      message(paste("The SAM ", sam, " does not exist in meta data", sep=""))
      return(GRanges())
    }
    if(type =="somatic"){
      if(file.exists(file.path(dir, sam, paste(sam, post, sep = "")))){
        gr <- load(file.path(dir, sam, paste(sam, post, sep = "")))
        gr <- get(gr)
      }else{
        message(paste("The SAM ", sam ," does not have sample specific variants", sep =""))
        return(GRanges())
      }
    }
    if(type == "called"){
      if(file.exists(file.path(dir,"bams/merged/", paste(sam, post, sep = "")))){
        bam_file <- file.path(dir,"bams/merged/", paste(sam, post, sep = ""))
        gg <- GmapGenome(genome=genome, directory=genome_dir)
        param <- BamTallyParam(which=gene, 
                               high_quality_cutoff = as.integer(bq),
                               minimum_mapq = 13L,
                               min_depth = 0L, variant_strand = 1L,
                               ignore_query_Ns = TRUE,
                               indels = FALSE)
        gr <- bam_tally(bam_file, gg, param)
        ##gr <- tally2GR(bamfiles=bam_file, genome=genome,genome_dir=genome_dir,chr_gr=gene, variant_strand=1, map_qual=13, bqual_thresh=bq)
        gr <-variantFilter(gr, useQual=TRUE)
        gr <- gr$filtered_granges
      }else{
        message(paste("The SAM ", sam, " does not have bam files", sep = ""))
        return(GRanges())
      }
    }
    if(type == "raw"){
      bam_file <- file.path(dir,"bams/merged/", paste(sam, post, sep = ""))
      gg <- GmapGenome(genome=genome, directory=genome_dir)
      param <- BamTallyParam(which=gene,
                             high_quality_cutoff = as.integer(bq),
                             minimum_mapq = 13L,
                             min_depth = 0L, variant_strand = 0L,
                             ignore_query_Ns = TRUE,
                             indels = FALSE)
      gr <- bam_tally(bam_file, gg, param)
      gr <- tally2GR(bamfiles=bam_file, genome=genome,genome_dir=genome_dir,chr_gr=gene, variant_strand=0, map_qual=13, bqual_thresh=bq)
    }

    ##gr <- normalizeChrNames(gr)
    if(exact && !is.null(locus)){
      vec1 <- paste(seqnames(gr), start(gr), values(gr)$read, sep = ":")
      vec2 <- paste(seqnames(gene), start(gene), values(gene)$variant, sep = ":")
      ind <- which(vec1 %in% vec2)
      InGene <- gr[ind]
    }else{
      seqlevels(gene) <- seqlevels(gr)
      seqlengths(gene) <- seqlengths(gr)
      over <- suppressWarnings(findOverlaps(gr, gene))
      InGene <- gr[unique(queryHits(over))]
      if(!is.null(save_dir)){
        InGene <- sort(InGene)
        cgpGr2vcf(GR=InGene, filename=paste(save_dir,"/", sam, ".vcf", sep = ""), project="Mutations_by_gene", sample_id=sam)
      }
    }
    return(InGene)
  })
  names(in_gene) <- as.character(ids)
  return(in_gene)
}

fisher_p <- function( x, y, m, n, relErr = 1 + 1e-7 ) {
   # 'x' and 'y' are entries in the top two cells; 'm' and 'n' are
   # column totals.  Code is excerpted from fisher.test, for
   # efficiency. Note that 'support' varies in length with the input
   # variables, so vectorization is only possible via an mapply.
   mapply(
          function( x, y, m, n ) {
            k <- x + y
            lo <- max( 0, k - n )
            hi <- min( k, m )
            support <- lo:hi
            d <- dhyper( support, m, n, k, log = TRUE )
            d <- exp( d - max( d ) )
            d <- d / sum( d )
            sum( d[ d <= d[ x - lo + 1 ] * relErr ] )
          },
          x, y, m, n
          )
 }

fisher_p_vectorized <- function(x, y, m, n, relErr = 1 + 1e-7) {
  ## 'x' and 'y' are entries in the top two cells; 'm' and 'n' are
  ## column totals. Vectorized version of Richard's fisher_p().
  k <- x + y
  lo <- pmax(0L, k - n)
  hi <- pmin(k, m)
  support <- IRanges:::unlist_as_integer(IRanges(lo, hi))
  w <- hi - lo + 1L
  m <- rep(m, w)
  n <- rep(n, w)
  k <- rep(k, w)
  d <- dhyper(support, m, n, k, log = TRUE)
  part <- PartitioningByWidth(w)
  dl <- relist(d, part)  
  maxd <- max(dl)
  dl <- exp(dl - maxd) / sum(dl)
  sel <- dl <= dl[as(x - lo + 1L, "IntegerList")] * relErr
  unname(sum(dl[sel]))
}

genome_gr <- getChrGRangesFromGmap(genome = genome, genome_dir = genome_dir)
chr_ids <- as.character(seqnames(genome_gr))
callVariantsP<-function(merged_bam,
                        read_length=-1,
                        genome=NULL,
                        genome_dir=NULL,
                        with_qual=FALSE,
                        bqual = 0,
                        mapq=0,
                        mc.cores = detectCores()){
  
  gr_list <- mclapply(chr_ids, mc.cores=mc.cores, function(x){
    chr_tally_gr <-tally2GR(bamfiles= merged_bam,
                            genome =genome,
                            genome_dir=genome_dir,
                            regions = genome_gr,
                            breaks=NULL,
                            bqual_thresh=56,
                            map_qual=13,
                            mc.cores = 1,
                            chr_ids=x)
    
    seqlevels(chr_tally_gr) =seqlevels(genome_gr)
    seqlengths(chr_tally_gr) =seqlengths(genome_gr)
    
    chr_variants_filtered <- variantFilter(granges=chr_tally_gr, useQual=TRUE)
  })
  tally_gr <- Reduce(f=merge_var, gr_list)
  return(tally_gr)
}
merge_var <- function(a, b){
  res <- list()
  res$raw_granges<- c(a$raw_granges, b$raw_granges)
  res$filtered_granges <- c(a$filtered_granges, b$filtered_granges)
  res$N_nucleotide_rejects <- sum( a$N_nucleotide_rejects, b$N_nucleotide_rejects, na.rm=T)
  res$low_count_rejects<-sum(a$low_count_rejects, b$low_count_rejects, na.rm=T)
  res$frequency_rejects<- sum(a$frequency_rejects, b$frequency_rejects, na.rm=T)
  res$fisher_test_rejects_2by2<-sum(a$fisher_test_rejects_2by2, b$fisher_test_rejects_2by2,na.rm=T)
  res$fisher_test_rejects_3by2<-sum(a$fisher_test_rejects_3by2, b$fisher_test_rejects_3by2, na.rm=T)
  return(res)
}

merged_bam <- "/gne/home/degenhj2/svn/Packages/VariantTools/tests/merged/SRC111111.TESTPREPENDSTR.merged.uniques.bam"
genome = "hg19"
genome_dir <- "/gnet/is2/data/bioinfo/gmap/data/genomes/"
genome_gr <- getChrGRangesFromGmap(genome = genome, genome_dir = genome_dir)
chr_ids <- as.character(seqnames(genome_gr))

library(genoset)
library(BigMatrix)
library(CGPtools)
library(VariantTools)
library(BSgenome.Hsapiens.UCSC.hg19)
source("~/svn/Packages/VariantTools/scripts/getAlleleByLocus_mod.R")

meta<-getCGPMetadata(JSON="http://research.gene.com/cgp/released.json")
## ds = readGenoSet("/gne/research/data/cgp/SNPArray.V4/illumina_2.5M.genoset.genotypes.and.raw.RData")
ds = readCGPSNPArray(version="Genotype")   # Very recent addition
my.levels = levels(ds[ , , "geno"])

het.levels = c("AC","AG","AT", "CA", "CG", "CT", "GA", "GC", "GT", "TA", "TC", "TG") # or something

locs_gr <- as(locData(ds), "GRanges")
seqlevels(locs_gr) <- paste("chr", seqlevels(locs_gr), sep = "")  
locs_gr <- locs_gr[!duplicated(paste(seqnames(locs_gr), start(locs_gr), sep = ""))]

plot_SNPS("SAM587284")

plot_SNPS<- function(SAM){
locs_gr <- as(locData(ds), "GRanges")
seqlevels(locs_gr) <- paste("chr", seqlevels(locs_gr), sep = "")
cov <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/", SAM, "/results/",SAM,".coverage.RData", sep = "")))

gr1 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/",
                      SAM,
                      "/results/",
                      SAM,
                      ".variants_granges.RData", sep = "")))
seqlevels(gr1) <- paste("chr", seqlevels(gr1), sep = "")
names(cov) <- paste("chr", names(cov), sep = "")
names <- seqlevels(locs_gr)
grl <- split(locs_gr, as.factor(seqnames(locs_gr)))
n <- lapply(names, function(x) { as.numeric(cov[[x]][ranges(grl[[x]])]) })
values(locs_gr)$cov <- unlist(n)
int <- which(pData(ds)$SAMPLE_ID ==sub("SAM", "", SAM))
geno <- ds[ , int, "geno"]
values(locs_gr)$geno <-geno
values(locs_gr)$count <-0
over <- findOverlaps(gr1, locs_gr)
values(locs_gr[subjectHits(over)])$count <- values(gr1[queryHits(over)])$count
values(locs_gr)$alt <- NA
values(locs_gr[subjectHits(over)])$alt <- as.character(values(gr1[queryHits(over)])$read)
seq <- getSeq(Hsapiens, locs_gr)
values(locs_gr)$ref<-as.character(seq)
##values(locs_gr[subjectHits(over)])$ref <- as.character(values(gr1[queryHits(over)])$ref)

values(locs_gr)$isHet <- as.character(values(locs_gr)$geno)%in% het.levels
values(locs_gr)$homRef <- (values(locs_gr)$geno ==paste(values(locs_gr)$ref, values(locs_gr)$ref, sep=""))
gr2 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/",
                      SAM,
                       "/results/",
                       SAM,
                       ".filtered_variants_granges.RData", sep = "")))
seqlevels(gr2) <- paste("chr", seqlevels(gr2), sep = "")

with_cov <- locs_gr[values(locs_gr)$cov>15]
SNPS <- file.path(system.file(package = "VariantTools", mustWork=TRUE),"extdata/exon_snps_ill25.RData")
SNPs <- get(load(SNPS))
over <- findOverlaps(SNPs, with_cov)
with_cov[subjectHits(over)]->ex

het <- ex[values(ex)$isHet]
hom <- ex[!values(ex)$isHet]
hom_ref <- hom[which(values(hom)$homRef)]
hom_alt <-hom[which(!values(hom)$homRef)]

over1 <- findOverlaps(het, gr2)
called_het <- het[queryHits(over1)]
not_called_het <- het[-queryHits(over1)]
over2 <- findOverlaps(hom_ref, gr2)
called_hom_ref <- hom_ref[queryHits(over2)]
not_called_hom_ref <- hom_ref[-queryHits(over2)]
over3 <- findOverlaps(hom_alt, gr2)
called_hom_alt <- hom_alt[queryHits(over3)]
not_called_hom_alt <- hom_alt[-queryHits(over3)] 

par(mfrow = c(1,3))
plot(log10(values(het)$count), log10(values(het)$cov), xlab = "log10 alt read count", ylab = "log10 total count", main = "Heterozygous SNPs by array", pch = NA)
points(log10(values(called_het)$count), log10(values(called_het)$cov), col = "blue", pch = 19)
points(log10(values(not_called_het)$count), log10(values(not_called_het)$cov), col = "red", pch = 19, cex = 0.4)
abline(0,1)
abline(.301,1)
plot(log10(values(hom_alt)$count), log10(values(hom_alt)$cov), xlab = "log10 alt read count", ylab = "log10 total count", main = "Homozygous Alt SNPs by array", pch = NA)
points(log10(values(called_hom_alt)$count), log10(values(called_hom_alt)$cov), col = "blue", pch = 19)
points(log10(values(not_called_hom_alt)$count), log10(values(not_called_hom_alt)$cov), col = "red", pch = 19, cex = 0.4)
abline(0,1)
abline(.301,1) 
plot(log10(values(hom_ref)$count), log10(values(hom_ref)$cov), xlab = "log10 alt read count", ylab = "log10 total count", main = "Homozygous Ref SNPs by array", pch = NA)
points(log10(values(called_hom_ref)$count), log10(values(called_hom_ref)$cov), col = "blue", pch = 19)
points(log10(values(not_called_hom_ref)$count), log10(values(not_called_hom_ref)$cov), col = "red", pch = 19, cex = 0.4)
abline(0,1)
abline(.301,1)
legend(x=2.5, y = 3.5, col = c("blue", "red"), legend=c("Called SNV", "No call"), pch = 19)
}

### =========================================================================
### Post filtering of variants (come after the calling filters)
### -------------------------------------------------------------------------

postFilterVariants <- function(x, post.filters = VariantPostFilters(...), ...) {
  if (!missing(post.filters) && length(list(...)) > 0L)
    stop("arguments in '...' were ignored, because 'post.filters' was given")
  subsetByFilter(x, post.filters)
}

VariantPostFilters <- function(max.nbor.count = 0.1, whitelist = NULL) {
  FilterRules(list(avgNborCount = AverageNeighborCountFilter(max.nbor.count,
                     whitelist = whitelist)))
}

twoWayTabulate <- function(x, y, x.nbins, y.nbins) {
  in.bin <- x > 0L & x <= x.nbins & y > 0L & y <= y.nbins
  new("dgTMatrix", i = x[in.bin] - 1L, j = y[in.bin] - 1L,
      x = rep.int(1, sum(in.bin)), Dim = c(x.nbins, y.nbins))
}

findAverageNeighborCount <- function(x, max.dist = 50L,
                                     weighter = function(d) 1/sqrt(d),
                                     neighbors = x)
{
  windows <- suppressWarnings(resize(neighbors, max.dist * 2L + 1L,
                                     fix = "center"))
  hits <- findOverlaps(x, windows)
  dist <- abs(start(neighbors)[subjectHits(hits)] - start(x)[queryHits(hits)])
  dist.counts <- twoWayTabulate(queryHits(hits), dist,
                                queryLength(hits), max.dist)
  weights <- weighter(seq_len(ncol(dist.counts)))
  norm.weights <- weights / sum(weights)
  suppressMessages(dist.counts %*% matrix(norm.weights))[,1]
}

### FIXME: do we want to consider the alt here, i.e.,
### should whitelist be a VRanges?
AverageNeighborCountFilter <- function(max.nbor.count = 0.1, max.dist = 50L,
                                       weighter = function(d) 1/sqrt(d),
                                       whitelist = NULL)
{
  function(x) {
    if (!is.null(whitelist)) {
      white.listed <- !MaskFilter(whitelist)(x)
      filtered.x <- x[!white.listed]
    } else filtered.x <- x
    count <- findAverageNeighborCount(filtered.x, max.dist, weighter)
    ans <- count <= max.nbor.count
    if (!is.null(whitelist)) {
      ans.padded <- rep.int(TRUE, length(x))
      ans.padded[!white.listed] <- ans
      ans.padded
    } else ans
  }
}

gsnapChromosomes <- function(genome, genome_dir) {
  read.table(pipe(paste("get-genome -L -d", genome, "-D", genome_dir)))[,1]
}


tally2GR<- function(bamfiles,
                    genome = "hg19_ucsc",
                    genome_dir = "/gnet/is2/data/bioinfo/gmap/data/genomes",
                    chr_ids = gsnapChromosomes(genome, genome_dir),
                    regions,
                    variant_strand=1,
                    min_cov =1,
                    breaks=NULL,
                    bqual_thresh=56,
                    map_qual=13)
{
  chr_ids <- as.list(as.character(chr_ids))
  has_regions <- !missing(regions)
  list_of_gr <- mclapply(chr_ids, function(chr_name){
    tally <- pipe(paste("bam_tally -B 0 -C -Q -q", map_qual, " -X ", variant_strand, " -n ",
                        min_cov, " -T -d", genome, "-D", genome_dir, bamfiles,
                        paste("'", chr_name, ":' 2> /dev/null", sep = "")))
    tab <- read.table(tally, colClasses = c("character", "integer", "integer",
                               "character"), sep = "\t",
                      col.names = c("chrom", "position", "count", "cycles"))
    if(dim(tab)[1] <1){
      gr <- GRanges()
    } else {      
      counts <- strsplit(tab[[4]], " ", fixed=TRUE)
      
      counts_part <- PartitioningByWidth(elementNROWS(counts))
      
      chr <- rep(tab[[1]], width(counts_part))
      pos <- rep(tab[[2]], width(counts_part))
      count.total <- rep(tab[[3]], width(counts_part))
      ## we only get duplicated locations if there is overlap across strands
      ## thus, we force the duplicated locations onto different strands
      
      location <- paste(tab[[1]], tab[[2]], sep = ":")
      location_dup <- duplicated(location)
      strand <- rep("*", length(location))
      
      strand[match(location[location_dup], location)] <- "+"
      strand[location_dup] <- "-"
      strand <- rep(strand, width(counts_part))
      location <- rep(location, width(counts_part))
      counts_flat <- unlist(counts, use.names=FALSE)
      bases <- sub("(\\D).*", "\\1", counts_flat)
      ref_rows <- start(counts_part)
      ref <- rep(bases[ref_rows], width(counts_part))
      zero_count <- !grepl("(", counts_flat, fixed=TRUE)
### FIXME: obscene hack to simplify code
      counts_flat[zero_count] <- paste(counts_flat[zero_count], "(0@NA)", sep="")
      cycles <- strsplit(sub(".*\\((.*?)\\).*", "\\1", counts_flat), ",", 
                         fixed=TRUE)
      ncycles <- elementNROWS(cycles)
      cycles_flat <- unlist(cycles, use.names=FALSE)
      cycles_counts <- unlist(strsplit(cycles_flat, "@", fixed=TRUE), 
                              use.names=FALSE)
      cycles_mat <- matrix(suppressWarnings(as.integer(cycles_counts)), nrow=2)
      ## insane code for counting up reads by strand
      neg_cycle <- cycles_mat[2,] < 0
      coord_ind <- rep(seq_along(cycles), ncycles)
      strand_key <- paste(coord_ind, neg_cycle)
      strand_key_uniq <- !duplicated(strand_key)
      strand_key <- factor(strand_key, strand_key[strand_key_uniq])
      neg_cycle_uniq <- neg_cycle[strand_key_uniq]
      coord_ind_uniq <- coord_ind[strand_key_uniq]
      strand_count <- rowsum(cycles_mat[1,], strand_key)
      count.pos <- count.neg <- rep(0L, length(cycles))
      ## have to use which() here to drop the NA values
      count.pos[coord_ind_uniq[which(!neg_cycle_uniq)]] <-
        strand_count[which(!neg_cycle_uniq)]
      count.neg[coord_ind_uniq[which(neg_cycle_uniq)]] <-
        strand_count[which(neg_cycle_uniq)]
      message("working on breaks") 
      if (!is.null(breaks)) {
### NOTE: overuse of rep() here might lead to over-long vectors            
        cycle_bins <- cut(rep(abs(cycles_mat[2,]), cycles_mat[1,]), breaks)      
        cycle_i <- factor(rep(rep(seq(length(cycles)), ncycles), cycles_mat[1,]),
                          seq(length(cycles)))
        cycle_tab <- table(cycle_i, cycle_bins)
        colnames(cycle_tab) <- paste(head(breaks, -1), tail(breaks, -1),
                                     sep = ".")
        ref_cycle_tab <- cycle_tab[rep(ref_rows, width(counts_part)),]
        colnames(ref_cycle_tab) <- paste(colnames(cycle_tab), "ref", sep = ".")
        cycle_tab <- cbind(cycle_tab, ref_cycle_tab)
      } else  cycle_tab <- matrix(nrow = length(pos), ncol = 0)
      message("completed breaks")
      base_counts <- as.integer(sub("\\D(\\d+).*", "\\1", counts_flat))
      ncycles[zero_count] <- 0L # otherwise 1 due to our obscene hack above
      ncycles.ref <- rep(ncycles[ref_rows], width(counts_part))
      count.ref <- rep(base_counts[ref_rows], width(counts_part))
      count.pos[zero_count] <- 0L
      count.pos.ref <- rep(count.pos[ref_rows], width(counts_part))
      count.neg[zero_count] <- 0L
      count.neg.ref <- rep(count.neg[ref_rows], width(counts_part))
      quals <- strsplit(sub(".*\\)\\((.*?)\\).*", "\\1", counts_flat), ",", 
                        fixed=TRUE)
      nquals <- elementNROWS(quals)

      count_above_thresh <- lapply(quals, function(x){
        mat <-matrix(unlist(strsplit(x, "Q", fixed=TRUE), 
                      use.names=FALSE), nrow=2)
        logical <- (as.numeric(mat[2,]))>bqual_thresh
        sum(as.numeric(mat[1,logical]), na.rm=T)
      })
      high_qual <- unlist(count_above_thresh)      
      gr <- GRanges(chr, IRanges(pos, width=1L), strand, location, 
                    ref = DNAStringSet(ref), read = DNAStringSet(bases),
                    ncycles, ncycles.ref,
                    count = base_counts, count.ref, count.total, high.quality = high_qual,
                    count.pos, count.pos.ref, count.neg, count.neg.ref,
                    cycleCount = unclass(cycle_tab))
      
      ##adding ref counts back in as counts per break segment
      ##there seems to be a bug somewhere above that is causing the ref positions to be listed twice some times. Removing these for now
      gr <- gr[as.character(values(gr)$ref) != as.character(values(gr)$read)]

      if (has_regions) {
        region_ol <- findOverlaps(gr, regions)
        region_strand <- as.vector(strand(regions))[subjectHits(region_ol)]
        strand(gr) <- region_strand
        rc <- region_strand == "-"
        values(gr)$ref[rc] <- reverseComplement(values(gr)$ref[rc])
        values(gr)$read[rc] <- reverseComplement(values(gr)$read[rc])
      }
      values(gr)$location <- paste(values(gr)$location, strand(gr), sep = ":")
    }
    message(paste("finished chr ", chr_name))
    seqlevels(gr) <- unlist(chr_ids)
    gr
  })
  GR_full <- do.call(c, list_of_gr)
  return(GR_full)
}

getFunctional<- function(vep_gr){
  vep_gr <- vep_gr[which(grepl(pattern = "GeneID", values(vep_gr)$Gene))]
  ns_inds <- which(grepl(pattern="NON_SYNONYMOUS_CODING",  values(vep_gr)$Consequence))
  stop_inds <- which(grepl(pattern="STOP_GAINED",  values(vep_gr)$Consequence))
  inds <- c(ns_inds, stop_inds)
  vep_gr <- vep_gr[inds]
  return(vep_gr)
}

##  Function to filter nextgen seq data to detect variants
##
## The function first checks and either performs Fisher's Exact Test(s) and appends the pvalues onto the granges,
## and saves the granges, or it saves the raw granges and removes  strands with 'N' nucleotide variant or reference
## occurences and strands with lower than 2 counts per variant, depending on the input parameter. It next filters
## out all granges with Fisher's Test pvals for either test less than the designated parameter. It then filters out all
## GRange data entries that have lower coverage as well as lower variant occurence frequencies than those
## designated by the parameters. Finally, if the parameter dictates that the function should not append pvals,
## it finally performs 2x2 and 3x2 Fisher's Exact Tests tofilter for independence between ref/variant strands and
## between positive and negative strands. 

variantFilter <- function(granges,
                          useQual=FALSE,
                          pval = 0.05,
                          lrt = FALSE,
                          lrt.p = 0.01,
                          lr=TRUE,
                          lower = 0.2,
                          error=1/1000,
                          fet_3x2 = FALSE,
                          fet_2x2 = TRUE,
                          append = FALSE) {
############################################################################

  output_list_names <- c("raw_granges", "filtered_granges", "N_nucleotide_rejects",
                         "low_count_rejects", "frequency_rejects", "fisher_test_rejects_2by2",
                         "fisher_test_rejects_3by2")
  output <- list(GRanges(), GRanges(), 0, 0,0,0,0)
  names(output) <- output_list_names
  
  if(length(granges) == 0){
    ##message("no variants found!")
    output <- list(granges, granges, 0, 0,0,0,0)
    names(output) <- output_list_names
    return(output)
  }
  granges <- granges[as.character(values(granges)$ref) != as.character(values(granges)$read)]
  if (append == TRUE) {
    ##saving Fisher's Test pvalues onto the raw granges
    if(fet_2x2 == TRUE){
   ##   message("Saving Fisher's Tests pvals to granges...") 
      two_by_2_pvals <- list()
      temp_mat1 <- cbind(values(granges)$count.pos.ref,
                         values(granges)$count.pos,
                         values(granges)$count.neg.ref,
                         values(granges)$count.neg)
      x1 <- vector(mode = 'numeric', length = nrow(temp_mat1))
      for(j in seq_len(nrow(temp_mat1))) {
        if (sum(as.numeric(temp_mat1[j,])) < 90000) {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = FALSE), workspace = 1000000)$p.val
        } else {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = FALSE), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      } 
      two_by_2_pvals <- as.list(x1)
      elementMetadata(granges)$fishers2by2.pval <- x1
    } else if (fet_3x2 == TRUE) {
      warning("Hard-coded to expect tally2GR was given three bins. Generalize this")
      three_by_2_pvals <- list()
      temp_mat <- cbind(values(granges)[,16],
                        values(granges)[,13],
                        values(granges)[,17],
                        values(granges)[,14],
                        values(granges)[,18],
                        values(granges)[,15])
      x <- vector(mode = 'numeric', length = nrow(temp_mat))
      for(j in seq_len(nrow(temp_mat))) {
        if (sum(as.numeric(temp_mat[j,])) < 90000) {
          x[j] <- fisher.test(matrix(temp_mat[j,], ncol = 3, byrow = FALSE), workspace = 1000000)$p.val
        } else {
          x[j] <- fisher.test(matrix(temp_mat[j,], ncol = 3, byrow = FALSE), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      three_by_2_pvals <- as.list(x)
      elementMetadata(granges)$fishers3by2.pval <- x
    }
    granges1 <- granges
    output[[1]] <- granges1
    if (length(granges1) == 0)
      return(output)
    
    ##Performing the Fisher's Test Filters
    ##message("Filtering granges by Fishtest pvals...")
    if(fet_2x2 == TRUE){
      two_by_two_filter_gr <- granges[values(granges)$fishers2by2.pval >= pval]
      fishers_2by2_rejects <- length(granges) - length(two_by_two_filter_gr)
    }else{
      two_by_two_filter_gr <- granges
    }
    if(fet_3x2 == TRUE){
      three_by_two_filter_gr <- two_by_two_filter_gr[values(two_by_two_filter_gr)$fishers3by2.pval >= pval]
      fishers_3by2_rejects <- length(two_by_two_filter_gr) - length(three_by_two_filter_gr)
      granges <- three_by_two_filter_gr
    }
  }
   output[['raw_granges']] <- granges
############################################################################
  ##removing N nucleotides and low counts of alternate reads
  Ngranges <- granges[as.character(values(granges)$read) !='N']
  Ngranges <- Ngranges[as.character(values(Ngranges)$ref) !='N'] 
  Nrejects <- length(granges) - length(Ngranges)
  lowcountgranges <- Ngranges[values(Ngranges)$count >= 2]
  lowcountrejects <- length(Ngranges) - length(lowcountgranges)
  ##if no more positions return
  if(lowcountrejects==length(Ngranges)){
    output[[2]]<-lowcountgranges;
     output[[3]] <- Nrejects
    output[[4]]<-lowcountrejects
    return(output)
  }
############################################################################
  if(lrt ==TRUE){
    ##message("LRT on frequency...")
    res <-pbinom((values(lowcountgranges)$count-1),
                 values(lowcountgranges)$count.total,
                 prob=error,
                 lower.tail = FALSE
                 )
    freq_granges <- lowcountgranges[res < lrt.p]
  }
  if(lr ==TRUE){
    freq <- freq(error, lower, 1)
    if(useQual==TRUE){
      lowcountgranges <- lowcountgranges[(values(lowcountgranges)$high.quality+values(lowcountgranges)$high.quality.ref) !=0]
      freq_granges <- lowcountgranges[(values(lowcountgranges)$high.quality/(values(lowcountgranges)$high.quality+values(lowcountgranges)$high.quality.ref)) >= freq]
    }else{
     freq_granges <- lowcountgranges[(values(lowcountgranges)$count/values(lowcountgranges)$count.total) >= freq]
   }
  }
     freqrejects <- length(lowcountgranges) - length(freq_granges)
     if(freqrejects==length(lowcountgranges)){
       output[[2]]<-freq_granges;
       output[[3]] <- Nrejects
       output[[4]]<-lowcountrejects
         output[[5]]<- freqrejects
       return(output)
     }
        if(append == TRUE) {
    output_granges <- freq_granges
  }
############################################################################
  if (!append) {
    ##Performing the Fisher's Test Filters
    if(fet_2x2 == TRUE){
     ## message("Filtering granges by Fishtest pvals...")
      two_by_2_pvals <- list()
      temp_mat1 <- cbind(values(freq_granges)$count.pos.ref,
                         values(freq_granges)$count.pos,
                         values(freq_granges)$count.neg.ref,
                         values(freq_granges)$count.neg)
      x1 <- vector(mode = 'numeric', length = nrow(temp_mat1))
      for(j in seq_len(nrow(temp_mat1))) {
        if (sum(as.numeric(temp_mat1[j,])) < 90000) {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000)$p.val
        } else {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      index <- which(x1 >= pval)
      two_by_two_filter_gr <- freq_granges[index]
      fishers_2by2_rejects <- length(freq_granges) - length(two_by_two_filter_gr)
      if(fishers_2by2_rejects==length(freq_granges)){
        output[[2]]<-lowcountgranges;
        output[[3]] <- Nrejects
        output[[6]]<-lowcountrejects
          output[[5]]<-freqrejects
        output[[6]]<-fishers_2by2_rejects
        return(output)
      }
    }else{
        two_by_two_filter_gr <- freq_granges
    }
    if(fet_3x2 == TRUE){
      warning("Hard-coded to expect tally2GR was given three bins. Generalize this")
      three_by_2_pvals <- list()
      temp_mat <- cbind(values(two_by_two_filter_gr)[,16],
                        values(two_by_two_filter_gr)[,13],
                        values(two_by_two_filter_gr)[,17],
                        values(two_by_two_filter_gr)[,14],
                        values(two_by_two_filter_gr)[,18],
                        values(two_by_two_filter_gr)[,15])
      x <- vector(mode = 'numeric', length = nrow(temp_mat))
      for(i in seq_len(nrow(temp_mat))) {
        if (sum(as.numeric(temp_mat[i,])) < 90000) {
          x[i] <- fisher.test(matrix(temp_mat[i,], ncol = 3, byrow = FALSE), workspace = 1000000)$p.val
        } else {
          x[i] <- fisher.test(matrix(temp_mat[i,], ncol = 3, byrow = FALSE), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      index2 <- which(x >= pval)
      output_granges <- two_by_two_filter_gr[index2]
      fishers_3by2_rejects <- length(two_by_two_filter_gr) - length(output_granges)
    }else{
      output_granges <- two_by_two_filter_gr
    }
  }
############################################################################

  output[['filtered_granges']] <- output_granges
  if (length(output_granges) == 0)
    return(output)    

  ##message("outputing result...")
  output[[3]] <- Nrejects
  output[[4]] <- lowcountrejects
  output[[5]] <- freqrejects
  if (fet_2x2 == TRUE) {
    output[[6]] <- fishers_2by2_rejects
  } else {
    output[[6]] <- 0
  }
  if (fet_3x2 == TRUE) {
    output[[7]] <- fishers_3by2_rejects
  } else {
    output[[7]] <- 0
  }
 ## message("done!")
  return(output)
}


freq <- function(p0,p1, n){
  num <- (log(1-p0)-log(1-p1))
  denom <- (log(p0)-log(p1)+log(1-p1)-log(1-p0))
  rat <- num/denom
  ret <- -n*rat
  return(ret)
}

trans2genome <-function(tx_name, cds_db, pos, type=c("codon", "nuc")){
  strand=suppressWarnings(as.character(unlist(strand(cds_db[tx_name])))[1])
  if(type=="codon"){
    pos=.codon2trans(pos)
  }
  genome_pos <-transcriptLocs2refLocs(list(as.integer(pos)),
                         start(cds_db[tx_name]),
                         end(cds_db[tx_name]),
                         strand=strand
                                      )
  if(type=="codon"){
    if(strand =="-"){
      gr <- GRanges(seqnames=suppressWarnings(as.character(unlist(seqnames(cds_db[tx_name])))[1]),
                    IRanges(start=(unlist(genome_pos)-2), width=3))
    }
    if(strand=="+"){
      gr <- GRanges(seqnames=suppressWarnings(as.character(unlist(seqnames(cds_db[tx_name])))[1]),
                    IRanges(start=(unlist(genome_pos)), width=3))
    }
  }else{
    gr <- GRanges(seqnames=suppressWarnings(as.character(unlist(seqnames(cds_db[tx_name])))[1]),
                  IRanges(start=(unlist(genome_pos)), width=1))
  }
  
  return(gr)
}

.codon2trans<-function(codon){
  pos <- ((codon-1)*3)+1
  return(pos)
}

library(VariantTools)
library(parallel)
bam <- "/gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/SAM587376/bams/SAM587376.analyzed.bam"
genome = "hg19_IGIS21"
genome_dir = "/gne/research/workspace/degenhj2/"
bam = file.path(system.file(package = "VariantTools", mustWork=TRUE),"extdata/merged/SRC111111.TESTPREPENDSTR.merged.uniques.bam")
genome = "hg19_ucsc"
genome_dir = "/gnet/is2/research/data/bioinfo/gmap/data/genomes"
bqual = 23
mapq=13
mc.cores = detectCores()
force_qual=TRUE
read_length=75
bin_fraction=0.2
genome <- GmapGenome(genome=genome, directory=genome_dir)
genome_gr <- as(seqinfo(genome), "GenomicRanges")

chr_inds <-seq_len(length(genome_gr))
qual_type <-detectQualityType(bam, 500)
breaks <- c(0, round(bin_fraction*read_length), round((1-bin_fraction)*read_length), read_length)
read_pos <- paste('cycleCount.', breaks[2], ".", breaks[3], sep ="")
useQual =TRUE
gr_list <- mclapply(chr_inds, mc.cores=mc.cores, function(x){
  chr_info <- genome_gr[x]
  bf <- GmapBamReader(bam)
  chr_tally_gr <-bam_tally(bf,
                           genome,
                           which=chr_info,
                           cycle_breaks= breaks,
                           minimum_mapq=mapq,
                           variant_strand=1,
                           high_quality_cutoff=bqual)
  chr_variants_filtered <- variantFilter(granges=chr_tally_gr, useQual=useQual, read_pos=read_pos)
  
})
var <- VariantTools:::merge_var(gr_list)

setGeneric("callableFraction",
           function(tx, reads, ...) standardGeneric("callableFraction"))

setMethod("callableFraction", c("GRangesList", "ANY"),
          function(tx, reads, ...) {
            callable <- callCallable(reads, ...)
            seqlevels(tx) <- names(cov)
            tx_gr <- unlist(tx, use.names = FALSE)
            tx_ranges <- split(ranges(tx_gr), seqnames(tx_gr))
            exon_sums <- unsplit(viewSums(Views(callable, tx_ranges)),
                                 seqnames(tx_gr))
            tx_sums <- rowsum(exon_sums, names(tx)[togroup(tx)])
            tx_sums / sum(width(tx))
          })

setMethod("callableFraction", c("TxDb", "ANY"),
          function(tx, reads, ...) {
            callGeneric(exonsBy(tx), reads, ...)
          })

##'  Function to filter raw varaint positions found in a bam file
##'
##' The function first checks and either performs Fisher's Exact Test(s) and appends the pvalues onto the granges,
##' and saves the granges, or it saves the raw granges and removes  strands with 'N' nucleotide variant or reference
##' occurences and strands with lower than 2 counts per variant, depending on the input parameter. It next filters
##' out all granges with Fisher's Test pvals for either test less than the designated parameter. It then filters out all
##' GRange data entries that have lower coverage as well as lower variant occurence frequencies than those
##' designated by the parameters. Finally, if the parameter dictates that the function should not append pvals,
##' it finally performs 2x2 and 3x2 Fisher's Exact Tests tofilter for independence between ref/variant strands and
##' between positive and negative strands. 
##' 
##' @title variantFilter
##' @param granges Grange object of variance appropriately formated from tally.R of the ExomeSeq package
##' @param cov_val A coverage threshold
##' @param cov An rle vector denoting the coverage from the bam file from the sequencing lane
##' @param pval The pval threshold for the Fisher's Exact Tests
##' @param freq The frequency threshold for variance
##' @param prepend_str An expression or ID to prepend to the grange file title
##' @param save_dir A directory into which the files will be saved
##' @param src_id The lane's SRC identification
##' @param fet_3x2 A logical parameter dictating whether the function should perform a 3x2 Fisher's Exact Test
##' on the strands.
##' @param fet_2x2 A logical parameter dictating whether the function should perform a 2x2 Fisher's Exact Test
##' on the strands.
##' @param append A logical parameter that dictates whether the function appends the Fisher's Exact Test
##' pvalues onto the granges or not.
##' @return A list containing the raw Granges, filtered GRange data set, the number of N nucleotide rejects, the
##' rejects with fewer than 2 counts, the number of coverage rejects, the number of frequency rejects, the
##' number of rejects from the 2x2 Fisher's, and the number of rejects from the 3x2 Fisher's in that order. Also
##' the vcf data and the raw granges are printed in separate files.
##' @author Jeremiah Degenhardt and Jason Young 
##' @export
variantFilter <- function(granges,
                          pval = 0.05,
                          binom.p = 0.01,
                          fet_3x2 = FALSE,
                          fet_2x2 = TRUE,
                          append = FALSE) {
############################################################################

  if (append  == TRUE) {
    ##saving Fisher's Test pvalues onto the raw granges
    if(fet_2x2 == TRUE){
      message("Saving Fisher's Tests pvals to granges...") 
      two_by_2_pvals <- list()
      temp_mat1 <- cbind(values(granges)$count.pos.ref,
                         values(granges)$count.pos,
                         values(granges)$count.neg.ref,
                         values(granges)$count.neg)
      x1 <- vector(mode = 'numeric', length = dim(temp_mat1)[1])
      for(j in 1:dim(temp_mat1)[1]) {
        if (sum(as.numeric(temp_mat1[j,])) < 90000) {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000)$p.val
        } else {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      } 
      two_by_2_pvals <- as.list(x1)
      elementMetadata(granges)$fishers2by2.pval <- x1
    }else{
    }
    if(fet_3x2 == TRUE){
      warning("Hard-coded to expect tally2GR was given three bins. Generalize this")
      three_by_2_pvals <- list()
      temp_mat <- cbind(
                        values(granges)[,16],
                        values(granges)[,13],
                        values(granges)[,17],
                        values(granges)[,14],
                        values(granges)[,18],
                        values(granges)[,15]
                        )
      x <- vector(mode = 'numeric', length = dim(temp_mat)[1])
      for(j in 1:dim(temp_mat)[1]) {
        if (sum(as.numeric(temp_mat[j,])) < 90000) {
          x[j] <- fisher.test(matrix(temp_mat[j,], ncol = 3, byrow = F), workspace = 1000000)$p.val
        } else {
          x[j] <- fisher.test(matrix(temp_mat[j,], ncol = 3, byrow = F), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      three_by_2_pvals <- as.list(x)
      elementMetadata(granges)$fishers3by2.pval <- x
      ##loginfo("...done")
    }else{
    }
    granges1 <- granges
    ##Performing the Fisher's Test Filters
    message("Filtering granges by Fishtest pvals...")
    if(fet_2x2 == TRUE){
      two_by_two_filter_gr <- granges[values(granges)$fishers2by2.pval >= pval]
      fishers_2by2_rejects <- length(granges) - length(two_by_two_filter_gr)
    }else{
      two_by_two_filter_gr <- granges
    }
    if(fet_3x2 == TRUE){
      three_by_two_filter_gr <- two_by_two_filter_gr[values(two_by_two_filter_gr)$fishers3by2.pval >= pval]
      fishers_3by2_rejects <- length(two_by_two_filter_gr) - length(three_by_two_filter_gr)
      granges <- three_by_two_filter_gr
    }
    ##loginfo("...done") 
  } else {
  
}
############################################################################
  ##removing N nucleotides and low counts of alternate reads
  message("filtering granges for N nucleotides and variant counts less than 2 per variant...")
  Ngranges <- granges[as.character(values(granges)$read) !='N']
  Ngranges <- Ngranges[as.character(values(Ngranges)$ref) !='N']
  Nrejects <- length(granges) - length(Ngranges)
  lowcountgranges <- Ngranges[values(Ngranges)$count >= 2]
  lowcountrejects <- length(Ngranges) - length(lowcountgranges)
  ##loginfo("...done")
############################################################################

### update this version
  message("LRT on frequency...")
   res <-   pbinom(
                   (values(lowcountgranges)$count-1),
                   values(lowcountgranges)$count.total,
                   prob=1/1000,
                   lower.tail = FALSE
                )
  freq_granges <- lowcountgranges[res < binom.p]
  freqrejects <- length(lowcountgranges) - length(freq_granges)
  if(append == TRUE) {
    output_granges <- freq_granges
  }
  ##loginfo("...done")
############################################################################
  if (append != TRUE) {
    ##Performing the Fisher's Test Filters
    if(fet_2x2 == TRUE){
      message("Filtering granges by Fishtest pvals...")
      two_by_2_pvals <- list()
      temp_mat1 <- cbind(values(freq_granges)$count.pos.ref,
                         values(freq_granges)$count.pos,
                         values(freq_granges)$count.neg.ref,
                         values(freq_granges)$count.neg)
      x1 <- vector(mode = 'numeric', length = dim(temp_mat1)[1])
      for(j in 1:dim(temp_mat1)[1]) {
        if (sum(as.numeric(temp_mat1[j,])) < 90000) {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000)$p.val
        } else {
          x1[j] <- fisher.test(matrix(temp_mat1[j,], ncol = 2, byrow = F), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      index <- which(x1 >= pval)
      two_by_two_filter_gr <- freq_granges[index]
      fishers_2by2_rejects <- length(freq_granges) - length(two_by_two_filter_gr)
    }else{
        two_by_two_filter_gr <- freq_granges
    }
    if(fet_3x2 == TRUE){
      warning("Hard-coded to expect tally2GR was given three bins. Generalize this")
      three_by_2_pvals <- list()
      temp_mat <- cbind(
                        values(two_by_two_filter_gr)[,16],
                        values(two_by_two_filter_gr)[,13],
                        values(two_by_two_filter_gr)[,17],
                        values(two_by_two_filter_gr)[,14],
                        values(two_by_two_filter_gr)[,18],
                        values(two_by_two_filter_gr)[,15]
                        )
      x <- vector(mode = 'numeric', length = dim(temp_mat)[1])
      for(i in 1:dim(temp_mat)[1]) {
        if (sum(as.numeric(temp_mat[i,])) < 90000) {
          x[i] <- fisher.test(matrix(temp_mat[i,], ncol = 3, byrow = F), workspace = 1000000)$p.val
        } else {
          x[i] <- fisher.test(matrix(temp_mat[i,], ncol = 3, byrow = F), workspace = 1000000, simulate.p.value = TRUE)$p.val
        }
      }
      index2 <- which(x >= pval)
      output_granges <- two_by_two_filter_gr[index2]
      fishers_3by2_rejects <- length(two_by_two_filter_gr) - length(output_granges)
    }else{
      output_granges <- two_by_two_filter_gr
    }
  }
  ##loginfo("...done")
  message("outputing result...")
  output <- list()
  if (append) {
    output[[1]] <- granges1
  } else {
    output[[1]] <- granges
  }
  output[[2]] <- output_granges
  output[[3]] <- Nrejects
  output[[4]] <- lowcountrejects
  output[[5]] <- freqrejects
  if (fet_2x2 == TRUE) {
    output[[6]] <- fishers_2by2_rejects
  } else {
    output[[6]] <- 0
  }
  if (fet_3x2 == TRUE) {
    output[[7]] <- fishers_3by2_rejects
  } else {
    output[[7]] <- 0
  }
  names(output) <- c("raw_granges", "filtered_granges", "N_nucleotide_rejects", "low_count_rejects", "frequency_rejects", "fisher_test_rejects_2by2", "fisher_test_rejects_3by2")
  return(output)
  #saveWithID(output, "raw_and_filtered_granges", prepend_str, save_dir=file.path(save_dir, "Breaks_RData"), compress=FALSE)
  ##loginfo("...done")
  message("done!")
}

library(VariantTools)
##library(RNASeqGenie)

cmd_args = commandArgs(trailingOnly=TRUE);
tumor_gr <- cmd_args[1];
normal_gr <- cmd_args[2];
normal_raw <- cmd_args[3];
normal_cov <- cmd_args[4];
save_dir <- cmd_args[5];
SAMid <- cmd_args[6];
project<-cmd_args[7];

normal_gr <- load(normal_gr)
normal_gr<-get(normal_gr)
tumor_gr<-load(tumor_gr)
tumor_gr<-get(tumor_gr)
normal_cov<- load(normal_cov)
normal_cov <- get(normal_cov)

if(file.exists(save_dir)){
  stop("The directory already exists")
}else{
  dir.create(dirname(save_dir))
  dir.create(save_dir)
}

TS<- tumorNormalCompare(tumor_gr=tumor_gr,
                        normal_gr=normal_gr,
                        normal_raw=normal_raw,
                        normal_cov=normal_cov)

TS <- sort(TS)

saveWithID(TS, "sample_specific_variants_granges", SAMid,
           save_dir = file.path(save_dir),
           compress = FALSE)

VariantTools::cgpGr2vcf(GR=TS, sample_id=SAMid,
          filename=file.path(save_dir,
            paste(SAMid,"_sample_specific_variants.vcf", sep = "")),
          project=project)

### =========================================================================
### Generating Pileups for Variant Calling
### -------------------------------------------------------------------------

## TODO: add calculation of mean base quality
## TODO: support the BPPARAM -- how? can we pass it to applyPileups?

pileupVariants <- function(bams, genome, param = ApplyPileupsParam(),
                           minAltDepth = 1L, baseOnly = TRUE,
                           BPPARAM = defaultBPPARAM())
{
  pileupFiles <- PileupFiles(bams)
  plpWhat(param) <- union(plpWhat(param), "seq")
  bamNames <- names(bams)
  if (is.null(bamNames))
    bamNames <- seq_len(length(bams))
  pileupFun <- function(x) {
    tally <- x$seq
    sn <- Rle(names(x$seqnames), x$seqnames)
    pos.gr <- GRanges(sn, IRanges(x$pos, width=1))
    seq.gr <- pos.gr
    if (!all(seqlevels(seq.gr) %in% seqlevels(genome))) {
      seqlevelsStyle(seq.gr) <- seqlevelsStyle(genome)
    }
    ref.pos <- as.character(getSeq(genome, seq.gr))
    ref <- rep(ref.pos, each=nrow(tally)*ncol(tally))
    bases <- rep(rownames(tally), ncol(tally) * dim(tally)[3])
    is.ref <- ref == bases
    nalt <- (nrow(tally)-1)
    per.pos <- nalt*ncol(tally)
    per.samp.pos <- rep(nalt, ncol(tally) * dim(tally)[3])
    sampleNames <- rep(Rle(bamNames, rep(nalt, length(bamNames))), dim(tally)[3])
    vr <- VRanges(rep(sn, each=per.pos),
                  IRanges(rep(x$pos, each=per.pos), width=1),
                  alt = bases[!is.ref],
                  ref = rep(ref[is.ref], each=nalt),
                  altDepth = tally[!is.ref],
                  refDepth = Rle(tally[is.ref], per.samp.pos),
                  totalDepth = Rle(colSums(tally), per.samp.pos),
                  sampleNames = sampleNames)
    vr <- vr[altDepth(vr) >= minAltDepth]
    if (baseOnly) {
      vr <- vr[ref(vr) != "N" & alt(vr) != "N"]
    }
    vr
  }
  gr <- do.call(c, applyPileups(pileupFiles, pileupFun, param=param))
  gr
}

### Some unexported work on a converter from a VRanges to a "pileup" GRanges:
pileupGRanges <- function(x) {
  indel <- isIndel(x)
  if (any(indel, na.rm=TRUE)) {
    x <- x[-which(indel)]
  }
  gr <- GRanges(seqnames(x), ranges(x), strand(x))
  sm <- selfmatch(gr)
  uniq <- sm == seq_len(length(gr))
  map <- integer()
  map[sm[uniq]] <- seq_len(sum(uniq))
  pos <- map[sm]
  base <- match(c(alt(x), ref(x)), DNA_BASES)
  depth <- c(altDepth(x), refDepth(x))
  samp <- as.factor(sampleNames(x))
  pileup <- array(0L, c(sum(uniq), length(levels(samp)), length(DNA_BASES)),
                  dimnames=list(NULL, levels(samp), base=DNA_BASES))
  pileup[cbind(pos, samp, base)[!is.na(base),]] <- depth[!is.na(base)]
  ugr <- gr[uniq]
  ugr$ref[pos] <- ref(x)
  ugr$pileup <- pileup
  ugr
}

pileupGRangesOverGenome <- function(x) {
  indel <- isIndel(x)
  if (any(indel, na.rm=TRUE)) {
    x <- x[-which(indel)]
  }
  genome <- unlist(tileGenome(seqinfo(x), tilewidth=1L))
  gr <- GRanges(seqnames(x), ranges(x), "*")
  m <- match(gr, genome)
  base <- match(c(alt(x), ref(x)), DNA_BASES)
  depth <- c(altDepth(x), refDepth(x))
  samp <- as.factor(sampleNames(x))
  pileup <- array(0L, c(length(genome), length(levels(samp)), length(DNA_BASES)),
                  dimnames=list(NULL, levels(samp), base=DNA_BASES))
  pileup[cbind(m, samp, base)[!is.na(base),]] <- depth[!is.na(base)]
  genome$REF <- "."
  genome$REF[m] <- ref(x)
  genome$REF <- DNAStringSet(genome$REF)
  genome$pileup <- pileup
  genome
}

### FIXME: this is too specific to the LR test. To make it more
### generic, we could combine p.error and p.lower into a special
### params object that would be used both for calling and for this
### function. Then, we would have a method that dispatches on that
### class. This params object could be a FilterRule, but we are
### currently not using polymorphism for FilterRule objects.

minCallableCoverage <- function(calling.filters, power = 0.80,
                                max.coverage = 1000L)
{
  if (!isSingleNumber(power) || power < 0 || power > 1)
    stop("'power' must be a single, non-NA number in [0,1]")
  if (!isSingleNumber(max.coverage) || max.coverage < 0)
    stop("'max.coverage' must be a single, non-negative, non-NA number")
  if (!is(calling.filters, "FilterRules"))
    stop("'calling.filters' must be a FilterRules object")
  lr.filter <- calling.filters$likelihoodRatio
  if (is.null(lr.filter))
    stop("'likelihoodRatio' filter not found in 'calling.filters'")
  rc.filter <- calling.filters$readCount
  if (is.null(rc.filter)) {
    min.depth <- 0L
  } else {
    min.depth <- params(rc.filter)$min.depth
  }
  size <- seq(1L, max.coverage)
  f <- lrtFreqCutoff(params(lr.filter)$p.lower, params(lr.filter)$p.error)
  p <- 1 - pbinom(ceiling(pmax(min.depth, size * f))-1L, size,
                  params(lr.filter)$p.lower)
  cov <- head(size[p > power], 1L)
  if (length(cov) == 0L)
    NA_integer_
  else cov
}

setGeneric("callCallable", function(x, ...) standardGeneric("callCallable"))

setMethod("callCallable", "BigWigFile", function(x, pos = NULL, ...) {
  which <- if (is.null(pos)) seqinfo(x) else pos
  rle <- import(x, which = which, as="RleList")
  callCallable(rle, pos = pos, ...)
})

setMethod("callCallable", "RleList", function(x, pos = NULL, ...) {
  if (!is.null(pos)) {
    if (!is(pos, "GenomicRanges"))
      stop("'pos' must be NULL or a GenomicRanges")
    pos.all.width.one <- all(width(pos) == 1L)
    if (!pos.all.width.one)
      stop("All 'pos' ranges must be of length one")
  }
  cutoff <- minCallableCoverage(...)
  if (!is.null(pos)) {
    cov <- extractCoverageForPositions(x, pos)
    cov >= cutoff
  } else {
    x >= cutoff
  }
})

setMethod("callCallable", "ANY", function(x, ...) {
  cov <- coverage(x, drop.D.ranges = TRUE)
  callCallable(cov, ...)
})

# FIXME: instead of 'pos' have a 'which' and a logical byPos argument
# That way, people can iterate over partitions
callWildtype <- function(reads, variants, calling.filters, pos = NULL,
                         ...)
{
  callable <- callCallable(reads, calling.filters = calling.filters,
                           pos = pos, ...)
  wildtype <- callable
  callable[is.na(callable)] <- FALSE
  wildtype[!callable] <- NA
  if (is.null(pos)) {
    seqlevels(variants) <- names(callable)
    var <- as(variants, "IntegerRangesList")
  } else {
    var <- pos %over% variants
  }
  wildtype[var] <- FALSE
  wildtype
}

plotFreq <- function(sam){ 
  gr1 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/TN/",
                        sam, "/",
                        sam,
                        ".sample_specific_variants_granges.RData",
                        sep = "")))

  gr2 <- get(load(paste("/gne/research/data/cgp/2.0/",
                        sam,
                        "/rna_seq_merged/RData/",
                        sam,
                        ".sample_specific_variants_granges.RData",
                        sep = "")))
  over <- findOverlaps(gr1, gr2)
  from_ex <- gr1[queryHits(over)]
  from_rna <- gr2[subjectHits(over)]
  rat1 <- values(from_ex)$count/values(from_ex)$count.total
  rat2 <- values(from_rna)$count/values(from_rna)$count.total
  plot(rat1, rat2, pch = 19, xlim = c(0,1), ylim = c(0,1),
       xlab = "Exome read frequency", ylab = "RNA read frequency",
       main = paste("Somatic mutations in", sam, sep = " "))
  abline(0,1)
}

plotFreq2 <- function(sam){
  gr1 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/Exome/",
                        sam, "/results/",
                        sam,
                        ".filtered_variants_granges.RData",
                        sep = "")))
  gr2 <- get(load(paste("/gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP2.0_var_rerun/RNA/",
                        sam, "/results/",
                        sam,
                        ".variants_granges.RData",
                        sep = "")))
  SNPs <- get(load("~/svn/Packages/VariantTools/inst/extdata/exon_snps_ill25.RData"))
  over <- findOverlaps(gr1, SNPs)
  in_snp1 <- gr1[queryHits(over)]
  in_snp1 <- in_snp1[values(in_snp1)$count >5]
  over <- findOverlaps(gr2, SNPs)
  in_snp2 <- gr2[queryHits(over)]
  in_snp2 <- in_snp2[values(in_snp2)$count >5]

  over <- findOverlaps(in_snp1, in_snp2)
  from_ex <- in_snp1[queryHits(over)]
  from_rna <- in_snp2[subjectHits(over)]
  rat1 <- values(from_ex)$count/values(from_ex)$count.total
  rat2 <- values(from_rna)$count/values(from_rna)$count.total
  plot(rat1, rat2, pch = 19, xlim = c(0,1), ylim = c(0,1),
       xlab = "Exome read frequency", ylab = "RNA read frequency",
       main = paste("Exonic SNP positions in", sam, sep = " "), cex=0.1)
  abline(0,1)
} 

 transparent <- function (cols, alpha = 0.4) {
    col_mat <- col2rgb(cols, alpha = TRUE)
    col_mat["alpha", ] <- alpha * 255
    result <- rgb(col_mat["red", ],
                  col_mat["green", ],
                  col_mat["blue",],
                  col_mat["alpha", ], maxColorValue = 255)
    names(result) <- names(cols)
    return(result)
  }

p.L <- function(x, n, alpha) {
  qbeta(alpha, x, n - x + 1)
}
p.U <- function(x, n, alpha) {
  qbeta(1 - alpha, x + 1, n - x)
}


.plotVarChrom <- function(normal_gr, tumor_gr, normal_cov_gr, chrom, freq = TRUE){
  overN <- findOverlaps(tumor_gr, normal_cov_gr)
  cat(length(tumor_gr[-queryHits(overN)]), "\n", sep = "")
  tumor_gr <- tumor_gr[queryHits(overN)]
  tumor_gr <- tumor_gr[(values(tumor_gr)$count)>3]
  over1 <- which(paste(values(tumor_gr)$location,values(tumor_gr)$alt)
                 %in%
                 paste(values(normal_gr)$location, values(normal_gr)$alt))
  tumor_and_norm <- tumor_gr[over1]
  tumor_specific <- tumor_gr[-over1]
  chrom_gr <- tumor_and_norm[seqnames(tumor_and_norm)==chrom]
  
  par(mfrow = c(2,1))
  plot(start(chrom_gr)/10e5,
       values(chrom_gr)$count/(values(chrom_gr)$count.total), pch = 19,
       cex = .5, col = "red", ylim = c(0,1),
       xlim = c(0,max(start(chrom_gr)/10e5)),
       xlab = paste(chrom, " position in Mb"),
       ylab = "Read frequency of Alt allele")
  
  chrom_ts_gr <- tumor_specific[seqnames(tumor_specific)==chrom]
  
  par(new=T)
  plot(start(chrom_ts_gr)/10e5,
       values(chrom_ts_gr)$count/(values(chrom_ts_gr)$count.total),
       pch = 19, cex = 1, ylim = c(0,1),
       xlim = c(0,max(start(chrom_gr)/10e5)), axes =F, xlab=NA, ylab =NA)

  if(freq==TRUE){
    c1 <- hist(values(chrom_ts_gr)$count/(values(chrom_ts_gr)$count.total), breaks = seq(0, 1, by = 0.05), plot=FALSE)$count
    c2 <- hist(values(chrom_gr)$count/(values(chrom_gr)$count.total), breaks = seq(0, 1, by = 0.05), plot=FALSE)$count
     barplot(rbind(c1, c2), beside=T,
            main = "Tumor-specific variant frequency",
            ylab = "Fraction", xlab = "Total count",
             legend = c("Tumor specific", "Tumor and normal"))
  }else{
    
    c1 <- hist(values(chrom_gr)$count.total, breaks = seq(0, max(values(chrom_gr)$count.total)+100, by=100), plot = FALSE)$count
    c2<- hist(values(chrom_ts_gr)$count.total, breaks = seq(0,  max(values(chrom_gr)$count.total)+100, by=100), plot = FALSE)$count
    barplot(rbind(c1/sum(c1), c2/sum(c2)), beside=T,
            main = "Tumor-specific variant coverage",
            ylab = "Count", xlab = "Total count")
  }
}

 plotVarGenome <- function(normal_gr, normal_raw, tumor_gr, normal_cov,
                           tumor_rna=NULL, freq =FALSE, SRC = NULL,
                           main = "NULL", expressed = FALSE, mosaic = TRUE,
                           bycount = FALSE){
   
   tumor_gr <- tumor_gr[(values(tumor_gr)$count)>=2]
   message("finding normal failed vs normal low freq")
   over1 <- which(paste(values(normal_raw)$location,values(normal_raw)$alt)
                  %in%
                  paste(values(normal_gr)$location, values(normal_gr)$alt))
   
   not_called <- normal_raw[-over1]
   
   normal_low <-not_called[values(not_called)$count/values(not_called)$count.total <.04023]
   
   failed_normal <- not_called[values(not_called)$count/values(not_called)$count.total >=.04023]
   
   ## make normal to test
   
   normal <- c(normal_gr, failed_normal)
   
   message( "find the variants called in tumor and in normal")
   over1 <- which(paste(values(tumor_gr)$location,values(tumor_gr)$alt)
                  %in%
                  paste(values(normal_gr)$location, values(normal_gr)$alt))
   
   tumor_and_normal <- tumor_gr[over1]
   
   message("find the tumor specific calls")
   over1 <- which(paste(values(tumor_gr)$location,values(tumor_gr)$alt)
                  %in%
                  paste(values(normal_raw)$location, values(normal_raw)$alt))
   
   tumor_specific <- tumor_gr[-over1]
   
   over2 <- which(paste(values(tumor_gr)$location,values(tumor_gr)$alt)
               %in%
                  paste(values(normal_low)$location, values(normal_low)$alt))
   
   over3 <- which(paste(values(normal_low)$location,values(normal_low)$alt)
                  %in%
                  paste(values(tumor_gr)$location, values(tumor_gr)$alt))
   
   low_in_tumor <- normal_low[over3]
   
   tumor_in_low <- tumor_gr[over2]
   
   grl <- split(tumor_specific, as.factor(seqnames(tumor_specific)))
#### Need to extend the coverage vectors to the max position of the grl for each chromosome
   
   ends <- lapply(grl, function(x) { max(end(x))})
   
   cov <- lapply(names(ends), function(x) {
    if(ends[[x]] > length(normal_cov[[x]])){
      extend = Rle(values=0, length= ends[[x]]- length(normal_cov[[x]]))
      normal_cov[[x]] = c(normal_cov[[x]], extend)
    }
    return(normal_cov[[x]])
  })         
  names(cov) <- names(grl)
  tumor_specific<- unlist(grl)
  p.lower <-  qbeta(0.125,
                    values(tumor_specific)$count,
                    values(tumor_specific)$count.total -
                    values(tumor_specific)$count +1)
  ##p.lower <- values(tumor_specific)$count/values(tumor_specific)$count.total
  names <-as.list(names(grl))
  n <- lapply(names, function(x) {as.numeric(cov[[x]][ranges(grl[[x]])])})
  n <- unlist(n)
  #p.lower2 <- values(tumor_specific)$count/values(tumor_specific)$count.total
   p.bin <- pbinom(0, n, p.lower)
   cat("Insuficient  power in normal ", sum(p.bin>.01), "\n", sep = "")
   tumor_specific<-tumor_specific[p.bin<0.01]
   p.lower <-  qbeta(0.125,
                     values(tumor_in_low)$count,
                     values(tumor_in_low)$count.total -
                     values(tumor_in_low)$count +1)
   p.bin <- pbinom(values(low_in_tumor)$count, values(low_in_tumor)$count.total, p.lower)
   add <- tumor_in_low[p.bin<0.01]
   
   tumor_specific <- c(tumor_specific, add)

  if(expressed==TRUE){
    over1 <- which(paste(values(tumor_specific)$location,values(tumor_specific)$alt)
               %in%
               paste(values(tumor_rna)$location, values(tumor_rna)$alt))
    cat("Unexpressed tumor variants ", length(tumor_specific)-length(over1), "\n", sep ="")
    tumor_specific <- tumor_specific[over1]
  }
   p<- values(tumor_and_normal)$count/(values(tumor_and_normal)$count.total)
   ##x <- values(tumor_and_normal)$count
   ##cov <- values(tumor_and_normal)$count.total
   ##ci = p.U(x, cov, 0.025) -p.L(x, cov, 0.025) 
   if(mosaic ==TRUE){
     nf <- layout(matrix(c(1,1,2,3), 2,2, byrow=TRUE), widths = c(1.5, 1.5), respect=TRUE)
   }
   if(freq ==TRUE){
      nf <- layout(matrix(c(1,1,2,2), 2, 2, byrow=TRUE), respect=TRUE)
    }
   chroms <- cumsum(as.numeric(seqlengths(Hsapiens)[1:24]))
   chroms[2:24] <- chroms[1:23]
   chroms[1] <- 0
   names(chroms) <- names(seqlengths(Hsapiens)[1:24])
   values(tumor_and_normal)$genome_pos <- start(tumor_and_normal)+chroms[as.character(seqnames(tumor_and_normal))]
   values(tumor_specific)$genome_pos <- start(tumor_specific)+chroms[as.character(seqnames(tumor_specific))]
  
   plot(values(tumor_and_normal)$genome_pos/10e5,
        p, pch = 19,
        cex = .1, col = "dark grey", ylim = c(0,1),
        xlim = c(0,max(values(tumor_and_normal)$genome_pos/10e5, na.rm=T)),
        xlab = paste("Position on genome"),
        ylab = "Read frequency of Alt allele",
        main = SRC)
   
   
   par(new=T)
   plot(values(tumor_specific)$genome_pos/10e5,
        values(tumor_specific)$count/(values(tumor_specific)$count.total),
        pch = 19, cex = .4, ylim = c(0,1),
        xlim = c(0,max(values(tumor_and_normal)$genome_pos/10e5, na.rm =T)), axes =F, xlab=NA, ylab =NA, col="orangered")
   abline(v = chroms/10e5)
   if(mosaic == TRUE){
     mosaicplot(table(as.character(values(tumor_and_normal)$ref), as.character(values(tumor_and_normal)$alt)),
                col = c("firebrick3", "dodgerblue3", "forestgreen", "goldenrod3"),
                cex.axis = 2,
                main = "Germline variants")
     mosaicplot(table(as.character(values(tumor_specific)$ref), as.character(values(tumor_specific)$alt)),
                col = c("firebrick3", "dodgerblue3", "forestgreen", "goldenrod3"),
                cex.axis = 2,
                main = "Tumor specific variants")
   }
   if(freq==TRUE){
     c1 <- hist(values(tumor_specific)$count/(values(tumor_specific)$count.total), breaks = seq(0, 1, by = 0.01), plot=FALSE)$count
     c2 <- hist(values(tumor_and_normal)$count/(values(tumor_and_normal)$count.total), breaks = seq(0, 1, by = 0.01), plot=FALSE)$count
     barplot(rbind(c1/sum(c1), c2/sum(c2)), beside=T,
             main = "Variant frequencies",
             ylab = "Fraction", xlab = "Alternate read frequency",
             legend = c("Tumor specific", "Tumor and normal"),
             names =seq(.01, 1, by = 0.01),
             las=2)
   }
   if(bycount ==TRUE){
     c1 <- hist(values(tumor_and_normal)$count.total, breaks = seq(0, max(values(tumor_and_normal)$count.total)+100, by=100), plot = F)$count
     c2<- hist(values(tumor_specific)$count.total, breaks = seq(0,  max(values(tumor_and_normal)$count.total)+100, by=100), plot = F)$count
     barplot(rbind(c1/sum(c1), c2/sum(c2)), beside=T,
             main = main,
             ylab = "Count", xlab = "Total count")
   }
 }

#!/bin/bash
source /gne/research/apps/modules/common/bashrc
module load apps/igis

zcat /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN//SAM587376/SAM587376_sample_specific_variants.vcf.gz |variant_effect_predictor.pl --sift=b --polyphen=b --condel=b --hgnc --check_existing --output_file /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/varEffOut/SAM587376_variant_annotations.txt

#!/bin/bash
source /gne/research/apps/modules/common/bashrc
module load apps/R

Rscript ~/svn/VariantTools/scripts/run_TN_wrapper.R /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/SAM587376/results/SAM587376.filtered_variants_granges.RData /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/SAM587377/results/SAM587377.filtered_variants_granges.RData /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/SAM587377/results/SAM587377.variants_granges.RData /gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/SAM587377/results/SAM587377.coverage.RData /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/TN/SAM587376 SAM587376 cgp_3.0_colon_TN_rerun

#!/bin/bash
source /gne/research/apps/modules/common/bashrc
module load apps/R

Rscript ~/svn/VariantTools/scripts/run_callVar_wrapper.R /gnet/is3/research/data/bioinfo/ngs_analysis/CGP_3.0/merged/NGS121/SAM587376/bams/SAM587376.analyzed.bam hg19_IGIS21 /gne/research/apps/ngs_pipeline/dev/x86_64-linux-2.6-sles11/share 75 /gnet/is3/research/data/bioinfo/ngs_analysis/degenhj2/CGP3.0_colon_var_rerun/Exome/SAM587376 SAM587376 cgp_rerun

%\VignetteIndexEntry{Introduction to VariantTools}
%\VignetteKeywords{variants, sequencing, SNPs, somatic mutations, RNA editing}
%\VignettePackage{VariantTools}
\documentclass[10pt]{article}

\usepackage{times}
\usepackage[colorlinks=true, linkcolor=blue, urlcolor=blue,
  citecolor=blue]{hyperref}

\textwidth=6.5in
\textheight=8.5in
\oddsidemargin=-.1in
\evensidemargin=-.1in
\headheight=-.3in

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Rcode}[1]{{\texttt{#1}}}
\newcommand{\software}[1]{\textsf{#1}} 
\newcommand{\R}{\software{R}}
\newcommand{\Bioconductor}{\software{Bioconductor}}
\newcommand{\VariantTools}{\Rpackage{VariantTools}}

\title{An Introduction to \VariantTools}
\author{Michael Lawrence, Jeremiah Degenhardt}
\date{\today}

\begin{document}

\maketitle
\tableofcontents

<<<options, echo=FALSE>>=
options(width=72)
@ 

\newpage 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

This vignette outlines the basic usages of the \VariantTools package
and the general workflow for loading data, calling single sample
variants and tumor-specific somatic mutations or other sample-specific
variant types (e.g., RNA editing). Most of the functions operate on
alignments (BAM files) or datasets of called variants. The user is
expected to have already aligned the reads with a separate tool, e.g.,
\software{GSNAP} via \Rpackage{gmapR}.

\section{Calling single-sample variants}

\subsection{Basic usage}
\label{sec:quick-start}

For our example, we take paired-end RNA-seq alignments from two lung
cancer cell lines from the same individual. H1993 is derived from a
metastatis and H2073 is derived from the primary tumor. 

Below, we call variants from a region around the p53 gene:
<<callVariants>>=
library(VariantTools)
bams <- LungCancerLines::LungCancerBamFiles()
bam <- bams$H1993
if (requireNamespace("gmapR", quietly=TRUE)) {
    p53 <- gmapR:::exonsOnTP53Genome("TP53")
    tally.param <- TallyVariantsParam(gmapR::TP53Genome(), 
                                      high_base_quality = 23L,
                                      which = range(p53) + 5e4,
                                      indels = TRUE, read_length = 75L)
    called.variants <- callVariants(bam, tally.param)
} else {
    data(vignette)
    called.variants <- callVariants(tallies_H1993)
}
@
%
In the above, we load the genome corresponding to the human p53 gene
region and the H1993 BAM file (stripped down to the same region). We
pass the BAM, genome, read length and quality cutoff to the
\Rfunction{callVariants} workhorse. The read length is not strictly
required, but it is necessary for one of the QA filters. The value
given for the high base quality cutoff is appropriate for Sanger and
Illumina 1.8 or above. By default, the high quality counts are used by
the likelihood ratio test during calling.

The returned \Robject{called\_variants} is a variant \Rclass{GRanges}, in
the same form as that returned by \Rfunction{bam\_tally} in the
\Rpackage{gmapR} package. \Rfunction{callVariants}
uses \Rfunction{bam\_tally} internally to generate the per-nucleotide
counts (pileup) from the BAM file. Note that \Rpackage{gmapR} is only
supported on UNIX-alikes (Linux, Mac OS X), so we load the precomputed
tallies on other platforms. The result is then filtered to
generate the variant calls. The \Rclass{VCF} class holds similar
information; however, we favor the simple tally \Rclass{GRanges},
because it has a separate record for each ALT, at each position. 
\Rclass{VCF}, the class and the file format, has a single record for a
position, collapsing over multiple ALT alleles, and this is much less
convenient for our purposes. 

We can post-filter the variants for those that are clustered too
closely on the genome:
<<callVariants-postFilter>>=
pf.variants <- postFilterVariants(called.variants)
@

We can subset the variants by those in an actual p53 exon (not an
intron):
<<callVariants-exonic>>=
subsetByOverlaps(called.variants, p53, ignore.strand = TRUE)
@ 

The next section goes into further detail on the process, including
the specific filtering rules applied, and how one might, for example,
tweak the parameters to avoid calling low-coverage variants,
like the one above.

\subsection{Step by step}
\label{sec:step-by-step}

The \Rfunction{callVariants} method for BAM files, introduced above,
is a convenience wrapper that delegates to several low-level functions
to perform each step of the variant calling process: generating the
tallies, basic QA filtering and the actual variant calling. Calling
these functions directly affords the user more control over the
process and provides access to intermediate results, which is useful
e.g. for diagnostics and for caching results. The workflow consists of
three function calls that rely on argument defaults to achieve the
same result as our call to \Rfunction{callVariants} above. Please see
their man pages for the arguments available for customization.

The first step is to tally the variants from the BAM file. By default,
this will return observed differences from the reference, excluding N
calls and only counting reads above 13 in mapping quality (MAPQ)
score. There are three read position bins: the first 10 bases, the final 10
bases, and the stretch between them (these will be used in the QA
step).
<<tallyVariants>>=
if (requireNamespace("gmapR", quietly=TRUE)) {
    tallies_H1993 <- tallyVariants(bam, tally.param)
}
@ 

Unless one is running a variant caller in a routine fashion over
familiar types of data, we highly recommend performing detailed QC of
the tally results. \Rpackage{VariantTools} provides several QA filters
that aim to expose artifacts, especially those generated during
alignment. These filters are \emph{not} designed for filtering during
actual calling; rather, they are meant for annotating the variants
during exploratory analysis. The filters include a check on the median
distance of alt calls from their nearest end of the read (default
passing cutoff >= 10), as well as a Fisher Exact Test on the
per-strand counts vs. reference for strand bias (p-value cutoff:
0.001). The intent is to ensure that the data are not due to
strand-specific nor read position-specific artifacts.

The \Rfunction{qaVariants} function will \emph{soft} filter the
variants via \Rfunction{softFilter}. No variants are removed; the
filter results are added to the \Rfunction{softFilterMatrix} component
of the object.
<<qaVariants>>=
qa.variants <- qaVariants(tallies_H1993)
summary(softFilterMatrix(qa.variants))
@ 

The final step is to actually call the variants. The
\Rfunction{callVariants} function uses a binomial likelihood ratio
test for this purpose. The ratio is $P(D|p=p_{lower}) /
P(D|p=p_{error})$, where $p_{lower} = 0.2$ is the assumed lowest
variant frequency and $p_{error} = 0.001$ is the assumed error rate in
the sequencing (default: 0.001).
<<callVariants-qa>>=
called.variants <- callVariants(qa.variants)
@ 

The \Rfunction{callVariants} function applies an additional set of
filters after the actual variant calling. These are known as ``post''
filters and consider the putative variant calls as a set, independent
of the calling algorithm.  Currently, there is only one post filter by
default, and it discards variants that are clumped together along the
chromosome, as these often result from mapping difficulties.

\subsection{Diagnosing the filters}
\label{sec:filter-diag}

The calls to \Rfunction{qaVariants} and \Rfunction{callVariants} are
essentially filtering the tallies, so it is important to know,
especially when faced with a new dataset, the effect of each filter
and the effect of the individual parameters on each filter. 

The filters are implemented as modules and are stored in a
\Rclass{FilterRules} object from the \Rpackage{IRanges} package. We
can create those filters directly and rely on some
\Rclass{FilterRules} utilities to diagnose the filtering process.

Here we construct the \Rclass{FilterRules} that implements the
\Rfunction{qaVariants} function. Again, we rely on the argument
defaults to generate the same answer.
<<VariantQAFilters>>=
qa.filters <- VariantQAFilters()
@ 
%
We can now ask for a summary of the filtering process, which gives the
number of variants that pass each filter, separately and then combined:
<<VariantQAFilters-summary>>=
summary(qa.filters, tallies_H1993)
@ 
%
Now we retrieve only the variants that pass the filters:
<<VariantQAFilters-subset>>=
qa.variants <- subsetByFilter(tallies_H1993, qa.filters)
@ 

We could do the same, except modify a filter parameter, such as the
p-value cutoff for the Fisher Exact Test for strand bias:
<<VariantQAFilters-fisherStrandP>>=
qa.filters.custom <- VariantQAFilters(fisher.strand.p.value = 1e-4)
summary(qa.filters.custom, tallies_H1993)
@ 
%
To get a glance at the additional variants we are discarding
compared to the previous cutoff, we can subset the filter sets down to the
Fisher strand filter, evaluate the old and new filter, and compare the
results: 
<<VariantQAFilters-compare>>=
fs.original <- eval(qa.filters["fisherStrand"], tallies_H1993) 
fs.custom <- eval(qa.filters.custom["fisherStrand"], tallies_H1993)
tallies_H1993[fs.original != fs.custom]
@ 

Below, we demonstrate how one might add a mask to e.g. filter out
variants in low complexity regions, where mapping errors tend to dominate:
<<VariantQAFilters-mask>>=
if (requireNamespace("gmapR", quietly=TRUE)) {
    tally.param@mask <- GRanges("TP53", IRanges(1010000, width=10000))
    tallies_masked <- tallyVariants(bam, tally.param)
}
@

We can also diagnose the filters for calling variants after basic QA
checks.
<<VariantCallingFilters>>=
calling.filters <- VariantCallingFilters()
summary(calling.filters, qa.variants)
@ 

Check how the post filter would perform prior to variant calling:
<<post-filter>>=
post.filters <- VariantPostFilters()
summary(post.filters, qa.variants)
@ 
%
What about if we preserved the ones we have already called?
<<post-filter-whitelist>>=
post.filters <- VariantPostFilters(whitelist = called.variants)
summary(post.filters, qa.variants)
@ 

\subsection{Extending and customizing the workflow}
\label{sec:extending}

Since the built-in filters are implemented using \Rclass{FilterRules},
it is easy to mix and match different filters, including those
implemented externally to the \Rpackage{VariantTools} package. This is
the primary means of extending and customizing the variant calling
workflow.

\section{Comparing variant sets across samples}

So far, we have called variants for the metastatic H1993 sample. We
leave the processing of the primary tumor H2073 sample as an exercise
to the reader and instead turn our attention to detecting the variants
that are specific to the metastatic sample, as compared to the primary
tumor.

\subsection{Calling sample-specific variants}

The function \Rfunction{callSampleSpecificVariants} takes the case
(e.g., tumor) sample and control (e.g., matched normal) sample as
input. In our case, we are comparing the metastatic line (H1993) to
the primary tumor line (H2073) from the same patient, a smoker. To
avoid inconsistencies, it is recommended to pass BAM files as input,
for which tallies are automatically generated, subjected to QA, and
called as variants vs. reference, prior to determining the
sample-specific variants. 

Here, we find the somatic mutations from a matched tumor/normal
pair. Since we are starting from BAM files, we have to provide
\Robject{tally.param} for the tally step. 
<<callSomaticMutations>>=
if (requireNamespace("gmapR", quietly=TRUE)) {
    tally.param@bamTallyParam@indels <- FALSE
    somatic <- callSampleSpecificVariants(bams$H1993, bams$H2073,
                                          tally.param)
} else {
    somatic <- callSampleSpecificVariants(called.variants, tallies_H2073,
                                          coverage_H2073)
}
@ 
%$
This can be time-consuming for the entire genome, since the tallies
need to be computed. To avoid repeated computation of the tallies, the
user can pass the raw tally \Rclass{GRanges} objects instead of the
BAM files. This is less safe, because anything could have happened to
those \Rclass{GRanges} objects.

The QA and initial calling are optionally controlled by passing
\Rclass{FilterRules} objects, typically those returned by
\Rfunction{VariantQAFilters} and \Rfunction{VariantCallingFilters},
respectively. For controlling the final step, determining the
sample-specific variants, one may pass filter parameters directly to
\Rfunction{callSampleSpecificVariants}.  Here is an example of
customizing some parameters.
<<callSomaticMutations-readCount>>=
calling.filters <- VariantCallingFilters(read.count = 3L)
if (requireNamespace("gmapR", quietly=TRUE)) {
    somatic <- callSampleSpecificVariants(bams$H1993, bams$H2073, tally.param,
                                          calling.filters = calling.filters,
                                          power = 0.9, p.value = 0.001)
} else {
    called.variants <- callVariants(tallies_H1993, calling.filters)
    somatic <- callSampleSpecificVariants(called.variants, tallies_H2073,
                                          coverage_H2073,
                                          power = 0.9, p.value = 0.001)
}
@ 

\section{Exporting the calls as VCF}

VCF is a common file format for communicating variants. To export our
variants to a VCF file, we first need to coerce the \Rclass{GRanges}
to a \Rclass{VCF} object. Then, we use \Rfunction{writeVcf} from the
\Rpackage{VariantAnnotation} package to write the file (indexing is
highly recommended for large files). Note that the sample names need
to be non-missing to generate the VCF. Also, for simplicity and
scalability, we typically do not want to output all of our metadata
columns, so we remove all of them here.
<<variantGR2VCF>>=
sampleNames(called.variants) <- "H1993"
mcols(called.variants) <- NULL
vcf <- asVCF(called.variants)
@ 
<<writeVcf, eval=FALSE>>=
writeVcf(vcf, "H1993.vcf", index = TRUE)
@ 

\section{Finding Wildtype and No-call Regions}

So far, our analysis has yielded a set of positions that are likely to
be variants. We have not made any claims about the status of the
positions outside of that set. For this, we need to decide, for each
position, whether there was sufficient coverage to detect a variant,
if one existed. The following call carries out a power test to decide
whether a region is variant, wildtype or is unable to be called due to
lack of coverage. The variants must have been called using the filters
returned by \Rfunction{VariantCallingFilters}. The algorithm depends
on the filter parameter settings, so it is possible and indeed
required for the user to pass filter object used for calling the
variants. This requirement is an attempt to ensure consistency and
will be made more convenient in the future. To request the calls for a
particular set of positions, one can pass a \Rclass{GenomicRanges}
(where all the ranges are of width 1) as the \Rfunarg{pos}
argument. When \Rfunarg{pos} is specfied, each element of the result
corresponds to an element in \Rfunarg{pos}.
<<callWildtype>>=
called.variants <- called.variants[!isIndel(called.variants)]
pos <- c(called.variants, shift(called.variants, 3))
wildtype <- callWildtype(bam, called.variants, VariantCallingFilters(), 
                         pos = pos, power = 0.85)
@
%
The returned object is a logical vector, with
\Rcode{TRUE} for wildtype, \Rcode{FALSE} for variant and \Rcode{NA}
for no-call. Thus, we could calculate the fraction called as follows:
<<percentageCalled>>=
mean(!is.na(wildtype))
@ 

Sometimes it is desirable for the wildtype calls to be returned as
simple vector, with a logical value for each position (range of width
one) in \Rfunction{bamWhich}.  Such a vector is returned when
\Rcode{global = FALSE} is passed to \Rfunction{callWildtype}. This is
the same as extracting the positions from the ordinary \Rclass{Rle}
return value, but it is implemented more efficiently, at least for a
relatively small number of positions.

% \section{Checking variant concordance}

% <<variantConcordance>>=
% concord <- checkVariantConcordance(tumor.variants, normal.variants)
% concord
% @ 

% \section{Making some plots}

% It is important to diagnose the behavior of these algorithms, and we
% provide some exploratory plots to facilitate this. 

% The first plot shows us the mutation transition/transversion rate matrix
% <<plotting>>=
% #plotTitv(TS, main = "tumor specific mutations")  
% @ 

% and finally we want to plot our variants on the genome 
% <<plotting>>= 
% #plotTumor(TS, tumor$filtered.granges)
% @ 

\end{document}

\documentclass[10pt]{article}

%\usepackage{times}
%\usepackage{hyperref}
\usepackage{Sweave}

\textwidth=6.5in
\textheight=8.5in
% \parskip=.3cm
\oddsidemargin=-.1in
\evensidemargin=-.1in
\headheight=-.3in

\newcommand{\software}[1]{\textsf{#1}}
\newcommand{\R}{\software{R}}

\title{Variant calling and somatic mutation detection in tumor/normal pairs and cell lines}
\author{Jeremiah Degenhardt}
\date{\today}

\begin{document}

\maketitle

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\subsection{Sample clustering}

<<SampleClustering, fig = TRUE>>=
Samples <- paste('SAM6338', 13:21, sep = '')
baseDir <- file.path('/gnet/is2/bioinfo/ngs_analysis/RNA-seq/')
sampleDir <- c("R116/demultiplex/001/s_1",
               "R116/demultiplex/001/s_2",
               "R116/demultiplex/001/s_3",
               "R116/demultiplex/001/s_4",
               "R116/demultiplex/001/s_5",
               "R116/demultiplex/001/s_6",
#               "R116/demultiplex/001/s_7_part_00",
#               "R116/demultiplex/001/s_8_part_00",
               "R116/demultiplex/001/s_7",
               "R116/demultiplex/001/s_8",
               "R112/demultiplex/001/s_1")
outputDirs <- paste(baseDir, sampleDir, sep = "")
sampleData <- data.frame(SAMID = Samples,
                         cell_line = rep(c("H1299", "H1993", "H2073"), each = 3),
                         treatment = rep(c("Control", "IFN-alpha", "IFN-alpha A/D"), times = 3),
                         read_len = c(rep("100bp", times = 8), "75bp"),
                         chunked_lane = c(rep(FALSE, times = 6),
                           rep(TRUE, times = 2), FALSE))
                         

rpkmsTable <- lapply(outputDirs,
                     function(x) {
                        result <- get(load(dir(file.path(x, 'RData'),
                                               pattern = 'rpkms.RData',
                                               full.names = TRUE)))
                        vectName <- names(result)[names(result) %in%
                                                  c("gene_coding", "genes_coding")]
                        result <- result[[vectName]]
                     })

## some of these have more genes than others, so we
## keep only the genes in common between each outout
allGenes <- unique(do.call(c,lapply(rpkmsTable,
                             function(x) {
                               result <- names(x)
                             })))
library(multicore)
unionGenes <- mclapply(allGenes,
                     function(y) {
                       myGene <- y
                       system(paste("echo",myGene,"\n"))
                       howMany <- sum(do.call(c,lapply(rpkmsTable,
                                                       function(x) {
                                                         result <- myGene %in% 
                                                         as.character(names(x))
                                                       })))
                       if(howMany == length(Samples)) {
                         inUnion <- TRUE
                       }else {
                         inUnion <- FALSE
                       }
                       inUnion
                     })
unionGenes <- do.call(c, unionGenes)
sum(unionGenes)

usableGenes <- allGenes[unionGenes]
system("mkdir output")
save(usableGenes, file = "output/usableGenes.RData")
load("output/usableGenes.RData")

correctedTable <- lapply(rpkmsTable,
                         function(x) {
                           results <- x[as.character(names(x)) %in%
                                        as.character(usableGenes)]
                           results <- results[order(names(results))]
                         })
rpkmsTable <- do.call(cbind,correctedTable)
                             
dimnames(rpkmsTable) <- list(rownames(rpkmsTable),Samples)


## ##################################
## For plotting purposes keep only
## probes with top 2.5% of sd
## ##################################
library(RColorBrewer)
library(matrixStats)
library(gplots)
HMcols=rev(brewer.pal(10,"RdBu"))

v <- rowSds(rpkmsTable,na.rm=TRUE)
threshold <- quantile(v,probs=c(0.975))
topSds <- rpkmsTable[v > threshold,]
dimnames(topSds) <- list(rownames(topSds),Samples)


## ##################################
## Draw heatmap
## ##################################
mycolors <- rep("grey", times = length(sampleData$treatment))
mycolors[sampleData$treatment == "IFN-alpha"] <- "red"
mycolors[sampleData$treatment == "IFN-alpha A/D"] <- "firebrick4"
heatmap.2(as.matrix(topSds),
          Rowv=NA,
          trace = "none",
          labRow=NA,
#        labCol=NA,
          scale="row",
          col=HMcols,
          ColSideColors=mycolors,
          margins = c(7,7),
          main="Interferon alpha vs Control\nRPKMs of top most variable genes (CDS)")
legend("topright",
       legend=c("IFN-alpha", "IFN-alpha A/D", "Control"),
       col=c("red", "firebrick4", "grey"),cex=0.7,box.col = "white", pch=16)

reportDir <- "report"
system(paste("mkdir", reportDir)
system(paste("mkdir", file.path(reportDir, "figures")))
       
dev.copy(pdf, file = "report/figures/heatmap_of_most_variable_genes.pdf")
dev.off()
dev.copy(jpeg, file = "report/figures/heatmap_of_most_variable_genes.jpg")
dev.off()
library(hwriter)


p=openPage(file.path(reportDir,"sample_clustering"))
hwrite('Interferon-alpha treatment in NSCLC cell lines', p, heading=1)
hwrite('Sample clustering', p, heading=2)

hwrite(sampleData, p,
       col.bgcolor = list(treatment = rep(c("#BEBEBE", "#FF0000", "#8B1A1A"),
                            times = 3)),
       row.bgcolor = '#CCCCCC',
       table.style = 'background-color: #F0F0F0', br = TRUE)
hwrite('', p, br = TRUE)

hwrite('Heatmap of the top 2.5% most variable genes (CDS)', p, br = TRUE)
hwrite('', p, br = TRUE)
hwriteImage(file.path("figures","heatmap_of_most_variable_genes.jpg"), p, link = file.path("figures","heatmap_of_most_variable_genes.pdf"), br = TRUE)
hwrite('', p, br = TRUE)

text <- paste('The samples cluster by cell line, with treatments being',
              'most similar within each cell line.')
hwrite(text, p, br=TRUE,
       col.width='20px') 



@ 

\subsection{Differential expression analysis}

Here we perform differential expression analysis using the DESeq method. However,
the pvalue and FDR calculations seem incorrect, so we will also use the edgeR method
further below.

<<data_import>>=
countsTable <- lapply(outputDirs,
                      function(x) {
                         result <- get(load(dir(file.path(x,'RData'),
                                                pattern = "hits_by_gene_coding",
                                                full.names = TRUE)))
                      })
correctedTable <- lapply(countsTable,
                         function(x) {
                           results <- x[as.character(names(x)) %in%
                                        as.character(usableGenes)]
                           results <- results[order(names(results))]
                         })
countsTable <- do.call(cbind,correctedTable)
                             
dimnames(countsTable) <- list(rownames(countsTable),Samples)


countsTable <- as.data.frame(countsTable)
@ 

<<DESeq,fig=TRUE>>=
library(DESeq)
library(qvalue)

sampleData$comparison1 <- ifelse(sampleData$treatment == "Control", "Control", "IFN")
conds <- factor(sampleData$comparison1)

cds <- newCountDataSet( countsTable, conds )
cds <- estimateSizeFactors( cds )
cds <- estimateDispersions( cds )
#res <- nbinomTest( cds, "Normal", "Tumor")
res <- nbinomTest( cds, "Control", "IFN")
save(res, file = "output/DESeq_res.RData")
load("output/DESeq_res.RData")

sigGenes <- subset(res, !is.na(pval) & pval < 0.05 & abs(log2FoldChange) > 2)
#qvals <- qvalue(sigGenes$pval)$qvalues
#sigGenes$qval <- qvals
sigGenes <- sigGenes[order(sigGenes$pval),]
rownames(sigGenes) <- NULL
head(sigGenes[,c(1,2,3,4,6,7)],n=20)

### write report
panel.boxnstripplot <- function(x, y, jitter.data = T, ...){
    op <- trellis.par.get()
    rect <- trellis.par.get('box.rectangle')
    rect$col <- 'grey'
    umbr <- trellis.par.get('box.umbrella')
    umbr$col <- 'grey'
    umbr$lty <- 1
    pc <- trellis.par.get('plot.symbol')
    pc$alpha <- 0
    trellis.par.set('box.rectangle', rect)
    trellis.par.set('box.umbrella', umbr)
    trellis.par.set('plot.symbol', pc)
    panel.bwplot(x,y,pch='|', cex=5,...)
    pc$alpha <- 1
    trellis.par.set('plot.symbol', pc)
    panel.stripplot(x,y,jitter.data=jitter.data,...) 
    trellis.par.set(op)
}

plotGene <- function(geneID, rpkmsTable, conds, geneName = NULL, sampleData) {
  myGene <- rpkmsTable[which(rownames(rpkmsTable) == geneID),]
  mydf <- data.frame(RPKM = myGene, Condition = conds)
  mydf <- cbind(mydf,sampleData)
  if(is.null(geneName)) {
    geneName <- egsymList[[geneID]]
  }
  mycols <- c("grey", "red", "firebrick4")
  mytheme <- trellis.par.get()
  mytheme$superpose.symbol
  print(
  stripplot(RPKM ~  Condition, 
            data = mydf, 
            groups = SAMID, 
            par.settings=list(
              superpose.symbol=list(col = mycols, fill = mycols, pch = rep(c(22, 24, 21), each = 3))),
            main=paste("Expression of", geneName, paste("(", geneID, ")", sep="")),
            ylab="Expression (RPKM)",
            auto.key = list(space = "right"),
            panel.boxnstripplot
            ) 
           )
}


source("../R/make_html_table.R")
## ADARs
ADARs <- c("103", "104", "105")
make_html_table(
                EGIDs = ADARs,
                rpkmsTable = rpkmsTable,
                conds = conds,
                reportDir = reportDir,
                reportFile = "adar_expression",
                reportSubTitle = "",
                reportTitle = "Expression of the ADARs",
                species = "Hs",
                sampleData = sampleData)


## DESeq MA plot
p=openPage(file.path(reportDir,"differential_expression_DESeq"))
hwrite('Differential expression (DESeq)', p, heading=2)
hwrite('', p, br = TRUE)

plot( log2(res$baseMean), res$log2FoldChange, 
     pch=20, cex=.8, 
     col = ifelse( res$pval < .05, 
       rgb(255,0,0,50,maxColorValue=255), 
       rgb(155,155,155,50,maxColorValue=255) ),
     xlab="log2(Mean expression)",
     ylab="log2FC (IFN - Control)",
     main=paste("IFN vs. Control")
     )
abline(h=c(-2,2),lwd=2, lty = 2, col="orange")
abline(h = 0, col = "dodgerblue", lty = 2)
dev.copy(pdf, file = file.path(reportDir, "figures/DESeq_MA_plot.pdf"))
dev.off()
dev.copy(jpeg, file = file.path(reportDir, "figures/DESeq_MA_plot.jpg"))
dev.off()

hwriteImage(file.path("figures","DESeq_MA_plot.jpg"), p, link = file.path("figures","DESeq_MA_plot.pdf"), br = TRUE)
hwrite('', p, br = TRUE)

## heatmap of DESeq DE genes
myrpkms <- rpkmsTable[match(as.character(sigGenes$id),
                            as.character(rownames(rpkmsTable))),]
myrpkms <- myrpkms[order(sigGenes$log2FoldChange),]
#mycolors <- ifelse(conds == "IFN", "red", "grey")
heatmap.2(as.matrix(myrpkms),
          Rowv=NA,
          trace = "none",
          scale="row",
          col=HMcols,
          ColSideColors=mycolors,
          margins = c(7,7),
          main="IFN vs Control\nRPKMs of DE genes (CDS)")
legend("topright",
       legend=c("IFN-alpha", "IFN-alpha A/D", "Control"),
       col=c("red", "firebrick4", "grey"),cex=0.7,box.col = "white", pch=16)
dev.copy(pdf, file = file.path(reportDir, "figures", "DESeq_RPKM_heatmap.pdf"))
dev.off()
dev.copy(jpeg, file = file.path(reportDir, "figures", "DESeq_RPKM_heatmap.jpg"))
dev.off()
    
hwrite('', p, br = TRUE)
hwrite('', p, br = TRUE)

hwriteImage(file.path("figures","DESeq_RPKM_heatmap.jpg"), p, link = file.path("figures","DESeq_RPKM_heatmap.pdf"), br = TRUE)
hwrite('', p, br = TRUE)

make_html_table(
                EGIDs = sigGenes$id,
                rpkmsTable = rpkmsTable,
                conds = conds,
                reportDir = reportDir,
                reportFile = reportName,
                reportTitle = "",
                reportSubTitle = "Differential expression based on pvalue < 0.05 and abs(log2FC) > 2",
                species = "Hs",
                p = p,
                sampleData = sampleData)



#####











@ 

\subsection{edgeR}

<<edgeR,fig=TRUE>>=
## ########################################
## Create comparisons
## ########################################
p=openPage(file.path(reportDir,"differential_expression_edgeR"))
hwrite('Differential expression (edgeR)', p, heading=2)
hwrite('', p, br = TRUE)
library(edgeR)

dge <- DGEList(countsTable, group = conds)
dge <- calcNormFactors(dge)

## filter very low read count genes as per edgeR vignette
cpm.dge <- cpm(dge)
dge <- dge[ rowSums(cpm.dge > 1) >=3, ]
dge <- calcNormFactors(dge)
dge

## optional MDS plot
library(limma)
plotMDS(dge, main="MDS Plot", #xlim=c(-1,1), 
        labels=sampleData$SAMID)
@ 

<<edgeR_part2, fig = TRUE>>=


## estimate the common dispersion
dge <- estimateCommonDisp(dge)
de.com <- exactTest(dge)
save(de.com, file = "output/de.com.RData")
load("output/de.com.RData")
topTags(de.com)
detags.com <- rownames(topTags(de.com)$table)
cpm.dge[detags.com, ]
topTags(de.com, n = 10, sort.by = "logFC")

summary(decideTestsDGE(de.com, p.value=0.01))

detags500.com <- rownames(topTags(de.com, n = 500)$table)

plotSmear(de.com, de.tags = detags500.com, main = "FC plot using common dispersion")
abline(h = 0, col = "dodgerblue", lwd = 2, lty = 2)
abline(h = c(-2, 2), col = "orange", lwd = 2, lty = 2)
dev.copy(pdf, file = file.path(reportDir, "figures/edgeR_MAplot.pdf"))
dev.off()
dev.copy(jpeg, file = file.path(reportDir, "figures/edgeR_MAplot.jpg"))
dev.off()

hwriteImage(file.path("figures","edgeR_MAplot.jpg"), p, link = file.path("figures","edgeR_MAplot.pdf"), br = TRUE)
hwrite('', p, br = TRUE)



res <- topTags(de.com, n = dim(countsTable)[1])$table
res <- res[order(res$adj.P.Val),]
res$EGID <- rownames(res)

sigUpInAD <- subset(res, logFC <= -2 & P.Value < 0.05)
sigDownInAD <- subset(res, logFC >= 2 & P.Value < 0.05)

sigGenes <- rbind(sigUpInAD,sigDownInAD)

## heatmap of edgeR DE genes
myrpkms <- rpkmsTable[match(as.character(sigGenes$EGID),
                            as.character(rownames(rpkmsTable))),]
myrpkms <- myrpkms[order(sigGenes$logFC),]
heatmap.2(as.matrix(myrpkms),
          Rowv=NA,
          trace = "none",
          scale="row",
          col=HMcols,
          ColSideColors=mycolors,
          margins = c(7,7),
          main="IFN vs Control\nRPKMs of DE genes (CDS)")
legend("topright",
       legend=c("IFN-alpha", "IFN-alpha A/D", "Control"),
       col=c("red", "firebrick4", "grey"),cex=0.7,box.col = "white", pch=16)
dev.copy(pdf, file = file.path(reportDir, "figures", "edgeR_RPKM_heatmap.pdf"))
dev.off()
dev.copy(jpeg, file = file.path(reportDir, "figures", "edgeR_RPKM_heatmap.jpg"))
dev.off()
    
hwrite('', p, br = TRUE)
hwriteImage(file.path("figures","edgeR_RPKM_heatmap.jpg"), p, link = file.path("figures","edgeR_RPKM_heatmap.pdf"), br = TRUE)
hwrite('', p, br = TRUE)



make_html_table(
                EGIDs = sigGenes$EGID,
                rpkmsTable = rpkmsTable,
                conds = conds,
                reportDir = reportDir,
                reportFile = reportName,
                reportTitle = "",
                reportSubTitle = "Differential expression based on pvalue < 0.05 and abs(log2FC) > 2",
                species = "Hs",
                p = p,
                sampleData = sampleData)




@ 

 
\subsection{GO analysis} 

<<GO_analysis>>=

## #######################
## GO analysis
## #######################
## GO analysis

## upregulated genes
upreg <- sigUpInAD

library(org.Mm.eg.db)

myEGIDs <- as.character(upreg$EGID[upreg$EGID!=""])

###########################################
## now we fetch all the entrez gene IDs with GO mappings
## (the EG universe used here should only contain EGs with mappings to GO)

egUniverse <- mappedkeys(org.Mm.egGO)

## toss any of our EGs that are not in the egUniverse (thus, have no GO mappings)

myEGIDs <- myEGIDs[myEGIDs %in% egUniverse]

## now we do some stats to look for over representation of GO categories in
## our set of putatively hyper edited genes

library(GOstats)
## BP = Biological Process
hgCutoff <- 0.01
BPparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "BP", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

BPparamsCond <- BPparams
conditional(BPparamsCond) <- TRUE

BPhgOver <- hyperGTest(BPparams)
BPhgCondOver <- hyperGTest(BPparamsCond)

cutoff <- 0.01
dim(summary(BPhgOver, pvalue = cutoff))
summary(BPhgOver, pvalue = cutoff)
dim(summary(BPhgOver, categorySize = 10))
dim(summary(BPhgCondOver, pvalue = cutoff))
summary(BPhgCondOver, pvalue = cutoff)

htmlReport(BPhgOver, file = "output/upregInAD_BP_hgo.html")
htmlReport(BPhgCondOver, file = "output/upregInAD_BP_hgco.html")

## CC = Cellular Component
CCparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "CC", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

CCparamsCond <- CCparams
conditional(CCparamsCond) <- TRUE

CChgOver <- hyperGTest(CCparams)
CChgCondOver <- hyperGTest(CCparamsCond)

cutoff = 0.01
dim(summary(CChgOver,pvalue = cutoff))
summary(CChgOver,pvalue = cutoff)
dim(summary(CChgCondOver,pvalue = cutoff))
summary(CChgCondOver,pvalue = cutoff)

htmlReport(CChgOver, file = "output/upregInAD_CC_hgo.html")
htmlReport(CChgCondOver, file = "output/upregInAD_CC_hgco.html")

## MF = Molecular Function
MFparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "MF", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

MFparamsCond <- MFparams
conditional(MFparamsCond) <- TRUE

MFhgOver <- hyperGTest(MFparams)
MFhgCondOver <- hyperGTest(MFparamsCond)

cutoff = 0.01
dim(summary(MFhgOver,pvalue = cutoff))
summary(MFhgOver,pvalue = cutoff)
dim(summary(MFhgCondOver,pvalue = cutoff))
summary(MFhgCondOver,pvalue = cutoff)

htmlReport(MFhgOver, file = "output/upregInAD_MF_hgo.html")
htmlReport(MFhgCondOver, file = "output/upregInAD_MF_hgco.html")

####################################################
## downregulated genes
downreg <- sigDownInAD
myEGIDs <- as.character(downreg$EGID[downreg$EGID!=""])

###########################################
## now we fetch all the entrez gene IDs with GO mappings
## (the EG universe used here should only contain EGs with mappings to GO)

egUniverse <- mappedkeys(org.Mm.egGO)

## toss any of our EGs that are not in the egUniverse (thus, have no GO mappings)

myEGIDs <- myEGIDs[myEGIDs %in% egUniverse]

## now we do some stats to look for over representation of GO categories in
## our set of putatively hyper edited genes

library(GOstats)
## BP = Biological Process
hgCutoff <- 0.01
BPparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "BP", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

BPparamsCond <- BPparams
conditional(BPparamsCond) <- TRUE

BPhgOver <- hyperGTest(BPparams)
BPhgCondOver <- hyperGTest(BPparamsCond)

cutoff <- 0.01
dim(summary(BPhgOver, pvalue = cutoff))
summary(BPhgOver, pvalue = cutoff)
dim(summary(BPhgOver, categorySize = 10))
dim(summary(BPhgCondOver, pvalue = cutoff))
summary(BPhgCondOver, pvalue = cutoff)

htmlReport(BPhgOver, file = "output/downregInAD_BP_hgo.html")
htmlReport(BPhgCondOver, file = "output/downregInAD_BP_hgco.html")

## CC = Cellular Component
CCparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "CC", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

CCparamsCond <- CCparams
conditional(CCparamsCond) <- TRUE

CChgOver <- hyperGTest(CCparams)
CChgCondOver <- hyperGTest(CCparamsCond)

cutoff = 0.01
dim(summary(CChgOver,pvalue = cutoff))
summary(CChgOver,pvalue = cutoff)
dim(summary(CChgCondOver,pvalue = cutoff))
summary(CChgCondOver,pvalue = cutoff)

htmlReport(CChgOver, file = "output/downregInAD_CC_hgo.html")
htmlReport(CChgCondOver, file = "output/downregInAD_CC_hgco.html")

## MF = Molecular Function
MFparams <- new("GOHyperGParams", geneIds = myEGIDs,
              universeGeneIds = egUniverse, annotation = "org.Mm.eg.db",
              ontology = "MF", pvalueCutoff = hgCutoff, conditional = FALSE,
              testDirection = "over")

MFparamsCond <- MFparams
conditional(MFparamsCond) <- TRUE

MFhgOver <- hyperGTest(MFparams)
MFhgCondOver <- hyperGTest(MFparamsCond)

cutoff = 0.01
dim(summary(MFhgOver,pvalue = cutoff))
summary(MFhgOver,pvalue = cutoff)
dim(summary(MFhgCondOver,pvalue = cutoff))
summary(MFhgCondOver,pvalue = cutoff)

htmlReport(MFhgOver, file = "output/downregInAD_MF_hgo.html")
htmlReport(MFhgCondOver, file = "output/downregInAD_MF_hgco.html")


@ 

<<RNA-editing>>=

## generate nuc tallies from bam files
bam_files <- sapply(outputDirs,
                    function(x) {
                      bam_file <- dir(file.path(x,"bams"), pattern = "concordant_uniq.bam$", full.names = TRUE)
                    })
       source("../R/rna-editing.R")
## these steps were done and output saved in output/
#sapply(1:(dim(sampleData)[1]),
#              function(x) {
#                do_bam_tally(bam_file = bam_files[x],
#                             output_file = file.path("output",
#                               paste(sampleData$SAMID[x], "bam_tally.txt", sep = ".")))
#              })
       
sapply(1:(dim(sampleData)[1]),
       function(x) {
         parse_bam_tally(bam_tally_output_file = file.path("output", 
                           paste(sampleData$SAMID[x],"bam_tally.txt", sep = ".")),
                         output_file = file.path("output",
                           paste(sampleData$SAMID[x], "parsed_bam_tally.txt", sep = ".")))
       })


@ 

<<SessionInfo>>=
sessionInfo()
@ 
\end{document}

\name{detectQualityType}
\alias{detectQualityType}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
A function to detect the quality score (Qscore) type contained in a BAM
file. This function is needed as gSNAP will produce non-standard bam
files 
}
\description{
  A function to detect the quality score (Qscore) type contained in a
  BAM file. This function is needed as gSNAP will produce non-standard
  bam files
}
\usage{
detectQualityType(bamfile, nrows = 5000)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{bamfile}{
%%     ~~Describe \code{bamfile} here~~
}
  \item{nrows}{
%%     ~~Describe \code{nrows} here~~
}
}
\author{
%%  ~~who you are~~
Jeremiah Degenhardt
}

\keyword{internal}

\name{annotateWithControlDepth}
\alias{annotateWithControlDepth}
\title{
  Annotate Case with Control Depth
}
\description{
  Matches the \code{case} variants to the (typically unfiltered)
  \code{control} variants and returns \code{case}, with the additional
  metadata columns \code{control.alt.depth} and
  \code{control.total.depth}, corresponding to \code{altDepth(control)}
  and \code{totalDepth(control)}, respectively.
}
\usage{
annotateWithControlDepth(case, control, control.cov)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{case}{
    The variants of interest, as a \code{VRanges}.
  }
  \item{control}{
    The control variants, typically unfiltered, as a \code{VRanges}.
  }
  \item{control.cov}{
    The control coverage, as an \code{RleList}.
  }
}
\value{
  \code{case}, plus two new metadata columns, \code{control.alt.depth}
  and \code{control.total.depth}
}
\details{
  If a \code{case} variant is not found in \code{control}, a count of
  \code{0} is assigned to \code{control.alt.depth}, under the assumption
  that the \code{control} object is not filtered, i.e., it contains the
  raw tallies.
}
\author{
  Michael Lawrence
}
\seealso{
  \code{\link{callSampleSpecificVariants}}, which uses this function.
}
\examples{
  bams <- LungCancerLines::LungCancerBamFiles()
  data(vignette)
  case <- callVariants(tallies_H1993)
  control <- tallies_H2073
  control.cov <- coverage(bams$H2073)
  annotateWithControlDepth(case, control, control.cov)
}

\name{callWildtype}
\alias{callWildtype}
\alias{minCallableCoverage}
\title{
  Calling Wildtype
}
\description{
  Decides whether a position is variant, wildtype, or uncallable,
  according to the estimated power of the given calling filters.
}
\usage{
callWildtype(reads, variants, calling.filters, pos = NULL, ...)
minCallableCoverage(calling.filters, power = 0.80, max.coverage = 1000L)
}
\arguments{
  \item{reads}{
    The read alignments, i.e., a path to a BAM file, or the coverage,
    including a \code{BigWigFile} object.
  }
  \item{variants}{
    The called variants, a tally \code{GRanges}.
  }
  \item{calling.filters}{
    Filters used to call the variants.
  }
  \item{pos}{
    A \code{GRanges} indicating positions to query; output is in the
    same order. If this is \code{NULL}, the entire genome is
    considered. This is not called \code{which}, because we are
    indicating positions, not selecting from regions.
  }
  \item{power}{
    The chance of detecting a variant if one is there.
  }
  \item{max.coverage}{
    The max coverage to be considered for the minimum (should not need
    to be tweaked).
  }
  \item{\dots}{
    Arguments to pass down to \code{minCallableCoverage}.
  }
}
\details{
  For each position (in the genome, or as specified by \code{pos}), the
  coverage is compared against the return value of
  \code{minCallableCoverage}. If the coverage is above the callable
  minimum, the position is called, either as a variant (if it is in
  \code{variants}) or wildtype. Otherwise, it is considered a no-call.

  The \code{minCallableCoverage} function
  expects and only considers the filters returned by
  \code{\link{VariantCallingFilters}}.
}
\value{
  A logical vector (or logical \code{RleList} if \code{pos} is
  \code{NULL}), that is \code{TRUE} for wildtype, \code{FALSE} for variant,
  \code{NA} for no-call.
}
\author{
  Michael Lawrence
}
\examples{
bams <- LungCancerLines::LungCancerBamFiles()
bam <- bams$H1993

data(vignette)
called.variants <- callVariants(tallies_H1993)

pos <- c(called.variants, shift(called.variants, 3))
wildtype <- callWildtype(bam, called.variants, VariantCallingFilters(), 
                         pos = pos, power = 0.85)
}

\name{annotateTrans}
\alias{annotateTrans}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  %%  ~~function to do ... ~~
  A function to annotate a transcript grl with variant information. 
}
\description{
  %%  ~~ A concise (1-5 lines) description of what the function does. ~~
  Given a txbd, coverage RLE and set of functional variants, this function
  will return a GRangesList of transcripts annotated with additional
  metadata which includes the fraction of the transcript that is callable
  defined by the function isCallable and the number of functional variants
  that fall in the transcript.
}
\usage{
annotateTrans(txdb, cov, anno_gr)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{txdb}{
  %%     ~~Describe \code{txdb} here~~
  a txdb object
}
\item{cov}{
  %%     ~~Describe \code{cov} here~~
  A coverage RLE as generated by the getCov function
}
\item{anno_gr}{
  %%     ~~Describe \code{anno_gr} here~~
  a GRanges object with variants annotaed for transcript occurance and
  consequence. The transcipt IDs are assumed to be ref_seq IDs. 
}
\item{cores}{
  %%     ~~Describe \code{cores} here~~
  Number of cores to be used in the parallel aspects of the
  code. Setting cores to 1 will run on a single core.
}
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
Returns a GRangesList of the transcripts with metadata columns
added for the fraction of the cds or exon region that is considered
callable and the number of protein altering mutations found in the total
cds regions.
}
\author{
%%  ~~who you are~~
Jeremiah Degenhardt
}

\keyword{internal}

\name{extractCoverageForPositions}
\alias{extractCoverageForPositions}
\title{
  Get Coverage at Positions
}
\description{
  Gets values from an \code{RleList} corresponding to positions (width 1
  ranges) in a \code{GRanges} (or \code{VRanges}). The result is a
  simple atomic vector.
}
\usage{
extractCoverageForPositions(cov, pos)
}
\arguments{
  \item{cov}{
    An \code{RleList} like that returned by \code{\link[IRanges]{coverage}}.
  }
  \item{pos}{
    A \code{GRanges} consisting only of width-1 ranges.
  }
}
\value{
  Atomic vector with one value from \code{cov} per position in \code{pos}.
}
\author{
  Michael Lawrence
}

\name{checkVariantConcordance}
\alias{checkVariantConcordance}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
Function to check the variant call concordance between two samples or
lane. 
}
\description{
This function takes two variant call GRanges objects (as produced by
callVariants) and will check the concordance of the variant calls. Uses
a subset of the variants with the best coverage.
}

\usage{
checkVariantConcordance(gr1, gr2, SNP_gr)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gr1}{
    %%     ~~Describe \code{gr1} here~~
    Variant GRanges object from sample/lane 1 (as produced by callVariants)
  }
  \item{gr2}{
    %%     ~~Describe \code{gr2} here~~
    Variant GRanges object from sample/lane 2 (as produced by callVariants)
  }
  \item{SNP_gr}{
    a GRanges object of the SNPs you would like to constrain the check
  by
  }
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
A numeric vector with the concordance and the number of positions checked
}
\author{
  %%  ~~who you are~~
  Jeremiah Degenhardt
}
\keyword{internal}
% \examples{
% }

\name{pileupVariants}
\alias{pileupVariants}
\title{
  Nucleotide pileup from alignments
}
\description{
  This is an alternative to \code{\link{tallyVariants}} for generating a
  \code{VRanges} from a set of alignments (BAM file) by counting the
  nucleotides at each position. This function uses the samtools-based
  \code{\link[Rsamtools]{applyPileups}} function, instead of
  \code{\link[gmapR]{bam_tally}}. Fewer dependencies, with fewer
  statistics (none beyond the fixed columns) available in the output.
}
\usage{
pileupVariants(bams, genome, param = ApplyPileupsParam(), minAltDepth = 1L,
               baseOnly = TRUE, BPPARAM = defaultBPPARAM())
}
\arguments{
  \item{bams}{
    A vector/list of BAM files as interpreted
    by \code{\link[Rsamtools]{PileupFiles}}.
  }
  \item{genome}{
    An object that provides sequence information via
    \code{\link[Biostrings]{getSeq}}.
  }
  \item{param}{
    A \code{\link[Rsamtools]{ApplyPileupsParam}} object that specifies the
    mode of iteration and various filters.
  }
  \item{minAltDepth}{
    Minimal alt depth to be included in the output. The default avoids
    outputting results for positions/alleles that show no differences.
  }
  \item{baseOnly}{
    Whether to drop records with \dQuote{N} in either the ref or alt.
  }
  \item{BPPARAM}{
    Not yet supported.
  }
}
\value{
  A VRanges object with read depth information for each position,
  allele, and sample.
}
\author{
  Michael Lawrence
}
\seealso{
  \code{\link{tallyVariants}} for more statistics.
}
\examples{
bams <- LungCancerLines::LungCancerBamFiles()
if (requireNamespace("gmapR")) {
    param <- Rsamtools::ApplyPileupsParam(which=gmapR::TP53Which())
    pileup <- pileupVariants(bams, gmapR::TP53Genome(), param)
}
}

\name{tallyVariants}
\alias{tallyVariants}
\alias{tallyVariants,character-method}
\alias{tallyVariants,BamFile-method}
\alias{tallyVariants,BamFileList-method}
\alias{VariantTallyParam}
\alias{TallyVariantsParam}

\title{
  Tally the positions in a BAM file
}
\description{
  Tallies the bases, qualities and read positions for every genomic
  position in a BAM file. By default, this only returns the positions
  for which an alternate base has been detected.  The typical usage is
  to pass a BAM file, the genome, the (fixed) \code{readlen} and (if the
  variant calling should consider quality) an appropriate
  \code{high_base_quality} cutoff.

  Passing a \code{which} argument allows computing on only a
  subregion of the genome. \code{which} is a ‘RangesList’ or
  something coercible to one that limits the tally to that range or
  set of ranges. By default, the entire genome is processed.

  For parallel evaluation (see \code{BPPARAM}): Specifically,
  \code{which} can be a ‘GenomicRanges’ or a ‘GRangesList’. If
  \code{which} is a ‘GenomicRanges’ and has length 1 it is tiled
  to create chunks for parallel evaluation. If it is longer
  than 1, each range becomes a chunk for parallel evaluation.
  If \code{which} is a ‘GRangesList’, each element (i.e. each
  ‘GenomicRanges’) becomes a chunk. The latter can be useful to
  ensure balanced worker load, e.g. in the case of regions covering
  multiple sequences(see \code{\link{equisplit}}).
}
\note{
  The \code{VariantTallyParam} constructor is \strong{DEPRECATED}.
}
\usage{
\S4method{tallyVariants}{BamFile}(x, param = TallyVariantsParam(...), ...,
                                  BPPARAM = defaultBPPARAM())
\S4method{tallyVariants}{BamFileList}(x, ...)
\S4method{tallyVariants}{character}(x, ...)
TallyVariantsParam(genome,
                   read_pos_breaks = NULL,
                   high_base_quality = 0L,
                   minimum_mapq = 13L,
                   variant_strand = 1L, ignore_query_Ns = TRUE,
                   ignore_duplicates = TRUE,
                   mask = GRanges(), keep_extra_stats = TRUE,
                   read_length = NA_integer_,
                   read_pos = !is.null(read_pos_breaks),
                   high_nm_score = NA_integer_,
                   ...)
}
\arguments{
  \item{x}{
    An indexed BAM file, either a path, \code{BamFile} or
    \code{BamFileList} object. If the latter, the tallies are computed
    separately for each file, and the results are stacked with
    \code{\link[VariantAnnotation]{stackSamples}} into a single
    \code{VRanges}.
  }
  \item{param}{
    The parameters for the tallying process, as a
    \code{\link[gmapR]{BamTallyParam}}, typically constructed with
    \code{TallyVariantsParam}, see arguments below.
  }
  \item{\dots}{
    For \code{tallyVariants}, arguments to pass to
    \code{TallyVariantsParam}, listed below. For
    \code{TallyVariantsParam}, arguments to pass to
    \code{\link[gmapR]{BamTallyParam}}.
  }
  \item{genome}{The genome, either a \code{\link[gmapR]{GmapGenome}} or
    something coercible to one.
  }
  \item{read_pos_breaks}{The breaks used for tabulating the read positions (read
    positions) at each position. If this information is included (not
    \code{NULL}), \code{\link{qaVariants}} will use it during filtering.
  }
  \item{high_base_quality}{The minimum cutoff for whether a base is
    counted as high quality. By default, \code{\link{callVariants}} will
    use the high quality counts in the likelihood ratio test. Note that
    \code{bam_tally} will shift your quality scores by 33 no matter what
    type they are. If Illumina (pre 1.8) this will result in a range of
    31-71. If Sanger/Illumina1.8 this will result in a range of
    0-40/41. The default counts all bases as high quality. We typically
    use 56 for old Illumina, 23 for Sanger/Illumina1.8.
  }
  \item{minimum_mapq}{Minimum MAPQ of a read for it to be included in
    the tallies. This depend on the aligner; the default is reasonable
    for \code{\link[gmapR]{gsnap}}.
  }
  \item{variant_strand}{On how many strands must an alternate base
    be detected for a position to be returned. Highly recommended to set
    this to at least 1 (otherwise, the result is huge and includes
    many uninteresting reference rows).
  }
  \item{ignore_query_Ns}{Whether to ignore N calls in the
    reads. Usually, there is no reason to set this to \code{FALSE}. If
    it is \code{FALSE}, beware of low quality datasets returning
    enormous results.
  }
  \item{ignore_duplicates}{whether to ignore reads flagged as
    PCR/optical duplicates}
  \item{mask}{A \code{GRanges} specifyin a mask; all variants falling
    within the mask are discarded.
  }
  \item{read_length}{The expected read length, used for calculating the
    \dQuote{median distance from nearest} end statistic. If not
    specified, an attempt is made to guess the read length from a random
    sample of the BAM file. If read length is found to be variable,
    statistics depending on the read length are not calculated.
  }
  \item{read_pos}{Whether to tally read positions, which can be
    computationally intensive.
  }
  \item{high_nm_score}{If not \code{NA},
    counts of reads with NM (mismatch count)
    score equal to or greater are returned in the
    \code{count.high.nm} and \code{count.high.nm.ref} columns.
  }
  \item{keep_extra_stats}{Whether to keep various summary statistics
    generated from the tallies; setting this to FALSE will save memory.
    The extra statistics are most useful for algorithm diagnostics and
    development. 
  }
  \item{BPPARAM}{A
    \code{\link[BiocParallel:BiocParallelParam-class]{BiocParallelParam}}
    object specifying the resources and strategy for parallelizing the
    tally operation over the chromosomes.
  }
}
\value{
  For \code{tallyVariants}, the tally \code{GRanges}.

  For \code{TallyVariantsParam}, an object with parameters suitable for
  variant calling.
}
\author{
  Michael Lawrence, Jeremiah Degenhardt
}

\examples{
if (requireNamespace("gmapR")) {
    tally.param <- TallyVariantsParam(gmapR::TP53Genome(), 
                                      high_base_quality = 23L,
                                      which = gmapR::TP53Which())
    bams <- LungCancerLines::LungCancerBamFiles()
    raw.variants <- tallyVariants(bams$H1993, tally.param)
}
}

\name{VEP2gr}
\alias{VEP2gr}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
Convert a Variant Effect Predictor tab separated file to a GRanges
object. 
}
\description{
%%  ~~ A concise (1-5 lines) description of what the function does. ~~
This simple function will convert a data.frame generated from
reading in the results of running Variant Effect Predictor into
a GRanges object with some of the associated information attached as
ElementMetaData columns 
}
\usage{
VEP2gr(VEP_tab)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{VEP_tab}{
%%     ~~Describe \code{VEP_tab} here~~
A data,frame object with appropriate column names generated by reading
in the output from Variant Effect Predictor
}
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
A GRanges object for each variant listed in the output of Variant Effect
Predictor. The Granges will have additional ElementMetaData columns
Allele (the variant allele), Gene (the gene ID given by VEP), TXID
(transcript ID filled in from the 'Feature' column in VEP),
Feature_type, Consequence and Existing_variation (giving rsID or cosmic
ID if they exist).
}

\author{
%%  ~~who you are~~
Jeremiah Degenhardt
}

\keyword{internal}

\name{FilterConstructors}
\alias{SetdiffVariantsFilter}
\alias{MinTotalDepthFilter}
\alias{MaxControlFreqFilter}
\alias{DepthFETFilter}

\title{Variant Filter Constructors}

\description{
  These functions construct filters (implemented as functions) suitable
  for collection into \code{FilterRules} objects, which are then used to
  filter variant calls. See examples.
}

\usage{
SetdiffVariantsFilter(other)
MinTotalDepthFilter(min.depth = 10L)
MaxControlFreqFilter(control, control.cov, max.control.freq = 0.03)
DepthFETFilter(control, control.cov, p.value.cutoff = 0.05)
}

\arguments{
  \item{other}{The set of variants (as a \code{VRanges}) to
    subtract from the set being filtered.
  }
  \item{min.depth}{The minimum depth for a variant to pass.}
  \item{control}{The control set of variants (as a \code{VRanges}) to
    use when filtering for case-specific variants.
  }
  \item{control.cov}{The coverage (as an \code{RleList}) for the sample
    corresponding to the calls in \code{control}.
  }
  \item{max.control.freq}{The maximum alt frequency allowed in the
    control for a variant to be considered case-specific.
  }
  \item{p.value.cutoff}{Passing variants must have a p-value below this
    value.}
}

\value{
  In all cases, a closure that returns a logical vector indicating which
  elements of its argument should be retained.
}

\author{
  Michael Lawrence
}

\seealso{
  There are some convenience functions that construct \code{FilterRules}
  objects that contain one or more of these filters. Examples are
  \code{\link{VariantQAFilters}} and
  \code{\link{VariantCallingFilters}}.
}

\examples{
  ## Find case-specific variants in a case/control study
  bams <- LungCancerLines::LungCancerBamFiles()

  data(vignette)
  case <- callVariants(tallies_H1993)
  control <- callVariants(tallies_H2073)
  
  control.cov <- coverage(bams$H2073)

  filters <-
    FilterRules(list(caseOnly = SetdiffVariantsFilter(control),
                       minTotalDepth = MinTotalDepthFilter(min.depth=10L),
                       maxControlFreq = MaxControlFreqFilter(control,
                           control.cov, max.control.freq=0.03),
                       depthFET = DepthFETFilter(control, control.cov,
                           p.value.cutoff=0.05)
                       ))
  
  specific <- subsetByFilter(case, filters)
}

\name{callWildtype}
\alias{callWildtype}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Call wildtype}
\description{
  A function to generate a logical RLE of homozygous ref (TRUE), variant
  (FALSE) and no call (NA) for a sample across the complete genome.
}
\usage{
callWildtype(cov, var_gr, power = 0.999, plower = 0.2, perror = 1/1000)
}
\arguments{
  \item{cov}{
    RLE vector of the coverage 
  }
  \item{var_gr}{
    Called variants to be used in generating the logical RLE
  }
  \item{power}{
    The minimum power need to be able to call a wildtype position
  }
  \item{plower}{
    The prefered alternate frequency to test against
  }
  \item{perror}{
    The assumed sequencing error rate
  }
}
\value{
  A logical RLE of called positions, TRUE = wildtype, FALSE = variant, and
  no-call positions (NA).
}
\author{
  Michael Lawrence, Jeremiah Degenhardt
}

\name{getChrGRangesFromGmap}
\alias{getChrGRangesFromGmap}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
A function to retreive a GRanges object of the chromosome names, start
and end positions for the genome being used by gSNAP
}
\description{
%%  ~~ A concise (1-5 lines) description of what the function does. ~~
Given the gSNAP genome and genome directory information return a GRanges
of the chromosome information
}
\usage{
getChrGRangesFromGmap(genome, genome_dir)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{genome}{
    %%     ~~Describe \code{genome} here~~
    the name of the gSNAP genome that you want to get chromosome
    information from
  }
  \item{genome_dir}{
    %%     ~~Describe \code{genome_dir} here~~
    The directory containing the gSNAP genome 
  }
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
GRanges object with the chromosome names, start and end position
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
  %%  ~~who you are~~
  Jeremiah Degenhardt
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line

\name{variantGR2Vcf}
\alias{variantGR2Vcf}
\title{
  Create a VCF for some variants
}
\description{
  The \strong{deprecated} way to create a
  \code{\link[VariantAnnotation]{VCF}} object from a variant/tally
  \code{GRanges}. This can then be output to a file using
  \code{\link[VariantAnnotation]{writeVcf}}. The flavor of VCF is
  specific for calling variants, not genotypes; see below.
}
\usage{
variantGR2Vcf(x, sample.id, project = NULL,
              genome = unique(GenomicRanges::genome(x)))
}
\note{
  This function is \strong{DEPRECATED}. The \code{callVariants} function
  now returns a \code{\link[VariantAnnotation]{VRanges}} object that can
  be coerced to a \code{VCF} object via \code{as(x, "VCF")}.
}
\arguments{
  \item{x}{
    The variant/tally \code{GRanges}.
  }
  \item{sample.id}{
    Unique ID for the sample in the VCF.
  }
  \item{project}{
    Description of the project/experiment; will be included in the VCF header.
  }
  \item{genome}{
    \code{GmapGenome} object, or the name of one (in the default genome
    directory). This is used for obtaining the anchor base when
    outputting indels.
  }
}
\details{
  A variant \code{GRanges} has an element for every unique combination
  of position and alternate base. A \code{VCF} object, like the file
  format, has a row for every position, with multiple alternate alleles
  collapsed within the row. This is the fundamental difference between
  the two data structures. We feel that the \code{GRanges} is easier to
  manipulate for filtering tasks, while \code{VCF} is obviously
  necessary for communication with external databases and tools.

  Normally, despite its name, VCF is used for communicating
  \emph{genotype} calls. We are calling \emph{variants}, not genotypes,
  so we have extended the format accordingly.

  Here is the mapping in detail:
  \itemize{
    \item{The \code{rowRanges} is formed by dropping the metadata columns
      from the \code{GRanges}.
    }
    \item{The \code{colData} consists of a single column,
      \dQuote{Samples}, with a single row, set to 1 and named
      \code{sample.id}.
    }
    \item{The \code{exptData} has an element \dQuote{header} with element
      \dQuote{reference} set to the \code{seqlevels(x)} and element
      \dQuote{samples} set to \code{sample.id}. This will also include the
      necessary metadata for describing our extensions to the format.
    }
    \item{The \code{fixed} table has the \dQuote{REF} and \dQuote{ALT}
      alleles, with \dQuote{QUAL} and \dQuote{FILTER} set to \code{NA}.
    }
    \item{The \code{geno} list has six matrix elements, all with a
      single column. The first is the mandatory \dQuote{GT} element, the
      genotype, which we set to \code{NA}. Then there is \dQuote{AD}
      (list matrix with the read count for each REF and ALT),
      \dQuote{DP} (integer matrix with the total read count), and
      \dQuote{AP} (list matrix of 0/1 flags for whether whether REF
      and/or ALT was present in the data).
    }
  }
}
\value{
  A \code{VCF} object.
}
\author{
  Michael Lawrence, Jeremiah Degenhardt
}
\examples{
\dontrun{
vcf <- variantGR2Vcf(variants, "H1993", "example")
writeVcf(vcf, "H1993.vcf", index = TRUE)
}
}

\name{callableFraction}
\alias{callCallable}
\alias{callableFraction}
\title{
  Callable fractions of transcripts 
}
\description{
  Given a set of alignments and a set of genomic features,
  \code{callableFraction} will return, for each feature, the callable
  fraction of its width. This is according to the likelihood ratio test,
  with power \code{power}. The helper function \code{callCallable}
  returns a logical \code{RleList} indicating the callable positions.
}
\usage{
callCallable(reads, plower = 0.2, perror = 1/1000, power = 0.999)
callableFraction(tx, reads, ...)
}
\arguments{
  \item{tx}{
    A \code{GRangesList} object, like one returned
    by \code{\link[GenomicFeatures]{exonsBy}} or
    \code{\link[GenomicFeatures]{cdsBy}}.
  }
  \item{reads}{
    A BAM file or \code{GappedAlignment(Pair)s} object
  }
  \item{plower}{
    The lower bound on the alternative frequency of interest.
    Should have the same value as used in the \code{\link{callVariants}}
    function.
  }
  \item{perror}{
    Assumed error rate from sequencing, should match that used by
    \code{\link{callVariants}}.
  }
  \item{prob}{
    The prefered power level for the homozygous reference
    detection. This parameter will affect the global false positive rate
    of hom-ref calls.
  }
  \item{\dots}{Arguments passed down to \code{callCallable}}
}
\value{
  Returns a named numeric vector the length of the transcripts in the txdb
  with the fraction of the transcript that has coverage above the
  callable threshold.
}

\author{
  Michael Lawrence, Jeremiah Degenhardt
}

\keyword{internal}

\name{qaVariants}
\alias{qaVariants}
\alias{VariantQAFilters}

\title{
  QA Filtering of Variants
}
\description{
  Filters a tally \code{GRanges} through a series of simple checks for
  strand and read position (read position) biases.
}
\usage{
qaVariants(x, qa.filters = VariantQAFilters(...), ...)
VariantQAFilters(fisher.strand.p.value = 1e-4, min.mdfne = 10L)
}

\arguments{
  \item{x}{
    A tally \code{GRanges} as output by \code{\link{tallyVariants}}.
  }
  \item{qa.filters}{
    The filters used for the QA process, typically constructed with
    \code{VariantQAFilters}, see arguments below.
  }
  \item{\dots}{
    Arguments passed to \code{VariantQAFilters}, listed below.
  }
  \item{fisher.strand.p.value}{p-value cutoff for the Fisher's Exact Test for
    strand bias (+/- counts, alt vs. ref). Any variants with p-values
    below this cutoff are discarded.
  }
  \item{min.mdfne}{Minimum allowed median distance of alt calls from
    their nearest end of the read.
  }
}
\details{
  There are currently two QA filters:
  \itemize{
    \item{Median distance of alt calls from nearest end of the read is
      required to be >= \code{min.mdfne}, which defaults to 10.
    }
    \item{Fisher's Exact Test for strand bias, using the +/- counts, alt
      vs. ref. If the null is rejected, the variant is discarded.
    }
  }
}
\value{
  For \code{qaVariants}, a tally \code{GRanges} of the variants that
  pass the QA checks.

  For \code{VariantQAFilters}, a \code{\link[S4Vectors]{FilterRules}}
  object with the QA and sanity filters.
}

\author{
  Michael Lawrence and Jeremiah Degenhardt
}

\examples{
data(vignette)
qaVariants(tallies_H1993, fisher.strand.p.value = 1e-4)
}

\name{callVariants}
\alias{callVariants}
\alias{callVariants,BamFile-method}
\alias{callVariants,character-method}
\alias{callVariants,GenomicRanges-method}
\alias{callVariants,VRanges-method}
\alias{VariantCallingFilters}

\title{
  Call Variants
}
\description{
  Calls variants from either a BAM file or a \code{VRanges}
  object. The variants are called using a binomial likelihood
  ratio test. Those calls are then subjected to a post-filtering step.
}
\usage{
\S4method{callVariants}{BamFile}(x, tally.param,
                                 calling.filters = VariantCallingFilters(...),
                                 post.filters = FilterRules(),
                                 ...)
\S4method{callVariants}{character}(x, ...)
\S4method{callVariants}{VRanges}(x,
     calling.filters = VariantCallingFilters(...),
     post.filters = FilterRules(),
     ...)
VariantCallingFilters(read.count = 2L, p.lower = 0.2, p.error = 1/1000)
}

\arguments{
  \item{x}{
    Either a path to an indexed bam, a \code{BamFile}
    object, or a \code{VRanges} as returned by \code{\link{tallyVariants}}. 
  }
  \item{tally.param}{
    Parameters controlling the variant tallying step,
    as typically constructed by \code{\link{TallyVariantsParam}}.
  }
  \item{calling.filters}{
    Filters used in the calling step, typically constructed with
    \code{VariantCallingFilters}, see arguments listed below.
  }
  \item{post.filters}{
    Filters that are applied after the initial calling step. These
    consider the set of variant calls as a whole and remove those with
    suspicious patterns.
  }
  \code{...}{Arguments for \code{VariantCallingFilters}, listed below.}
  \item{read.count}{
    Require at least this many high quality reads with
    the alternate base. The default value is designed to catch sequencing
    errors where coverage is too low to rely on the LRT. Increasing this
    value has a significant negative impact on power.
  }
  \item{p.lower}{The lower bound on the binomial probability for a true
    variant.
  }
  \item{p.error}{The binomial probability for a sequencing error (default
    is reasonable for Illumina data with the default quality cutoff).
  }
  \item{\dots}{
    Arguments to pass to \code{VariantCallingFilters}.
  }
}
\details{
  There are two steps for calling
  variants: the actual statistical test that decides whether a variant
  exists in the data, and a post-filtering step. By default, the initial
  calling is based on a binomial likelihood ratio test
  (P(D|p=\code{p.lower}) / P(D|p=\code{p.error}) > 1). The test amounts
  to excluding putative variants with less than ~4\% alt frequency. A
  variant is also required to be represented by at least 2 alt
  reads. The post-filtering stage considers the set of variant calls as
  a whole and removes variants with suspicious patterns. Currently,
  there is a single post-filter, disabled by default, that removes
  variants that are clumped together on the chromosome (see the
  \code{max.nbor.count} parameter).
}
\value{
  For \code{callVariants}, a \code{VRanges} of the called variants (the
  tallies that pass the calling filters). See the documentation
  of \code{\link[gmapR]{bam_tally}} for complete details.
  
  For \code{VariantCallingFilters}, a \code{\link[S4Vectors]{FilterRules}}
  object with the filters for calling the variants.
}
\author{
  Michael Lawrence, Jeremiah Degenhardt
}
\examples{
bams <- LungCancerLines::LungCancerBamFiles()
if (requireNamespace("gmapR")) {
    tally.param <- TallyVariantsParam(gmapR::TP53Genome(), 
                                      high_base_quality = 23L,
                                      which = gmapR::TP53Which())

    ## simple usage
    variants <- callVariants(bams$H1993, tally.param)
}
## customize
data(vignette)
calling.filters <- VariantCallingFilters(p.error = 1/1000)
callVariants(tallies_H1993, calling.filters)
}

\name{plotTiTv}
\alias{plotTiTv}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  A function to make transition/transversion mosaic plots from variant calls
}
\description{
  This function will take a variant call Granges and produce a Ti/Tv
  mosaic plot
}
\usage{
plotTiTv(gr, main = "TiTv")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gr}{
    Variant call Granges as produced by callVariantsP, or tumorNormalCompare
  }
  \item{main}{
    main title passed down to plot
    }
}
\value{
  Generates a plot of the data
}
\author{
  Jeremiah Degenhardt
}

\keyword{internal}

\name{callSampleSpecificVariants}
\alias{callSampleSpecificVariants}
\alias{callSampleSpecificVariants,BamFile,BamFile-method}
\alias{callSampleSpecificVariants,character,character-method}
\alias{callSampleSpecificVariants,GenomicRanges,GenomicRanges-method}
\alias{callSampleSpecificVariants,VRanges,VRanges-method}
\alias{SampleSpecificVariantFilters}

\title{Call Sample-Specific Variants}
\description{
  Calls sample-specific variants by comparing case and control variants
  from paired samples, starting from the BAM files or unfiltered
  tallies. For example, these variants would be considered somatic
  mutations in a tumor vs. normal comparison.
}
\usage{
SampleSpecificVariantFilters(control, control.cov, calling.filters,
                             power = 0.8, p.value = 0.01)
\S4method{callSampleSpecificVariants}{BamFile,BamFile}(case, control,
  tally.param, calling.filters = VariantCallingFilters(), post.filters =
  FilterRules(), ...)
\S4method{callSampleSpecificVariants}{character,character}(case, control, ...)
\S4method{callSampleSpecificVariants}{VRanges,VRanges}(case,
  control, control.cov, ...)
## DEPRECATED
\S4method{callSampleSpecificVariants}{GenomicRanges,GenomicRanges}(case,
  control, control.cov,
  calling.filters = VariantCallingFilters(), post.filters =
  FilterRules(), ...)
}
\arguments{
  \item{case}{
    The BAM file for the case, or the called variants as output by
    \code{\link{callVariants}}.
  }
  \item{control}{
    The BAM file for the control, or the raw tallies as output by
    \code{\link{tallyVariants}}.
  }
  \item{tally.param}{Parameters controlling the variant tallying step,
    as typically constructed by \code{\link{TallyVariantsParam}}.}
  \item{calling.filters}{Filters to use for the initial,
    single-sample calling against reference, typically constructed by
    \code{\link{VariantCallingFilters}}.
  }
  \item{post.filters}{
    Filters that are applied after the initial calling step. These
    consider the set of variant calls as a whole and remove those with
    suspicious patterns. They are only applied to the \code{case}
    sample; only QA filters are applied to \code{control}.
  }
  \item{\dots}{
    For a BAM file, arguments to pass down to the \code{GenomicRanges}
    method. For the \code{GenomicRanges} method, arguments to pass down
    to \code{SampleSpecificVariantFilters}, except for
    \code{control.cov}, \code{control.called}, \code{control.raw} and
    \code{lr.filter}.
  }
  \item{control.cov}{The coverage for the control sample.}
  \item{power}{The power cutoff, beneath which a variant will not be
    called case-specific, due to lack of power in control.
  }
  \item{p.value}{The binomial p-value cutoff for determining whether the
    control frequency is sufficiently extreme (low) compared to the case
    frequency. A p-value below this cutoff means that the variant will
    be called case-specific.
  }
}
\details{
  For each sample, the variants are tallied (when the input is BAM), QA
  filtered (case only), called and determined to be sample-specific.
  The \code{callSampleSpecificVariants} function is fairly high-level,
  but it still allows the user to override the parameters and filters
  for each stage of the process. See \code{\link{TallyVariantsParam}},
  \code{\link{VariantQAFilters}}, \code{\link{VariantCallingFilters}}
  and \code{SampleSpecificVariantFilters}.

  It is safest to pass a BAM file, so that the computations are
  consistent for both samples. The \code{GenomicRanges} method is
  provided mostly for optimization purposes, since tallying the variants
  over the entire genome is time-consuming. For small gene-size regions,
  performance should not be a concern.
  
  This is the algorithm that determines whether a variant is specific to
  the case sample:
  \enumerate{
    \item{Filter out all case calls that were also called in
      control. The \code{callSampleSpecificVariants} function does
      \strong{not} apply the QA filters when calling variants in
      control. This prevents a variant from being called specific to
      case merely due to questionable data in the control.
    }
    \item{For the remaining case calls, calculate whether there was
      sufficient power in control under the likelihood ratio test, for a
      variant present at the \code{p.lower} frequency. If that is below
      the \code{power} cutoff, discard it.
    }
    \item{For the remaining case calls, test whether the control
      frequency is sufficient extreme (low) compared to the case
      frequency, under the binomial model. The null hypothesis is that
      the frequencies are the same, so if the test p-value is above
      \code{p.value}, discard the variant. Otherwise, the variant is
      called case-specific.
    }
  }
}
\value{
  A \code{VRanges} with the case-specific variants (such as
  somatic mutations).
}
\author{
  Michael Lawrence, Jeremiah Degenhardt
}
\examples{
bams <- LungCancerLines::LungCancerBamFiles()
if (requireNamespace("gmapR", quietly=TRUE)) {
    tally.param <- TallyVariantsParam(gmapR::TP53Genome(), 
                                      high_base_quality = 23L,
                                      which = gmapR::TP53Which())
    callSampleSpecificVariants(bams$H1993, bams$H2073, tally.param)
} else {
    data(vignette)
    calling.filters <- VariantCallingFilters(read.count = 3L)
    called.variants <- callVariants(tallies_H1993, calling.filters)
    callSampleSpecificVariants(called.variants, tallies_H2073,
                               coverage_H2073)
}
}

\name{vignette}
\alias{tallies_H1993}
\alias{tallies_H2073}
\alias{coverage_H1993}
\alias{coverage_H2073}
\alias{genome_p53}
\alias{p53}

\docType{data}
\title{Vignette Data}
\description{
  Precomputed data for use in the vignette, mostly for the sake of
  Windows, where gmapR and its tallying functionality are unsupported.
}
\usage{data(vignette)}
\format{
  The following objects are included:
  \describe{
    \item{tallies_H1993, tallies_H2073}{Tallies for the two samples.}
    \item{coverage_H1993, coverage_H2073}{Coverage for the two samples.}
    \item{p53}{A GRanges of the p53 exons}
    \item{genome_p53}{DNAStringSet with the genome sequence of the p53
      region}
  }
}
\details{
  The following demonstrates how we created these objects:
  \preformatted{
    bams <- LungCancerLines::LungCancerBamFiles()
    tally.param <- TallyVariantsParam(gmapR::TP53Genome(), 
                                      high_base_quality = 23L,
				      which = range(p53) + 5e4,
                                      indels = TRUE, read_length = 75L)
    tallies_H1993 <- tallyVariants(bams$H1993, tally.param)
    tallies_H2073 <- tallyVariants(bams$H2073, tally.param)
    coverage_H1993 <- coverage(bams$H1993)
    coverage_H2073 <- coverage(bams$H2073)
    genome_p53 <- DNAStringSet(getSeq(gmapR::TP53Genome()))
    p53 <- gmapR:::exonsOnTP53Genome("TP53")
  }
}
\source{
  Computed from the data in the LungCancerLines package.
}
\examples{
data(vignette)
}
\keyword{datasets}

\name{TwoSampleFET}
\alias{TwoSampleFET}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
A function to find sample-specific mutations by a two-sample comparison
using the Fisher's exact test and a simple frequency threshold
}
\description{
  %%  ~~ A concise (1-5 lines) description of what the function does. ~~
  This function takes and GRanges of called varaints from sample1 , a GRanges of raw
  variants from sample2, the coverage object from sample2 and two
  parameters and returns a GRanges of the variants specific to sample1 
}
\usage{
TwoSampleFET(gr, raw, cov, pval = 0.001, freq = 0.05)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gr}{
    %%     ~~Describe \code{gr} here~~
    GRanges of called variants from sample1
}
  \item{raw}{
    %%     ~~Describe \code{raw} here~~
    GRnages of raw variants from sample 2
}
  \item{cov}{
    %%     ~~Describe \code{cov} here~~
    RLE vector of coverage from sample2
}
  \item{pval}{
    %%     ~~Describe \code{pval} here~~
    the FET p-value to be used for filtering
}
  \item{freq}{
    %%     ~~Describe \code{freq} here~~
    The minimum frequency threshold needed in sample2 before the variant
    is considered to be seen
}
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
Variant GRanges of the sample-specific variants
}
\author{
%%  ~~who you are~~
Jeremiah Degenhardt
}
\keyword{internal}

\name{callGenotypes}
\alias{callGenotypes}
\alias{callGenotypes,VRanges-method}
\alias{callGenotypes,TabixFile-method}
\alias{CallGenotypesParam}

\title{
  Call Genotypes
}
\description{
  Calls genotypes from a set of tallies (such as a \code{VRanges} or VCF
  file) and the coverage (currently as a \code{BigWigFile}). We call the
  genotype with the highest likelihood, where the likelihood is based on
  a binomial model of the variant frequency.
}
\usage{
\S4method{callGenotypes}{VRanges}(variants, cov,
  param = CallGenotypesParam(variants),
  BPPARAM = defaultBPPARAM())
\S4method{callGenotypes}{TabixFile}(variants, cov,
  param = CallGenotypesParam(variants),
  BPPARAM = defaultBPPARAM())
CallGenotypesParam(genome,
                   gq.breaks = c(0, 5, 20, 60, Inf),
                   p.error = 0.05,
                   which = tileGenome(seqinfo(genome), ntile=ntile),
                   ntile = 100L)
}

\arguments{
  \item{variants}{
    Either \code{VRanges} as returned by \code{\link{tallyVariants}}, or
    a \code{TabixFile} object pointing to a VCF file. Typically, these
    tallies are \emph{not} filtered by e.g. \code{\link{callVariants}},
    because it would seem more appropriate to filter on the genotype
    quality.
  }
  \item{cov}{
    The coverage, as an RleList or a \code{BigWigFile}.
  }
  \item{param}{
    Parameters controlling the genotyping, constructed by
    \code{\link{CallGenotypesParam}}. The default value uses the genome
    from \code{variants}.
  }
  \item{genome}{
    An object with a \code{getSeq} method representing the genomic
    sequence used during tallying.
  }
  \item{gq.breaks}{A numeric vector representing an increasing sequence
    of genotype quality breaks to segment the wildtype runs.
  }
  \item{p.error}{The binomial probability for an error. This is used to
    calculate the expected frequency of hom-ref and hom-alt variants.
  }
  \item{which}{A \code{GenomicRangesList} indicating the genomic regions
    in which to compute the genotypes. The default is to partition the
    \code{genome} into \code{ntile} tiles.
  }
  \item{ntile}{When \code{which} is missing, this indicates the number
    of tiles to generate from the genome.
  }
  \item{BPPARAM}{A \code{BiocParallelParam} object communicating the
    parallelization strategy. One job is created per tile.}
}
\details{
  In general, the behavior is very similar to that of the GATK
  UnifiedGenotyper (see references). For every position in the tallies,
  we compute a binomial likelihood for each of wildtype (0/0), het (0/1)
  and hom-alt (1/1), assuming the alt allele frequency to be
  \code{p.error}, \code{0.5} and \code{1 - p.error}, respectively. The
  genotype with the maximum likelihood is chosen as the genotype, and
  the genotype quality is computed by taking the fraction of the maximum
  likelihood over the sum of the three likelihoods.

  We assume that any position not present in the input tallies is
  wildtype (0/0) and compute the quality for every such position, using
  the provided coverage data. For scalability reasons, we segment runs
  of these positions according to user-specified breaks on the genotype
  quality. The segments become special records in the returned
  \code{VRanges}, where the range represents the segment, the \code{ref}
  is the first reference base, \code{alt} is \code{<NON_REF>} and the
  \code{totalDepth} is the mean of the coverage.

  The genotype information is recorded as metadata columns named
  according to gVCF conventions:
  \describe{
    \item{\code{GT}}{The genotype call string: \code{0/0}, \code{0/1},
      \code{1/1}.
    }
    \item{\code{GQ}}{The numeric genotype quality, phred scaled. For
      wildtype runs, this is minimum over the run.
    }
    \item{\code{PL}}{A 3 column matrix with the likelihood for each
      genotype, phred scaled. We take the minimum over wildtype runs.
    }
    \item{\code{MIN_DP}}{The minimum coverage over a wildtype
      run; \code{NA} for single positions.
    }
  }  
}
\value{
  For \code{callGenotypes}, a \code{VRanges} annotated with the genotype
  call information, as described in the details.
}
\author{
  Michael Lawrence
}
\references{
  The Genome Analysis Toolkit: a MapReduce framework for analyzing
  next-generation DNA sequencing data McKenna A, Hanna M, Banks E,
  Sivachenko A, Cibulskis K, Kernytsky A, Garimella K, Altshuler D,
  Gabriel S, Daly M, DePristo MA, 2010 GENOME RESEARCH 20:1297-303.
}
\examples{
bams <- LungCancerLines::LungCancerBamFiles()
data(vignette)
tallies <- tallies_H1993

sampleNames(tallies) <- "H1993"
mcols(tallies) <- NULL

cov <- coverage(bams$H1993)

## simple usage
## (need gmapR to find the genome in the GMAP database, otherwise,
##  provide sequence directly as shown later)
if (requireNamespace("gmapR", quietly=TRUE)) {
    genotypes <- callGenotypes(tallies, cov,
                               BPPARAM=BiocParallel::SerialParam())
}

## customize
params <- CallGenotypesParam(genome_p53, p.error = 1/1000)
genotypes <- callGenotypes(tallies, cov, params)

## write to gVCF
writeVcf(genotypes, tempfile("genotypes", fileext="vcf"), index=TRUE)
}

\name{postFilterVariants}
\alias{postFilterVariants}
\alias{VariantPostFilters}

\title{
  Post-filtering of Variants
}
\description{
  Applies filters to a set of called variants. The only current filter
  is a cutoff on the weighted neighbor count of each variant. This
  filtering is performed automatically by \code{\link{callVariants}}, so
  these functions are for when more control is desired.
}
\usage{
postFilterVariants(x, post.filters = VariantPostFilters(...), ...)
VariantPostFilters(max.nbor.count = 0.1, whitelist = NULL)
}

\arguments{
  \item{x}{
    A tally \code{GRanges} containing called variants, as output by
    \code{\link{callVariants}}.
  }
  \item{post.filters}{
    The filters applied to the called variants.
  }
  \item{\dots}{
    Arguments passed to \code{VariantPostFilters}, listed below.
  }
  \item{max.nbor.count}{
    Maximum allowed number of neighbors (weighted by distance)
  }
  \item{whitelist}{
    Positions to ignore; these will always pass the filter, and are
    excluded from the neighbor counting.
  }
}
\details{
  The neighbor count is calculated within a 100bp window centered on the
  variant. Each neighbor is weighted by the inverse square root of the
  distance to the neighbor. This was motivated by fitting logistic regression
  models including a term the count (usually 0, 1, 2) at each
  distance. The inverse square root function best matched the trend in
  the coefficients.
}
\value{
  For \code{postFilterVariants}, a tally \code{GRanges} of the variants that
  pass the filters.

  For \code{VariantPostFilters}, a \code{\link[S4Vectors]{FilterRules}}
  object with the filters.
}

\author{
  Michael Lawrence and Jeremiah Degenhardt
}

\examples{
bams <- LungCancerLines::LungCancerBamFiles()
## post-filters are not enabled by default during calling
data(vignette)
called.variants <- callVariants(tallies_H1993)
## but can be applied at a later time...
postFilterVariants(called.variants, max.nbor.count = 0.15)

# or enable during calling
called.variants <- callVariants(tallies_H1993,
                                post.filters = VariantPostFilters())
}

\name{matchVariants}
\alias{matchVariants}
\alias{\%variant_in\%}
\alias{\%variant_in\%,GenomicRanges,GenomicRanges-method}
\title{
  Match variants by position and allele
}
\description{
  These are \strong{deprecated} functions for operating on the old
  variant GRanges. New code should use \code{match} and \code{\%in\%}.
  This function behaves like \code{match}, where two elements match when
  they share the same position and \dQuote{alt} allele.
}
\usage{
matchVariants(x, table)
x \%variant_in\% table
}
\arguments{
  \item{x}{
    The variants (\code{GRanges}) to match into \code{table}; the alt
    allele must be in the \dQuote{alt} metacolumn.
  }
  \item{table}{
    The variants (\code{GRanges}) to be matched into; the alt
    allele must be in the \dQuote{alt} metacolumn.
  }
}
\value{
  For \code{matchVariants}, an integer vector with the matching index in
  \code{table} for each variant in \code{x}, or \code{NA} if there is no
  match. For \code{\%variant_in\%}, a logical vector indicating whether
  there was such a match.
}
\author{
  Michael Lawrence
}

\name{concordance}
\alias{calculateConcordanceMatrix}
\alias{calculateVariantConcordance}
\alias{callVariantConcordance}
\title{
  Variant Concordance
}
\description{
  Functions for calculating concordance between variant sets and
  deciding whether two samples have identical genomes.
}
\usage{
calculateVariantConcordance(gr1, gr2, which = NULL)
calculateConcordanceMatrix(variantFiles, ...)
callVariantConcordance(concordanceMatrix, threshold)
}
\arguments{
  \item{gr1, gr2}{The two tally \code{GRanges} to compare}
  \item{which}{
    A \code{GRanges} of positions to which the comparison is limited.
  }
  \item{variantFiles}{
    Character vector of paths to files representing tally
    \code{GRanges}. Currently supports serialized (rda) and VCF files.
    If the file extension is not \dQuote{vcf}, we assume rda. Will be
    improved in the future.
  }
  \item{concordanceMatrix}{
    A matrix of concordance fractions between sample pairs, as returend by
    \code{calculateConcordanceMatrix}.
  }
  \item{threshold}{
    The concordance fraction above which edges are generated between
    samples when forming the graph.
  }
  \item{\dots}{Arguments to pass to the loading function, e.g., \code{readVcf}.}
}
\details{
  The \code{calculateVariantConcordance} calculates the fraction of
  concordant variants between two samples. Concordance is defined as
  having the same position and alt allele.

  The \code{calculateConcordanceMatrix} function generates a numeric
  matrix with the concordance for each pair of samples. It accepts paths
  to serialized objects so that all variant calls are not loaded in
  memory at once. This probably should support VCF files, eventually.

  The \code{callVariantConcordance} function generates a
  concordant/non-concordant/undecidable status for each sample (that are
  assumed to originate from the same individual), given the output of
  \code{calculateConcordanceMatrix}. The status is decided as follows. A
  graph is formed from the concordance matrix using \code{threshold} to
  generate the edges. If there are multiple cliques in the graph that
  each have more than one sample, every sample is declared
  undecidable. Otherwise, the samples in the clique with more than one
  sample, if any, are marked as concordant, and the others (in singleton
  cliques) are marked as discordant.
}
\value{
  Fraction of concordant variants for \code{calculateVariantConcordance}, a
  numeric matrix of concordances for \code{calculateConcordanceMatrix},
  or a character vector of status codes, named by sample, for
  \code{callVariantConcordance}.
}
\author{
  Cory Barr (code), Michael Lawrence (inferred documentation)
}
